@@ -110,7 +110,7 @@ static void AGUIX_im_dest_callback( XIM callim, XPointer client_data, XPointer c
 
 #endif //USE_XIM
 
-long AGUIX::timerEvent = 0;
+std::atomic< long > AGUIX::timerEvent( 0 );
 std::map<std::string,int> AGUIX::_colors_for_widgets;
 
 static void (*other_sighandler)( int ) = NULL;
