@@ -1,6 +1,6 @@
 /* aguix.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2010,2012-2013 Ralf Hoffmann.
+ * Copyright (C) 2001-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -43,6 +43,7 @@
 
 #include <map>
 #include <list>
+#include <atomic>
 
 #include "aguixfont.hh"
 #include "aguixcolor.hh"
@@ -207,7 +208,7 @@ public:
   int getStrlen4WidthMaxlen( const char *str, int maxlen,
                              int width, int *return_width, AGUIXFont *font = NULL );
 
-  static long timerEvent;
+    static std::atomic< long > timerEvent;
 
   void drawBorder( Drawable buffer, GC usegc, bool pressed, int x, int y, int w, int h, int topright_space );
   void drawBorder( Drawable buffer, bool pressed, int x, int y, int w, int h, int topright_space );
