@@ -1,6 +1,6 @@
 /* awindow.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2009 Ralf Hoffmann.
+ * Copyright (C) 2001-2009,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -32,7 +32,8 @@ AWindow::AWindow( AGUIX *aguix,
 		  int width,
 		  int height,
 		  int bg,
-		  std::string title ) : Widget( aguix )
+		  std::string title,
+                  wm_window_type_t type ) : Widget( aguix )
 {
   setIsTopLevelWidget( true );
 
@@ -57,6 +58,8 @@ AWindow::AWindow( AGUIX *aguix,
   msgW = msgH = -1;
 
   _callback_forbidden = false;
+
+  m_window_type = type;
 }
 
 AGUIX *AWindow::getAGUIX() const
@@ -670,8 +673,8 @@ void AWindow::centerScreen()
 {
   int rw, rh, nx, ny;
 
-  rw = _aguix->getRootWindowWidth();
-  rh = _aguix->getRootWindowHeight();
+  _aguix->getLargestDimensionOfCurrentScreen( NULL, NULL, &rw, &rh );
+
   if ( ( rw < 1 ) || ( rh < 1 ) ) return;
   
   nx = ( rw / 2 ) - ( _w / 2 );
@@ -1010,12 +1013,38 @@ void AWindow::doCreateStuff()
   
   // command so the window manager can restart worker
   XSetCommand( dsp, win, _aguix->getargv(), _aguix->getargc() );
-  
+
+  if ( _aguix->getApplyWindowDialogType() ) {
+      Atom window_type_atom;
+
+      if ( m_window_type == AWINDOW_DIALOG ) {
+          window_type_atom = _aguix->getAtom( AGUIX::NET_WM_WINDOW_TYPE_DIALOG );
+      } else {
+          window_type_atom = _aguix->getAtom( AGUIX::NET_WM_WINDOW_TYPE_NORMAL );
+      }
+
+      XChangeProperty( dsp, win,
+                       _aguix->getAtom( AGUIX::NET_WM_WINDOW_TYPE ),
+                       XA_ATOM, 32, PropModeReplace, (unsigned char *) &window_type_atom, 1 );
+  }
+
+  if ( m_window_type == AWINDOW_DIALOG ) {
+      const AWindow *twin = _aguix->getTransientWindow();
+
+      if ( twin ) {
+          setTransientForAWindow( twin );
+      }
+  }
+
   // clear window
   XClearWindow( dsp, win );
 
   // insert window into _aguix
-  _aguix->insertWindow( this );
+  if ( m_window_type == AWINDOW_DIALOG ) {
+      _aguix->insertWindow( this, false );
+  } else {
+      _aguix->insertWindow( this, true );
+  }
   
 #ifdef USE_XIM
   // create xinput stuff
@@ -1055,21 +1084,6 @@ void AWindow::doDestroyStuff()
   Widget::doDestroyStuff();
 }
 
-Widget *AWindow::add( Widget *w )
-{
-  std::list<Widget*>::iterator it1;
-  
-  if ( w == NULL ) return NULL;
-  
-  it1 = std::find( _childs.begin(), _childs.end(), w );
-  if ( it1 != _childs.end() ) return NULL;
-  
-  if ( w->setParent( this ) != 0 ) return NULL;
-  _childs.push_back( w );
-  if ( _created == true ) w->create();
-  return w;
-}
-
 void AWindow::remove( Widget *w )
 {
   std::list<Widget*>::iterator it1;
