@@ -1,6 +1,6 @@
 /* button.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2005,2010,2012 Ralf Hoffmann.
+ * Copyright (C) 2001-2005,2010,2012-2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -396,7 +396,7 @@ bool Button::handleMessage(XEvent *E,Message *msg)
 		setState( instate );
 		agmsg->button.state = instate;
 	      }
-              msgAndCB( std::auto_ptr<AGMessage>( agmsg ) );
+              msgAndCB( std::unique_ptr<AGMessage>( agmsg ) );
 	      returnvalue=true;
 	    }
 	  }
@@ -406,12 +406,12 @@ bool Button::handleMessage(XEvent *E,Message *msg)
 	    agmsg->type=AG_BUTTONRELEASED;
 	    agmsg->button.button=this;
 	    agmsg->button.state=instate;
-            msgAndCB( std::auto_ptr<AGMessage>( agmsg ) );
+            msgAndCB( std::unique_ptr<AGMessage>( agmsg ) );
 	    agmsg = AGUIX_allocAGMessage();
 	    agmsg->type=AG_BUTTONCLICKED;
 	    agmsg->button.button=this;
 	    agmsg->button.state=instate;
-            msgAndCB( std::auto_ptr<AGMessage>( agmsg ) );
+            msgAndCB( std::unique_ptr<AGMessage>( agmsg ) );
 	  }
 	  if(instate!=0) {
 	    setState(0);
@@ -454,7 +454,7 @@ bool Button::handleMessage(XEvent *E,Message *msg)
 	    agmsg->type=AG_BUTTONCLICKED;
 	    agmsg->button.button=this;
 	    agmsg->button.state=1;
-            msgAndCB( std::auto_ptr<AGMessage>( agmsg ) );
+            msgAndCB( std::unique_ptr<AGMessage>( agmsg ) );
 	    if(instate!=0) {
 	      setState(0);
 	      instate=0;
