@@ -1,6 +1,6 @@
 /* lowlevelfunc.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2006,2010-2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -24,44 +24,15 @@
 #include <algorithm>
 #include "utf8.hh"
 
-#ifdef USE_MEM_SYSTEM
-typedef struct _safemem {void *ptr;
-                         size_t size;
-                         struct _safemem *next;} SafeMem;
-SafeMem *firstmemstr;
-unsigned long maxmemusage,memusage;
-short memsystem_inited = 0;
-void _memsysteminit();
-
-#include "mutex.h"
-MutEx memlock;
-
-#endif
-
 void *_allocsafe(size_t size)
 { /* gibt Speicher zurck oder beendet sofort das Programm */
   void *ptr;
-#ifdef USE_MEM_SYSTEM
-  SafeMem *tsmptr;
-#endif
 
   ptr=(void*)malloc(size);
   if(ptr==NULL) {
     fprintf( stderr, "Worker Error:No Memory!!!!Aborting\n" );
     exit(1);
   }
-#ifdef USE_MEM_SYSTEM
-  memlock.lock();
-  if ( memsystem_inited != 1 ) _memsysteminit();
-  tsmptr=(SafeMem*)malloc(sizeof(SafeMem));
-  tsmptr->ptr=ptr;
-  tsmptr->size=size;
-  tsmptr->next = firstmemstr;
-  firstmemstr=tsmptr;
-  memusage+=size;
-  if(memusage>maxmemusage) maxmemusage=memusage;
-  memlock.unlock();
-#endif
   return ptr;
 }
 
@@ -89,78 +60,6 @@ void waittime(unsigned long msec)
 #endif /* HAVE_NANOSLEEP */
 }
 
-#ifdef USE_MEM_SYSTEM
-void _freesafe(void *p)
-{
-  SafeMem *tsmptr,*found,*prefound;
-
-  memlock.lock();
-  if ( memsystem_inited != 1 ) _memsysteminit();
-  tsmptr=firstmemstr;
-  found=NULL;
-  prefound=NULL;
-  while(tsmptr!=NULL) {
-    if(tsmptr->ptr==p) {
-      found=tsmptr;
-      break;
-    }
-    prefound=tsmptr;
-    tsmptr=tsmptr->next;
-  }
-  if(found!=NULL) {
-    memusage-=found->size;
-    if(firstmemstr==found) {
-      firstmemstr=found->next;
-      free(found);
-    } else {
-      prefound->next=found->next;
-      free(found);
-    }
-  } else {
-    fprintf( stderr, "Freigabe eines unauthorisierten Speichers!\n");
-  }
-  memlock.unlock();
-  free(p);
-}
-
-void _memsystemcheck()
-{
-  SafeMem *tsmptr;
-  int nr;
-
-  if ( memsystem_inited != 1 ) return;
-  memlock.lock();
-  tsmptr=firstmemstr;
-  nr=0;
-  while(tsmptr!=NULL) {
-    nr++;
-/*    printf("lasse nr %d free:\n",nr);
-    free(tsmptr->ptr);*/
-    tsmptr=tsmptr->next;
-  }
-#ifdef DEBUG
-  printf("Es gibt noch %d nicht freigelassene Speicherstcke!\n",nr);
-  printf("Maximal benutzter Speicher:%ld Bytes\n",maxmemusage);
-  printf("Noch benutzter Speicher:%ld Bytes\n",memusage);
-#else
-  if(nr>0) {
-    printf("Es gibt noch %d nicht freigelassene Speicherstcke!\n",nr);
-  }
-#endif
-  memlock.unlock();
-}
-
-void _memsysteminit()
-{
-  printf("Init Mem-System\n");
-  firstmemstr=NULL;
-  maxmemusage=0;
-  memusage=0;
-  memsystem_inited = 1;
-  atexit( _memsystemcheck );
-}
-#endif
-
 char *dupstring(const char *str)
 {
   char *tstr;
