@@ -1,6 +1,6 @@
 /* message.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2007,2010-2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -43,7 +43,7 @@ class TextView;
 typedef struct message
 {
   int type;
-  enum { NONE, TIMEREVENT } specialType;
+  enum { NONE, TIMEREVENT, XIMSTUCK } specialType;
   Widget *lockElement;
   bool ack;
   int loop;
@@ -69,11 +69,21 @@ enum {AG_NONE,AG_KEYPRESSED,AG_KEYRELEASED,AG_MOUSEPRESSED,AG_MOUSERELEASED,AG_M
       AG_SIZECHANGED,
       AG_DND_START,AG_DND_END,AG_DND_CANCEL,
       AG_CYCLEBUTTONCLICKED,
-      AG_FIELDLV_ONESELECT,AG_FIELDLV_MULTISELECT,AG_FIELDLV_PRESSED,AG_FIELDLV_HEADERCLICKED,AG_FIELDLV_DOUBLECLICK,AG_FIELDLV_ENTRY_PRESSED,
+
+      AG_FIELDLV_ONESELECT,
+      AG_FIELDLV_MULTISELECT,
+      AG_FIELDLV_PRESSED,
+      AG_FIELDLV_HEADERCLICKED,
+      AG_FIELDLV_DOUBLECLICK,
+      AG_FIELDLV_ENTRY_PRESSED,
+      AG_FIELDLV_VSCROLL,
+      AG_FIELDLV_HSCROLL,
+
       AG_KARTEIBUTTONCLICKED,
       AG_SLIDER_PRESSED, AG_SLIDER_CHANGED,
       AG_POPUPMENU_CLICKED, AG_POPUPMENU_CLOSED, AG_POPUPMENU_ENTRYEDITED,
-      AG_TEXTVIEW_END_REACHED
+      AG_TEXTVIEW_END_REACHED,
+      AG_CLIENT_MESSAGE
 };
 
 typedef struct agkey {
@@ -233,6 +243,16 @@ typedef struct {
   TextView *textview;
 } AGTextView;
 
+typedef enum AGClientMessageType {
+    CM_GENERIC,
+    CM_XIM_STUCK
+} ag_client_message_t;
+
+typedef struct {
+    int type;
+    ag_client_message_t cm_type;
+} AGClientMessage;
+
 typedef union agmessage {
   int type;
   AGKey key;
@@ -251,6 +271,7 @@ typedef union agmessage {
   AGSlider slider;
   AGPopUpMenu popupmenu;
   AGTextView textview;
+  AGClientMessage clientmessage;
   long direct[20];
 } AGMessage;
 
