@@ -305,14 +305,18 @@ void PopUpMenu::showEntry( int entry )
                     sub_y = it1->y;
                     sub_y += getY();
                     sub_y -= it1->submenu->getBorderWidth();
+
+                    int rw, rh;
+
+                    _aguix->getLargestDimensionOfCurrentScreen( NULL, NULL, &rw, &rh );
                     
-                    if ( sub_y + it1->submenu->getHeight() >= _aguix->getRootWindowHeight() ) {
-                        sub_y = _aguix->getRootWindowHeight() - it1->submenu->getHeight();
+                    if ( sub_y + it1->submenu->getHeight() >= rh ) {
+                        sub_y = rh - it1->submenu->getHeight();
                     }
                     if ( sub_y < 0 ) sub_y = 0;
                     
                     sub_x = getX() + getWidth();
-                    if ( sub_x + it1->submenu->getWidth() >= _aguix->getRootWindowWidth() ) {
+                    if ( sub_x + it1->submenu->getWidth() >= rw ) {
                         sub_x = getX() - it1->submenu->getWidth();
                     }
                     if ( sub_x < 0 ) sub_x = 0;
@@ -637,7 +641,7 @@ void PopUpMenu::sendClickMessage( int entry )
         std::list<int> entries;
         
         entries.push_back( m_entries[entry].id );
-        sendMessage( this, entries, std::auto_ptr<AGMessage>( agmsg ) );
+        sendMessage( this, entries, std::unique_ptr<AGMessage>( agmsg ) );
     }
 }
 
@@ -651,11 +655,11 @@ void PopUpMenu::sendEditedMessage( int entry )
         std::list<int> entries;
         
         entries.push_back( m_entries[entry].id );
-        sendMessage( this, entries, std::auto_ptr<AGMessage>( agmsg ) );
+        sendMessage( this, entries, std::unique_ptr<AGMessage>( agmsg ) );
     }
 }
 
-void PopUpMenu::sendMessage( const PopUpMenu *submenu, const std::list<int> &entries, std::auto_ptr<AGMessage> agmsg )
+void PopUpMenu::sendMessage( const PopUpMenu *submenu, const std::list<int> &entries, std::unique_ptr<AGMessage> agmsg )
 {
     std::list<int> new_entries( entries );
     
@@ -671,7 +675,7 @@ void PopUpMenu::sendMessage( const PopUpMenu *submenu, const std::list<int> &ent
     }
     
     if ( m_parent_menu != NULL ) {
-        m_parent_menu->sendMessage( this, new_entries, agmsg );
+        m_parent_menu->sendMessage( this, new_entries, std::move( agmsg ) );
     } else {
         agmsg->popupmenu.menu = this;
         agmsg->popupmenu.recursive_ids = new std::list<int>( new_entries );
