@@ -1,6 +1,6 @@
 /* request.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2006,2009 Ralf Hoffmann.
+ * Copyright (C) 2001-2006,2009,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -73,7 +73,7 @@ int Requester::request(const char *title,const char *text,const char *buttons, c
     twin = aguix->getTransientWindow();
   }
   
-  AWindow *win = new AWindow( aguix, 10, 10, 10, 10, 0, title );
+  AWindow *win = new AWindow( aguix, 10, 10, 10, 10, 0, title, AWindow::AWINDOW_DIALOG );
   win->create();
   if ( twin != NULL ) {
     win->setTransientForAWindow( twin );
@@ -106,12 +106,12 @@ int Requester::request(const char *title,const char *text,const char *buttons, c
   buts = (Button**)_allocsafe( nbuttons * sizeof( Button* ) );
 
   if ( nbuttons == 1 ) {
-    buts[0] = (Button*)ac1_buttons->add( new Button( aguix, 0, 0, buttonss[0], 1, 0, 0 ),
-                                         1, 0, AContainer::CO_FIX );
+    buts[0] = ac1_buttons->addWidget( new Button( aguix, 0, 0, buttonss[0], 1, 0, 0 ),
+                                      1, 0, AContainer::CO_FIX );
   } else {
     for ( i = 0; i < nbuttons; i++ ) {
-      buts[i] = (Button*)ac1_buttons->add( new Button( aguix, 0, 0, buttonss[i], 1, 0, i ),
-                                           i, 0, AContainer::CO_FIX );
+      buts[i] = ac1_buttons->addWidget( new Button( aguix, 0, 0, buttonss[i], 1, 0, i ),
+                                        i, 0, AContainer::CO_FIX );
     }
   }
 
@@ -253,7 +253,7 @@ int Requester::string_request( const char *title, const char *text, const char *
     twin = aguix->getTransientWindow();
   }
   
-  AWindow *win = new AWindow( aguix, 10, 10, 10, 10, 0, title );
+  AWindow *win = new AWindow( aguix, 10, 10, 10, 10, 0, title, AWindow::AWINDOW_DIALOG );
   win->create();
   if ( twin != NULL ) {
     win->setTransientForAWindow( twin );
@@ -270,8 +270,8 @@ int Requester::string_request( const char *title, const char *text, const char *
   ac1_texts->setBorderWidth( 5 );
   win->addMultiLineText( text, *ac1_texts, 0, 0, NULL, NULL );
 
-  sg = (StringGadget*)ac1->add( new StringGadget( aguix, 0, 0, 100, default_str, 0 ),
-                                0, 1, AContainer::CO_INCW );
+  sg = ac1->addWidget( new StringGadget( aguix, 0, 0, 100, default_str, 0 ),
+                       0, 1, AContainer::CO_INCW );
   sg->setAcceptFocus( true );
 
   ac1->setMinHeight( 5, 0, 2 );
@@ -292,12 +292,12 @@ int Requester::string_request( const char *title, const char *text, const char *
   buts = (Button**)_allocsafe( nbuttons * sizeof( Button* ) );
 
   if ( nbuttons == 1 ) {
-    buts[0] = (Button*)ac1_buttons->add( new Button( aguix, 0, 0, buttonss[0], 1, 0, 0 ),
-                                         1, 0, AContainer::CO_FIX );
+    buts[0] = ac1_buttons->addWidget( new Button( aguix, 0, 0, buttonss[0], 1, 0, 0 ),
+                                      1, 0, AContainer::CO_FIX );
   } else {
     for ( i = 0; i < nbuttons; i++ ) {
-      buts[i] = (Button*)ac1_buttons->add( new Button( aguix, 0, 0, buttonss[i], 1, 0, i ),
-                                           i, 0, AContainer::CO_FIX );
+      buts[i] = ac1_buttons->addWidget( new Button( aguix, 0, 0, buttonss[i], 1, 0, i ),
+                                        i, 0, AContainer::CO_FIX );
     }
   }
 
@@ -476,7 +476,7 @@ int Requester::request_choose( const char *title,
     twin = aguix->getTransientWindow();
   }
   
-  AWindow *win = new AWindow( aguix, 10, 10, 10, 10, 0, title );
+  AWindow *win = new AWindow( aguix, 10, 10, 10, 10, 0, title, AWindow::AWINDOW_DIALOG );
   win->create();
   if ( twin != NULL ) {
     win->setTransientForAWindow( twin );
@@ -495,7 +495,7 @@ int Requester::request_choose( const char *title,
   ac1_1->setMaxSpace( 5 );
 
   for ( i = 0; i < lines; i++ ) {
-    ac1_1->add( new Text( aguix, 0, 0, liness[i], 1 ),
+    ac1_1->addWidget( new Text( aguix, 0, 0, liness[i], 1 ),
 		0, i, AContainer::CO_INCWNR );
   }
   
@@ -506,10 +506,10 @@ int Requester::request_choose( const char *title,
   ac1_2->setMinSpace( 5 );
   ac1_2->setMaxSpace( -1 );
   ac1_2->setBorderWidth( 0 );
-  ChooseButton *cb1 = (ChooseButton*)ac1_2->add( new ChooseButton( aguix, 0,
-								   0, choose_var,
-								   choose_text, LABEL_RIGHT, 1, 1 ),
-						 1, 0, AContainer::CO_FIXNR );
+  ChooseButton *cb1 = ac1_2->addWidget( new ChooseButton( aguix, 0,
+                                                          0, choose_var,
+                                                          choose_text, LABEL_RIGHT, 1, 1 ),
+                                        1, 0, AContainer::CO_FIXNR );
   AContainer *ac1_3 = ac1->add( new AContainer( win, nbuttons, 1 ), 0, 2 );
   ac1_3->setMinSpace( 5 );
   ac1_3->setMaxSpace( -1 );
@@ -517,8 +517,8 @@ int Requester::request_choose( const char *title,
 
   buts = (Button**)_allocsafe( nbuttons * sizeof( Button* ) );
   for ( i = 0; i < nbuttons; i++ ) {
-    buts[i] = (Button*)ac1_3->add( new Button( aguix, 0, 0, buttonss[i], 1, 0, i ),
-				   i, 0, AContainer::CO_FIX );
+    buts[i] = ac1_3->addWidget( new Button( aguix, 0, 0, buttonss[i], 1, 0, i ),
+                                i, 0, AContainer::CO_FIX );
   }
   for ( i = 0; i < nbuttons; i++ ) {
     buts[i]->setAcceptFocus( true );
@@ -664,7 +664,7 @@ int Requester::request( const char *title, TextStorage &ts, const char *buttons,
     twin = aguix->getTransientWindow();
   }
   
-  AWindow *win = new AWindow( aguix, 10, 10, 10, 10, 0, title );
+  AWindow *win = new AWindow( aguix, 10, 10, 10, 10, 0, title, AWindow::AWINDOW_DIALOG );
   win->create();
   if ( twin != NULL ) {
     win->setTransientForAWindow( twin );
@@ -678,10 +678,15 @@ int Requester::request( const char *title, TextStorage &ts, const char *buttons,
 
   TextView *tv = new TextView( aguix, 0, 0, 100, 100, 0, "", ts );
   tv->setDisplayFocus( true );
-  ac1->add( tv, 0, 0, AContainer::CO_MIN );
+  ac1->addWidget( tv, 0, 0, AContainer::CO_MIN );
   tv->create();
   tv->show();
-  tv->maximizeX( aguix->getRootWindowWidth() * 60 / 100 );
+
+  int rw;
+
+  aguix->getLargestDimensionOfCurrentScreen( NULL, NULL, &rw, NULL );
+
+  tv->maximizeX( rw * 60 / 100 );
   tv->setLineWrap( true );
   tv->maximizeYLines( 15 );
   ac1->readLimits();
@@ -693,8 +698,8 @@ int Requester::request( const char *title, TextStorage &ts, const char *buttons,
     ac1_3->setBorderWidth( 0 );
     
     buts = (Button**)_allocsafe( nbuttons * sizeof( Button* ) );
-    buts[0] = (Button*)ac1_3->add( new Button( aguix, 0, 0, buttonss[0], 1, 0, 0 ),
-				   1, 0, AContainer::CO_FIX );
+    buts[0] = ac1_3->addWidget( new Button( aguix, 0, 0, buttonss[0], 1, 0, 0 ),
+                                1, 0, AContainer::CO_FIX );
   } else {
     AContainer *ac1_3 = ac1->add( new AContainer( win, nbuttons, 1 ), 0, 1 );
     ac1_3->setMinSpace( 5 );
@@ -703,8 +708,8 @@ int Requester::request( const char *title, TextStorage &ts, const char *buttons,
     
     buts = (Button**)_allocsafe( nbuttons * sizeof( Button* ) );
     for ( i = 0; i < nbuttons; i++ ) {
-      buts[i] = (Button*)ac1_3->add( new Button( aguix, 0, 0, buttonss[i], 1, 0, i ),
-				     i, 0, AContainer::CO_FIX );
+      buts[i] = ac1_3->addWidget( new Button( aguix, 0, 0, buttonss[i], 1, 0, i ),
+                                  i, 0, AContainer::CO_FIX );
     }
   }
   for ( i = 0; i < nbuttons; i++ ) {
