@@ -1,6 +1,6 @@
 /* slider.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2005-2009,2012 Ralf Hoffmann.
+ * Copyright (C) 2005-2009,2012-2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -133,7 +133,7 @@ bool Slider::handleMessage( XEvent *E, Message *msg )
 	agmsg->slider.time = msg->time;
 	agmsg->slider.mouse = true;
 	agmsg->type = AG_SLIDER_PRESSED;
-        msgAndCB( std::auto_ptr<AGMessage>( agmsg ) );
+        msgAndCB( std::unique_ptr<AGMessage>( agmsg ) );
       }
     } else if ( msg->type == KeyPress ) {
       if ( ( getAcceptFocus() == true ) && ( getHasFocus() == true ) ) {
@@ -704,7 +704,7 @@ void Slider::msgAndCallback()
     agmsg->slider.time = time( NULL );
     agmsg->slider.mouse = false;
     agmsg->type = AG_SLIDER_CHANGED;
-    msgAndCB( std::auto_ptr<AGMessage>( agmsg ) );
+    msgAndCB( std::unique_ptr<AGMessage>( agmsg ) );
     
     callback( offset );
     
