@@ -1,6 +1,6 @@
 /* stringgadget.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2008,2010,2012 Ralf Hoffmann.
+ * Copyright (C) 2001-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -131,6 +131,9 @@ private:
   void prepareBG( bool force = false );
   int getBorderWidth() const;
   int clearTextBG();
+
+  void jumpToNextWord();
+  void jumpToPrevWord();
 };
 
 #endif
