@@ -1,6 +1,6 @@
 /* textview.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2005-2012 Ralf Hoffmann.
+ * Copyright (C) 2005-2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -46,10 +46,11 @@ TextView::TextView( AGUIX *parent,
 		    int bg,
 		    std::string title,
 		    TextStorage &_ts ) : AWindow( parent, x, y, width, height, bg, title ),
-                                         ts( _ts ),
-                                         tvcb( this ),
-                                         _show_line_numbers( false ),
-                                         m_buffer( 0 )
+                                 ts( _ts ),
+                                 tvcb( this ),
+                                 _show_line_numbers( false ),
+                                 m_buffer( 0 ),
+                                 m_show_frame( true )
 {
   hbar = vbar = NULL;
   cont = NULL;
@@ -149,13 +150,15 @@ void TextView::boxRedraw()
   _aguix->setFG( getBG() );
   _aguix->FillRectangle( m_buffer, 0, 0, _w, _h );
 
-  _aguix->setFG( 0, 2 );
-  _aguix->DrawLine( m_buffer, 0, 0, _w - 1, 0 );
-  _aguix->DrawLine( m_buffer, 0, 0, 0, _h - 1 );
-  _aguix->setFG( 0, 1 );
-  _aguix->DrawLine( m_buffer, 0, _h - 1, _w - 1, _h - 1 );
-  _aguix->DrawLine( m_buffer, _w - 1, _h - 1, _w - 1, 1 );
-  
+  if ( m_show_frame ) {
+      _aguix->setFG( 0, 2 );
+      _aguix->DrawLine( m_buffer, 0, 0, _w - 1, 0 );
+      _aguix->DrawLine( m_buffer, 0, 0, 0, _h - 1 );
+      _aguix->setFG( 0, 1 );
+      _aguix->DrawLine( m_buffer, 0, _h - 1, _w - 1, _h - 1 );
+      _aguix->DrawLine( m_buffer, _w - 1, _h - 1, _w - 1, 1 );
+  }
+
   if ( getDisplayFocus() == true ) {
     if ( getHasFocus() == true ) {
       _aguix->setFG( 0, 2 );
@@ -667,6 +670,7 @@ void TextView::maximizeX( int max_width )
 {
   int l;
   int bw;
+  int rw;
 
   if ( line_wrap == true ) return;
   
@@ -676,8 +680,10 @@ void TextView::maximizeX( int max_width )
   l += 2 * bw + 2 + ( ( vbar != NULL ) ? vbar->getWidth() : 0 );
   l += getTextStartOffset();
 
-  if ( l >= _aguix->getRootWindowWidth() )
-    l = _aguix->getRootWindowWidth();
+  _aguix->getLargestDimensionOfCurrentScreen( NULL, NULL, &rw, NULL );
+
+  if ( l >= rw )
+    l = rw;
 
   if ( max_width > 0 && l > max_width ) {
       l = max_width;
@@ -690,6 +696,7 @@ void TextView::maximizeYLines( int max_lines )
 {
   int l, elementHeight;
   int bw;
+  int rh;
 
   if ( font == NULL ) elementHeight = _aguix->getCharHeight();
   else elementHeight = font->getCharHeight();
@@ -701,8 +708,10 @@ void TextView::maximizeYLines( int max_lines )
   l = ( l + 1 ) * elementHeight;
   l += 2 * bw + 2 + ( ( ( hbar != NULL ) && ( line_wrap == false ) ) ? hbar->getHeight() : 0 );
 
-  if ( l >= _aguix->getRootWindowHeight() )
-    l = _aguix->getRootWindowHeight();
+  _aguix->getLargestDimensionOfCurrentScreen( NULL, NULL, NULL, &rh );
+
+  if ( l >= rh )
+    l = rh;
 
   resize( getWidth(), l );
 }
@@ -1232,3 +1241,8 @@ void TextView::copySelectionToClipboard()
         _aguix->copyToClipboard( s );
     }
 }
+
+void TextView::showFrame( bool nv )
+{
+    m_show_frame = nv;
+}
