@@ -1,6 +1,6 @@
 /* textview.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2005-2008, 2010-2012 Ralf Hoffmann.
+ * Copyright (C) 2005-2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -117,6 +117,8 @@ public:
   void makeSelectionVisible();
 
   int getYOffset();
+
+    void showFrame( bool nv );
 protected:
   class AContainer *cont;
   Slider *hbar, *vbar;
@@ -147,6 +149,8 @@ protected:
   Pixmap m_buffer;
 
     std::pair< int, int > m_button_press_unwrapped_line_pos;
+
+    bool m_show_frame;
   
   void boxRedraw();
   void createContainer();
