@@ -1,6 +1,6 @@
 /* util.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2008,2010,2012 Ralf Hoffmann.
+ * Copyright (C) 2001-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -403,4 +403,39 @@ namespace AGUIXUtils
 
         return false;
     }
+
+    void rstrip( std::string &str )
+    {
+        if ( str.empty() ) return;
+
+        int pos = str.length() - 1;
+
+        for ( pos = str.length() - 1;
+              pos >= 0 && str[pos] == ' ';
+              pos-- );
+
+        str.resize( pos + 1 );
+    }
+
+    void strip( std::string &str )
+    {
+        if ( str.empty() ) return;
+
+        int pos = str.length() - 1;
+
+        for ( pos = str.length() - 1;
+              pos >= 0 && str[pos] == ' ';
+              pos-- );
+
+        str.resize( pos + 1 );
+
+        for ( pos = 0;
+              pos < (int)str.length() && str[pos] == ' ';
+              pos++ );
+
+        if ( pos > 0 ) {
+            std::string nv( str, pos );
+            str = nv;
+        }
+    }
 }
