@@ -1,6 +1,6 @@
 /* basic_actions.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2008,2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2008,2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,7 +21,6 @@
 
 #include "basic_actions.h"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "wcfiletype.hh"
 #include "fileentry.hh"
@@ -30,8 +29,10 @@
 #include "dnd.h"
 #include <aguix/button.h>
 #include <aguix/cyclebutton.h>
+#include <aguix/awindow.h>
 #include "wconfig.h"
 #include "datei.h"
+#include "virtualdirmode.hh"
 
 /***********
   DNDAction
@@ -66,18 +67,11 @@ DNDAction *DNDAction::duplicate() const
 
 int DNDAction::run( WPUContext *wpu, ActionMessage *msg )
 {
-  ListerMode *lm1;
-  // do nothing in case of DND
-  if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1 = msg->getWorker()->getActiveLister();
-    if(l1!=NULL) {
-      lm1=l1->getActiveMode();
-      if(lm1!=NULL)
-        if(lm1->isType("NormalMode")==true)
-          normalmodedndaction( msg );
+    // do nothing in case of DND
+    if(msg->mode!=msg->AM_MODE_DNDACTION) {
+        normalmodedndaction( msg );
     }
-  }
-  return 0;
+    return 0;
 }
 
 const char *DNDAction::getDescription()
@@ -87,12 +81,10 @@ const char *DNDAction::getDescription()
 
 void DNDAction::normalmodedndaction( ActionMessage *am )
 {
-  ListerMode *lm1;
+  ListerMode *lm1 = NULL;
   Lister *l1 = am->getWorker()->getActiveLister();
-  NormalMode *nm=NULL;
   ActionMessage amsg( am->getWorker() );
   const FileEntry *fe = NULL;
-  NM_specialsourceExt *ss1;
 
   switch ( am->mode ) {
     case ActionMessage::AM_MODE_DNDACTION:
@@ -123,52 +115,46 @@ void DNDAction::normalmodedndaction( ActionMessage *am )
     default:
       if(l1!=NULL) {
         lm1=l1->getActiveMode();
-        if(lm1!=NULL)
-          if(lm1->isType("NormalMode")==true) nm=(NormalMode*)lm1;
       }
-      if(nm!=NULL) {
-        std::list<NM_specialsourceExt*> *files=new std::list<NM_specialsourceExt*>;
-	std::list<NM_specialsourceExt*>::iterator it1;
-        nm->getSelFiles( files,
-                         ( am->mode == ActionMessage::AM_MODE_ONLYACTIVE ) ? NormalMode::NM_GETFILES_ONLYACTIVE :
-                                                                             NormalMode::NM_GETFILES_SELORACT,
+      if (lm1 != NULL ) {
+        std::list< NM_specialsourceExt > files;
+        lm1->getSelFiles( files,
+                         ( am->mode == ActionMessage::AM_MODE_ONLYACTIVE ) ? ListerMode::LM_GETFILES_ONLYACTIVE :
+                                                                             ListerMode::LM_GETFILES_SELORACT,
                          false );
         WCFiletype *ft;
         
         amsg.startLister=l1;
         amsg.mode=amsg.AM_MODE_SPECIAL;
         
-        if(files->size()==0) {
+        if ( files.size() == 0 ) {
           ft = wconfig->getvoidtype();
           if(ft!=NULL) {
             amsg.setFE( NULL );
-	    amsg.filetype = ft;
+            amsg.filetype = ft;
             amsg.m_action_descr = RefCount<ActionDescr>( new DNDAction::DNDActionDescr() );
             am->getWorker()->interpret(ft->getDNDActions(),&amsg);
           }
         } else {
-	  for ( it1 = files->begin(); it1 != files->end(); it1++ ) {
-	    ss1 = *it1;
-            fe = ss1->entry();
-            if(fe!=NULL) {
-	      ft=fe->filetype;
-	      if ( ft == NULL ) {
-		if ( fe->isDir() == true ) ft = wconfig->getdirtype();
-		else ft = wconfig->getnotyettype();
-	      }
-	      if(ft!=NULL) {
-		// NM_specialsourceExt already contains a duplicated FE so we can use
-		// it here without another copy
-		amsg.setFE( fe );
-		amsg.filetype = ft;
-                amsg.m_action_descr = RefCount<ActionDescr>( new DNDAction::DNDActionDescr() );
-		am->getWorker()->interpret(ft->getDNDActions(),&amsg);
-	      }
+            for ( auto &ss1 : files ) {
+                fe = ss1.entry();
+                if ( fe != NULL ) {
+                    ft = fe->filetype;
+                    if ( ft == NULL ) {
+                        if ( fe->isDir() == true ) ft = wconfig->getdirtype();
+                        else ft = wconfig->getnotyettype();
+                    }
+                    if ( ft != NULL ) {
+                        // NM_specialsourceExt already contains a duplicated FE so we can use
+                        // it here without another copy
+                        amsg.setFE( fe );
+                        amsg.filetype = ft;
+                        amsg.m_action_descr = RefCount<ActionDescr>( new DNDAction::DNDActionDescr() );
+                        am->getWorker()->interpret( ft->getDNDActions(), &amsg );
+                    }
+                }
             }
-            delete ss1;
-          }
         }
-        delete files;
       }
       break;
   }
@@ -217,17 +203,8 @@ DoubleClickAction *DoubleClickAction::duplicate() const
 
 int DoubleClickAction::run( WPUContext *wpu, ActionMessage *msg )
 {
-  ListerMode *lm1;
-  if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1 = msg->getWorker()->getActiveLister();
-    if(l1!=NULL) {
-      lm1=l1->getActiveMode();
-      if(lm1!=NULL)
-        if(lm1->isType("NormalMode")==true)
-          normalmodeddaction( msg );
-    }
-  } else normalmodeddaction( msg );
-  return 0;
+    normalmodeddaction( msg );
+    return 0;
 }
 
 const char *DoubleClickAction::getDescription()
@@ -237,12 +214,10 @@ const char *DoubleClickAction::getDescription()
 
 void DoubleClickAction::normalmodeddaction( ActionMessage *am )
 {
-  ListerMode *lm1;
+  ListerMode *lm1 = NULL;
   Lister *l1;
-  NormalMode *nm=NULL;
   ActionMessage amsg( am->getWorker() );
   const FileEntry *fe = NULL;
-  NM_specialsourceExt *ss1;
 
   switch ( am->mode ) {
     case ActionMessage::AM_MODE_DNDACTION:
@@ -299,52 +274,46 @@ void DoubleClickAction::normalmodeddaction( ActionMessage *am )
       l1 = am->getWorker()->getActiveLister();
       if(l1!=NULL) {
         lm1=l1->getActiveMode();
-        if(lm1!=NULL)
-          if(lm1->isType("NormalMode")==true) nm=(NormalMode*)lm1;
       }
-      if(nm!=NULL) {
-        std::list<NM_specialsourceExt*> *files = new std::list<NM_specialsourceExt*>;
-	std::list<NM_specialsourceExt*>::iterator it1;
-        nm->getSelFiles( files,
-                         ( am->mode == ActionMessage::AM_MODE_ONLYACTIVE ) ? NormalMode::NM_GETFILES_ONLYACTIVE :
-                                                                             NormalMode::NM_GETFILES_SELORACT,
+      if ( lm1 != NULL ) {
+        std::list< NM_specialsourceExt > files;
+        lm1->getSelFiles( files,
+                         ( am->mode == ActionMessage::AM_MODE_ONLYACTIVE ) ? ListerMode::LM_GETFILES_ONLYACTIVE :
+                                                                             ListerMode::LM_GETFILES_SELORACT,
                          false );
         WCFiletype *ft;
       
         amsg.startLister=l1;
         amsg.mode=amsg.AM_MODE_SPECIAL;
       
-        if(files->size()==0) {
+        if ( files.size() == 0 ) {
           ft = wconfig->getvoidtype();
           if(ft!=NULL) {
             amsg.setFE( NULL );
-	    amsg.filetype = ft;
+            amsg.filetype = ft;
             amsg.m_action_descr = RefCount<ActionDescr>( new DoubleClickAction::DoubleClickActionDescr() );
             am->getWorker()->interpret(ft->getDoubleClickActions(),&amsg);
           }
         } else {
-	  for ( it1 = files->begin(); it1 != files->end(); it1++ ) {
-	    ss1 = *it1;
-            fe = ss1->entry();
-            if(fe!=NULL) {
-	      ft=fe->filetype;
-	      if ( ft == NULL ) {
-		if ( fe->isDir() == true ) ft = wconfig->getdirtype();
-		else ft = wconfig->getnotyettype();
-	      }
-	      if(ft!=NULL) {
-		// NM_specialsourceExt already contains a duplicated FE so we can use
-		// it here without another copy
-		amsg.setFE( fe );
-		amsg.filetype = ft;
-                amsg.m_action_descr = RefCount<ActionDescr>( new DoubleClickAction::DoubleClickActionDescr() );
-		am->getWorker()->interpret(ft->getDoubleClickActions(),&amsg);
-	      }
+            for ( auto &ss1 : files ) {
+                fe = ss1.entry();
+                if ( fe != NULL ) {
+                    ft = fe->filetype;
+                    if ( ft == NULL ) {
+                        if ( fe->isDir() == true ) ft = wconfig->getdirtype();
+                        else ft = wconfig->getnotyettype();
+                    }
+                    if ( ft != NULL ) {
+                        // NM_specialsourceExt already contains a duplicated FE so we can use
+                        // it here without another copy
+                        amsg.setFE( fe );
+                        amsg.filetype = ft;
+                        amsg.m_action_descr = RefCount<ActionDescr>( new DoubleClickAction::DoubleClickActionDescr() );
+                        am->getWorker()->interpret( ft->getDoubleClickActions(), &amsg );
+                    }
+                }
             }
-            delete ss1;
-          }
         }
-        delete files;
       }
       break;
   }
@@ -393,17 +362,8 @@ ShowAction *ShowAction::duplicate() const
 
 int ShowAction::run( WPUContext *wpu, ActionMessage *msg )
 {
-  ListerMode *lm1;
-  if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1 = msg->getWorker()->getActiveLister();
-    if(l1!=NULL) {
-      lm1=l1->getActiveMode();
-      if(lm1!=NULL)
-        if(lm1->isType("NormalMode")==true)
-          normalmodeshow( msg );
-    }
-  } else normalmodeshow( msg );
-  return 0;
+    normalmodeshow( msg );
+    return 0;
 }
 
 const char *ShowAction::getDescription()
@@ -413,12 +373,10 @@ const char *ShowAction::getDescription()
 
 void ShowAction::normalmodeshow( ActionMessage *am )
 {
-  ListerMode *lm1;
-  NormalMode *nm=NULL;
+  ListerMode *lm1 = NULL;
   ActionMessage amsg( am->getWorker() );
   Lister *l1;
   const FileEntry *fe = NULL;
-  NM_specialsourceExt *ss1;
 
   switch ( am->mode ) {
     case ActionMessage::AM_MODE_DNDACTION:
@@ -475,52 +433,46 @@ void ShowAction::normalmodeshow( ActionMessage *am )
       l1 = am->getWorker()->getActiveLister();
       if(l1!=NULL) {
         lm1=l1->getActiveMode();
-        if(lm1!=NULL)
-          if(lm1->isType("NormalMode")==true) nm=(NormalMode*)lm1;
       }
-      if(nm!=NULL) {
-        std::list<NM_specialsourceExt*> *files = new std::list<NM_specialsourceExt*>;
-	std::list<NM_specialsourceExt*>::iterator it1;
-        nm->getSelFiles( files,
-                         ( am->mode == ActionMessage::AM_MODE_ONLYACTIVE ) ? NormalMode::NM_GETFILES_ONLYACTIVE :
-                                                                             NormalMode::NM_GETFILES_SELORACT,
+      if ( lm1 != NULL ) {
+        std::list< NM_specialsourceExt > files;
+        lm1->getSelFiles( files,
+                         ( am->mode == ActionMessage::AM_MODE_ONLYACTIVE ) ? ListerMode::LM_GETFILES_ONLYACTIVE :
+                                                                             ListerMode::LM_GETFILES_SELORACT,
                          false );
         WCFiletype *ft;
       
         amsg.startLister=l1;
         amsg.mode=amsg.AM_MODE_SPECIAL;
       
-        if(files->size()==0) {
+        if ( files.size() == 0 ) {
           ft = wconfig->getvoidtype();
           if(ft!=NULL) {
             amsg.setFE( NULL );
-	    amsg.filetype = ft;
+            amsg.filetype = ft;
             amsg.m_action_descr = RefCount<ActionDescr>( new ShowAction::ShowActionDescr() );
             am->getWorker()->interpret(ft->getShowActions(),&amsg);
           }
         } else {
-	  for ( it1 = files->begin(); it1 != files->end(); it1++ ) {
-	    ss1 = *it1;
-            fe = ss1->entry();
-            if(fe!=NULL) {
-	      ft=fe->filetype;
-	      if ( ft == NULL ) {
-		if ( fe->isDir() == true ) ft = wconfig->getdirtype();
-		else ft = wconfig->getnotyettype();
-	      }
-	      if(ft!=NULL) {
-		// NM_specialsourceExt already contains a duplicated FE so we can use
-		// it here without another copy
-		amsg.setFE( fe );
-		amsg.filetype = ft;
-                amsg.m_action_descr = RefCount<ActionDescr>( new ShowAction::ShowActionDescr() );
-		am->getWorker()->interpret(ft->getShowActions(),&amsg);
-	      }
+            for ( auto &ss1 : files ) {
+                fe = ss1.entry();
+                if ( fe != NULL ) {
+                    ft = fe->filetype;
+                    if ( ft == NULL ) {
+                        if ( fe->isDir() == true ) ft = wconfig->getdirtype();
+                        else ft = wconfig->getnotyettype();
+                    }
+                    if ( ft != NULL ) {
+                        // NM_specialsourceExt already contains a duplicated FE so we can use
+                        // it here without another copy
+                        amsg.setFE( fe );
+                        amsg.filetype = ft;
+                        amsg.m_action_descr = RefCount<ActionDescr>( new ShowAction::ShowActionDescr() );
+                        am->getWorker()->interpret( ft->getShowActions(), &amsg );
+                    }
+                }
             }
-            delete ss1;
-          }
         }
-        delete files;
       }
       break;
   }
@@ -569,17 +521,8 @@ RawShowAction *RawShowAction::duplicate() const
 
 int RawShowAction::run( WPUContext *wpu, ActionMessage *msg )
 {
-  ListerMode *lm1;
-  if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1 = msg->getWorker()->getActiveLister();
-    if(l1!=NULL) {
-      lm1=l1->getActiveMode();
-      if(lm1!=NULL)
-        if(lm1->isType("NormalMode")==true)
-          normalmoderawshow( msg );
-    }
-  } else normalmoderawshow( msg );
-  return 0;
+    normalmoderawshow( msg );
+    return 0;
 }
 
 const char *RawShowAction::getDescription()
@@ -589,12 +532,10 @@ const char *RawShowAction::getDescription()
 
 void RawShowAction::normalmoderawshow( ActionMessage *am )
 {
-  ListerMode *lm1;
-  NormalMode *nm=NULL;
+  ListerMode *lm1 = NULL;
   ActionMessage amsg( am->getWorker() );
   Lister *l1;
   const FileEntry *fe = NULL;
-  NM_specialsourceExt *ss1;
 
   switch ( am->mode ) {
     case ActionMessage::AM_MODE_DNDACTION:
@@ -651,52 +592,46 @@ void RawShowAction::normalmoderawshow( ActionMessage *am )
       l1 = am->getWorker()->getActiveLister();
       if(l1!=NULL) {
         lm1=l1->getActiveMode();
-        if(lm1!=NULL)
-          if(lm1->isType("NormalMode")==true) nm=(NormalMode*)lm1;
       }
-      if(nm!=NULL) {
-        std::list<NM_specialsourceExt*> *files = new std::list<NM_specialsourceExt*>;
-	std::list<NM_specialsourceExt*>::iterator it1;
-        nm->getSelFiles( files,
-                         ( am->mode == ActionMessage::AM_MODE_ONLYACTIVE ) ? NormalMode::NM_GETFILES_ONLYACTIVE :
-                                                                             NormalMode::NM_GETFILES_SELORACT,
-                         false );
+      if ( lm1 != NULL ) {
+        std::list< NM_specialsourceExt > files;
+        lm1->getSelFiles( files,
+                          ( am->mode == ActionMessage::AM_MODE_ONLYACTIVE ) ? ListerMode::LM_GETFILES_ONLYACTIVE :
+                          ListerMode::LM_GETFILES_SELORACT,
+                          false );
         WCFiletype *ft;
       
         amsg.startLister=l1;
         amsg.mode=amsg.AM_MODE_SPECIAL;
       
-        if(files->size()==0) {
+        if ( files.size() == 0 ) {
           ft = wconfig->getvoidtype();
           if(ft!=NULL) {
             amsg.setFE( NULL );
-	    amsg.filetype = ft;
+            amsg.filetype = ft;
             amsg.m_action_descr = RefCount<ActionDescr>( new RawShowAction::RawShowActionDescr() );
             am->getWorker()->interpret(ft->getRawShowActions(),&amsg);
           }
         } else {
-	  for ( it1 = files->begin(); it1 != files->end(); it1++ ) {
-	    ss1 = *it1;
-            fe = ss1->entry();
-            if(fe!=NULL) {
-	      ft=fe->filetype;
-	      if ( ft == NULL ) {
-		if ( fe->isDir() == true ) ft = wconfig->getdirtype();
-		else ft = wconfig->getnotyettype();
-	      }
-	      if(ft!=NULL) {
-		// NM_specialsourceExt already contains a duplicated FE so we can use
-		// it here without another copy
-		amsg.setFE( fe );
-		amsg.filetype = ft;
-                amsg.m_action_descr = RefCount<ActionDescr>( new RawShowAction::RawShowActionDescr() );
-		am->getWorker()->interpret(ft->getRawShowActions(),&amsg);
-	      }
+            for ( auto &ss1 : files ) {
+                fe = ss1.entry();
+                if ( fe != NULL ) {
+                    ft=fe->filetype;
+                    if ( ft == NULL ) {
+                        if ( fe->isDir() == true ) ft = wconfig->getdirtype();
+                        else ft = wconfig->getnotyettype();
+                    }
+                    if ( ft != NULL ) {
+                        // NM_specialsourceExt already contains a duplicated FE so we can use
+                        // it here without another copy
+                        amsg.setFE( fe );
+                        amsg.filetype = ft;
+                        amsg.m_action_descr = RefCount<ActionDescr>( new RawShowAction::RawShowActionDescr() );
+                        am->getWorker()->interpret( ft->getRawShowActions(), &amsg );
+                    }
+                }
             }
-            delete ss1;
-          }
         }
-        delete files;
       }
       break;
   }
@@ -748,17 +683,8 @@ UserAction *UserAction::duplicate() const
 
 int UserAction::run( WPUContext *wpu, ActionMessage *msg )
 {
-  ListerMode *lm1;
-  if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1 = msg->getWorker()->getActiveLister();
-    if(l1!=NULL) {
-      lm1=l1->getActiveMode();
-      if(lm1!=NULL)
-        if(lm1->isType("NormalMode")==true)
-          normalmodeuseraction( msg );
-    }
-  } else normalmodeuseraction( msg );
-  return 0;
+    normalmodeuseraction( msg );
+    return 0;
 }
 
 const char *UserAction::getDescription()
@@ -768,12 +694,10 @@ const char *UserAction::getDescription()
 
 void UserAction::normalmodeuseraction( ActionMessage *am )
 {
-  ListerMode *lm1;
-  NormalMode *nm=NULL;
+  ListerMode *lm1 = NULL;
   ActionMessage amsg( am->getWorker() );
   Lister *l1;
   const FileEntry *fe = NULL;
-  NM_specialsourceExt *ss1;
 
   switch ( am->mode ) {
     case ActionMessage::AM_MODE_DNDACTION:
@@ -830,52 +754,46 @@ void UserAction::normalmodeuseraction( ActionMessage *am )
       l1 = am->getWorker()->getActiveLister();
       if(l1!=NULL) {
         lm1=l1->getActiveMode();
-        if(lm1!=NULL)
-          if(lm1->isType("NormalMode")==true) nm=(NormalMode*)lm1;
       }
-      if(nm!=NULL) {
-        std::list<NM_specialsourceExt*> *files = new std::list<NM_specialsourceExt*>;
-	std::list<NM_specialsourceExt*>::iterator it1;
-        nm->getSelFiles( files,
-                         ( am->mode == ActionMessage::AM_MODE_ONLYACTIVE ) ? NormalMode::NM_GETFILES_ONLYACTIVE :
-                                                                             NormalMode::NM_GETFILES_SELORACT,
-                         false );
+      if ( lm1 != NULL ) {
+        std::list< NM_specialsourceExt > files;
+        lm1->getSelFiles( files,
+                          ( am->mode == ActionMessage::AM_MODE_ONLYACTIVE ) ? ListerMode::LM_GETFILES_ONLYACTIVE :
+                          ListerMode::LM_GETFILES_SELORACT,
+                          false );
         WCFiletype *ft;
       
         amsg.startLister=l1;
         amsg.mode=amsg.AM_MODE_SPECIAL;
       
-        if(files->size()==0) {
+        if ( files.size() == 0 ) {
           ft = wconfig->getvoidtype();
           if(ft!=NULL) {
             amsg.setFE( NULL );
-	    amsg.filetype = ft;
+            amsg.filetype = ft;
             amsg.m_action_descr = RefCount<ActionDescr>( new UserAction::UserActionDescr( nr ) );
             am->getWorker()->interpret(ft->getUserActions(nr),&amsg);
           }
         } else {
-	  for ( it1 = files->begin(); it1 != files->end(); it1++ ) {
-	    ss1 = *it1;
-            fe = ss1->entry();
-            if(fe!=NULL) {
-	      ft=fe->filetype;
-	      if ( ft == NULL ) {
-		if ( fe->isDir() == true ) ft = wconfig->getdirtype();
-		else ft = wconfig->getnotyettype();
-	      }
-	      if(ft!=NULL) {
-		// NM_specialsourceExt already contains a duplicated FE so we can use
-		// it here without another copy
-		amsg.setFE( fe );
-		amsg.filetype = ft;
-                amsg.m_action_descr = RefCount<ActionDescr>( new UserAction::UserActionDescr( nr ) );
-		am->getWorker()->interpret(ft->getUserActions(nr),&amsg);
-	      }
+            for ( auto &ss1 : files ) {
+                fe = ss1.entry();
+                if ( fe != NULL ) {
+                    ft=fe->filetype;
+                    if ( ft == NULL ) {
+                        if ( fe->isDir() == true ) ft = wconfig->getdirtype();
+                        else ft = wconfig->getnotyettype();
+                    }
+                    if ( ft != NULL ) {
+                        // NM_specialsourceExt already contains a duplicated FE so we can use
+                        // it here without another copy
+                        amsg.setFE( fe );
+                        amsg.filetype = ft;
+                        amsg.m_action_descr = RefCount<ActionDescr>( new UserAction::UserActionDescr( nr ) );
+                        am->getWorker()->interpret( ft->getUserActions( nr ), &amsg );
+                    }
+                }
             }
-            delete ss1;
-          }
         }
-        delete files;
       }
       break;
   }
@@ -892,7 +810,7 @@ int UserAction::configure()
   
   tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(4))+1);
   sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(4));
-  win =new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win =new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe(tstr);
 
@@ -1035,24 +953,24 @@ const char *ChangeHiddenFlag::getName()
 int ChangeHiddenFlag::run( WPUContext *wpu, ActionMessage *msg )
 {
   ListerMode *lm1;
-  NormalMode *nm1;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
     Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       lm1=l1->getActiveMode();
       if(lm1!=NULL) {
-        if(lm1->isType("NormalMode")==true) {
-          nm1=(NormalMode*)lm1;
-          switch(mode) {
-            case 1:
-              nm1->setShowHidden(false);
-              break;
-            case 2:
-              nm1->setShowHidden(true);
-              break;
-            default:
-              nm1->toggleHidden();          
-          }
+        if ( auto vdm = dynamic_cast< VirtualDirMode * >( lm1 ) ) {
+            switch ( mode ) {
+                case 1:
+                    vdm->setShowHidden( false );
+                    break;
+                case 2:
+                    vdm->setShowHidden( true );
+                    break;
+                default:
+                    vdm->toggleHidden();          
+            }
+        } else {
+            lm1->not_supported();
         }
       }
     }
@@ -1093,7 +1011,7 @@ int ChangeHiddenFlag::configure()
   
   tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(7))+1);
   sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(7));
-  win=new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win=new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe(tstr);
 
