@@ -1,6 +1,6 @@
 /* calendar.cc
  * This file belongs to Worker, a file manager for UNIX/X11.
- * Copyright (C) 2008,2011 Ralf Hoffmann.
+ * Copyright (C) 2008,2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -84,7 +84,7 @@ bool Calendar::showCalendar()
     int endmode = -1;
     std::string str1;
 
-    win = new AWindow( aguix, 10, 10, 10, 10, 0, catalog.getLocale( 819 ) );
+    win = new AWindow( aguix, 10, 10, 10, 10, 0, catalog.getLocale( 819 ), AWindow::AWINDOW_DIALOG );
     win->create();
     
     AContainer *ac1 = win->setContainer( new AContainer( win, 1, 3 ), true );
