@@ -1,6 +1,6 @@
 /* changelabelop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2008,2011 Ralf Hoffmann.
+ * Copyright (C) 2008,2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,7 +21,6 @@
 
 #include "changelabelop.hh"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "bookmarkdbproxy.hh"
 #include "bookmarkdbentry.hh"
@@ -35,6 +34,7 @@
 #include <aguix/fieldlistview.h>
 #include <aguix/button.h>
 #include <aguix/stringgadget.h>
+#include <aguix/awindow.h>
 #include "wconfig.h"
 
 const char *ChangeLabelOp::name = "ChangeLabelOp";
@@ -74,46 +74,42 @@ int ChangeLabelOp::run( WPUContext *wpu, ActionMessage *msg )
 {
     Lister *l1 = NULL;
     ListerMode *lm1 = NULL;
-    NormalMode *nm1 = NULL;
     
     l1 = msg->getWorker()->getActiveLister();
     if ( l1 == NULL )
         return 1;
 
     lm1 = l1->getActiveMode();
-    if ( lm1 != NULL && lm1->isType( "NormalMode" ) == true ) {
-        nm1 = (NormalMode*)lm1;
-    } else {
+    if ( lm1 == NULL ) {
         return 1;
     }
 
     if ( m_ask_for_label == true ) {
-        nm1->openLabelPopUp();
+        lm1->runCommand( "open_label_popup" );
     } else {
         BookmarkDBProxy &bookmarks = msg->getWorker()->getBookmarkDBInstance();
         //TODO read necessary?
         bookmarks.read();
         
-        std::list<NM_specialsourceExt*> sellist;
-        nm1->getSelFiles( &sellist, NormalMode::NM_GETFILES_ONLYACTIVE );
+        std::list< NM_specialsourceExt > sellist;
+        lm1->getSelFiles( sellist, ListerMode::LM_GETFILES_ONLYACTIVE );
         
         std::string dirname, basename;
 
-        if ( nm1->getCurrentDir() != NULL )
-            dirname = nm1->getCurrentDir();
+        dirname = lm1->getCurrentDirectory();
 
         if ( sellist.empty() == false ) {
-            if ( (*sellist.begin())->entry() != NULL ) {
-                basename = (*sellist.begin())->entry()->name;
+            if ( (*sellist.begin()).entry() != NULL ) {
+                basename = (*sellist.begin()).entry()->name;
+                dirname = NWC::Path::dirname( sellist.begin()->entry()->fullname );
             }
         }
-        nm1->freeSelFiles( &sellist );
 
         if ( dirname.length() > 0 && basename.length() > 0 ) {
             std::string fullname = NWC::Path::join( dirname, basename );
             //TODO normalize fullname?
 
-            std::auto_ptr<BookmarkDBEntry> entry = bookmarks.getEntry( fullname );
+            std::unique_ptr<BookmarkDBEntry> entry = bookmarks.getEntry( fullname );
             if ( entry.get() != NULL ) {
                 if ( entry->getCategory() == m_label ) {
                     bookmarks.delEntry( *entry );
@@ -129,7 +125,7 @@ int ChangeLabelOp::run( WPUContext *wpu, ActionMessage *msg )
             }
             bookmarks.write();
 
-            nm1->updateOnBookmarkChange();
+            lm1->updateOnBookmarkChange();
         }
     }
 
@@ -162,7 +158,7 @@ int ChangeLabelOp::configure()
     
     tstr = (char*)_allocsafe( strlen( catalog.getLocale( 293 ) ) + strlen( catalog.getLocaleCom( 55 ) ) + 1 );
     sprintf( tstr, catalog.getLocale( 293 ), catalog.getLocaleCom( 55 ) );
-    win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+    win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
     win->create();
     _freesafe( tstr );
     
