@@ -1,6 +1,6 @@
 /* changelayoutop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2009-2011 Ralf Hoffmann.
+ * Copyright (C) 2009-2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,7 +21,6 @@
 
 #include "changelayoutop.hh"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include <aguix/awindow.h>
 #include <aguix/text.h>
@@ -193,7 +192,7 @@ ChangeLayoutOp::configure()
     tstr = (char*)_allocsafe( strlen( catalog.getLocale( 293 ) ) +
                               strlen( getDescription() ) + 1 );
     sprintf( tstr, catalog.getLocale( 293 ), getDescription() );
-    m_win = new AWindow( m_aguix, 10, 10, 10, 10, 0, tstr );
+    m_win = new AWindow( m_aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
     m_win->create();
     _freesafe(tstr);
     
