@@ -1,6 +1,6 @@
 /* changesymlinkop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2004,2008,2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2004,2008,2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,10 +21,11 @@
 
 #include "changesymlinkop.h"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "nmspecialsourceext.hh"
 #include "worker_locale.h"
+#include "changesymlinkorder.hh"
+#include "virtualdirmode.hh"
 
 const char *ChangeSymlinkOp::name="ChangeSymlinkOp";
 
@@ -65,9 +66,13 @@ ChangeSymlinkOp::run( WPUContext *wpu, ActionMessage *msg )
     if(l1!=NULL) {
       startlister=l1;
       lm1=startlister->getActiveMode();
-      if(lm1!=NULL)
-        if(lm1->isType("NormalMode")==true)
-          normalmodechangesl( msg );
+      if(lm1!=NULL) {
+          if(dynamic_cast< VirtualDirMode *>( lm1 ) ) {
+              normalmodechangesl( msg );
+          } else {
+              lm1->not_supported();
+          }
+      }
     }
   }
   return 0;
@@ -82,16 +87,13 @@ ChangeSymlinkOp::getDescription()
 int
 ChangeSymlinkOp::normalmodechangesl( ActionMessage *am )
 {
-  NormalMode *nm1=NULL;
-  ListerMode *lm1=NULL;
+  ListerMode *lm1 = NULL;
   NM_specialsourceExt *specialsource=NULL;
   struct NM_changesymlinkorder chslorder;
   
   if(startlister==NULL) return 1;
   lm1=startlister->getActiveMode();
   if(lm1==NULL) return 1;
-  if(lm1->isType("NormalMode")!=true) return 1;
-  nm1=(NormalMode*)lm1;
 
   memset( &chslorder, 0, sizeof( chslorder ) );
   if(am->mode==am->AM_MODE_ONLYACTIVE)
@@ -111,7 +113,10 @@ ChangeSymlinkOp::normalmodechangesl( ActionMessage *am )
   } else
     chslorder.source=chslorder.NM_ALLENTRIES;
 
-  nm1->changesymlink(&chslorder);
+  if ( auto vdm = dynamic_cast< VirtualDirMode * >( lm1 ) ) {
+      vdm->changesymlink( &chslorder );
+  }
+
   if(chslorder.source==chslorder.NM_SPECIAL) {
     if ( specialsource != NULL ) delete specialsource;
     delete chslorder.sources;
