@@ -1,6 +1,6 @@
 /* chmodop.c
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2004,2008,2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2004,2008,2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,13 +21,14 @@
 
 #include "chmodop.h"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "nmspecialsourceext.hh"
 #include "datei.h"
 #include <aguix/button.h>
 #include <aguix/choosebutton.h>
 #include "worker_locale.h"
+#include "chmodorder.hh"
+#include "virtualdirmode.hh"
 
 const char *ChModOp::name="ChModOp";
 
@@ -77,9 +78,13 @@ ChModOp::run( WPUContext *wpu, ActionMessage *msg )
     if(l1!=NULL) {
       startlister=l1;
       lm1=startlister->getActiveMode();
-      if(lm1!=NULL)
-        if(lm1->isType("NormalMode")==true)
-          normalmodechmod( msg );
+      if(lm1!=NULL) {
+          if(dynamic_cast< VirtualDirMode * > ( lm1 ) ) {
+              normalmodechmod( msg );
+          } else {
+              lm1->not_supported();
+          }
+      }
     }
   }
   return 0;
@@ -105,7 +110,6 @@ ChModOp::getDescription()
 int
 ChModOp::normalmodechmod( ActionMessage *am )
 {
-  NormalMode *nm1=NULL;
   ListerMode *lm1=NULL;
   NM_specialsourceExt *specialsource = NULL;
   bool cont=true;
@@ -114,8 +118,6 @@ ChModOp::normalmodechmod( ActionMessage *am )
   if(startlister==NULL) return 1;
   lm1=startlister->getActiveMode();
   if(lm1==NULL) return 1;
-  if(lm1->isType("NormalMode")!=true) return 1;
-  nm1=(NormalMode*)lm1;
 
   if(requestflags==true) {
     if(doconfigure(1)!=0) cont=false;
@@ -149,7 +151,11 @@ ChModOp::normalmodechmod( ActionMessage *am )
     cmorder.onfiles=tonfiles;
     cmorder.ondirs=tondirs;
     cmorder.recursive=trecursive;
-    nm1->chmodf(&cmorder);
+
+    if ( auto vdm = dynamic_cast< VirtualDirMode * >( lm1 ) ) {
+        vdm->chmodf( &cmorder );
+    }
+
     if(cmorder.source==cmorder.NM_SPECIAL) {
       if ( specialsource != NULL ) delete specialsource;
       delete cmorder.sources;
@@ -185,7 +191,7 @@ ChModOp::doconfigure(int mode)
   
   tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(38))+1);
   sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(38));
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe(tstr);
   
