@@ -1,6 +1,6 @@
 /* chownop.c
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2004,2008,2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2004,2008,2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,13 +21,14 @@
 
 #include "chownop.h"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "nmspecialsourceext.hh"
 #include "datei.h"
 #include <aguix/button.h>
 #include <aguix/choosebutton.h>
 #include "worker_locale.h"
+#include "chownorder.hh"
+#include "virtualdirmode.hh"
 
 const char *ChOwnOp::name = "ChOwnOp";
 
@@ -78,9 +79,13 @@ ChOwnOp::run( WPUContext *wpu, ActionMessage *msg )
     if ( l1 != NULL ) {
       startlister = l1;
       lm1 = startlister->getActiveMode();
-      if ( lm1 != NULL )
-        if ( lm1->isType( "NormalMode" ) == true )
-          normalmodechown( msg );
+      if ( lm1 != NULL ) {
+          if ( dynamic_cast< VirtualDirMode * > ( lm1 ) ) {
+              normalmodechown( msg );
+          } else {
+              lm1->not_supported();
+          }
+      }
     }
   }
   return 0;
@@ -106,7 +111,6 @@ ChOwnOp::getDescription()
 int
 ChOwnOp::normalmodechown( ActionMessage *am )
 {
-  NormalMode *nm1 = NULL;
   ListerMode *lm1 = NULL;
   NM_specialsourceExt *specialsource = NULL;
   bool cont = true;
@@ -115,8 +119,6 @@ ChOwnOp::normalmodechown( ActionMessage *am )
   if ( startlister == NULL ) return 1;
   lm1 = startlister->getActiveMode();
   if ( lm1 == NULL ) return 1;
-  if ( lm1->isType( "NormalMode" ) != true ) return 1;
-  nm1 = (NormalMode*)lm1;
 
   if ( requestflags == true ) {
     if ( doconfigure( 1 ) != 0 ) cont = false;
@@ -149,7 +151,11 @@ ChOwnOp::normalmodechown( ActionMessage *am )
     coorder.onfiles = tonfiles;
     coorder.ondirs = tondirs;
     coorder.recursive = trecursive;
-    nm1->chownf( &coorder );
+
+    if ( auto vdm = dynamic_cast< VirtualDirMode * >( lm1 ) ) {
+        vdm->chownf( &coorder );
+    }
+
     if ( coorder.source == coorder.NM_SPECIAL ) {
       if ( specialsource != NULL ) delete specialsource;
       delete coorder.sources;
@@ -185,7 +191,7 @@ ChOwnOp::doconfigure( int mode )
   
   tstr = (char*)_allocsafe( strlen( catalog.getLocale( 293 ) ) + strlen( catalog.getLocaleCom( 43 ) ) + 1 );
   sprintf( tstr, catalog.getLocale( 293 ), catalog.getLocaleCom( 43 ) );
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe( tstr );
 
