@@ -1,6 +1,6 @@
 /* clipboardop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2012 Ralf Hoffmann.
+ * Copyright (C) 2012-2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,7 +21,6 @@
 
 #include "clipboardop.hh"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "ownop.h"
 #include "wpucontext.h"
@@ -35,6 +34,7 @@
 #include <aguix/button.h>
 #include <aguix/choosebutton.h>
 #include <aguix/stringgadget.h>
+#include <aguix/awindow.h>
 #include "dnd.h"
 
 const char *ClipboardOp::name = "ClipboardOp";
@@ -78,7 +78,7 @@ ClipboardOp::run( WPUContext *wpu, ActionMessage *msg )
     if ( wpu->parse( m_string.c_str(),
                      res_str,
                      a_max( EXE_STRING_LEN - 1024, 256 ),
-                     false ) == NormalMode::PARSE_SUCCESS ) {
+                     false ) == WPUContext::PARSE_SUCCESS ) {
         msg->getWorker()->copyToClipboard( res_str );
     }
 
@@ -126,7 +126,7 @@ ClipboardOp::doconfigure(int mode)
   
     tstr = (char*)_allocsafe( strlen( catalog.getLocale( 293 ) ) + strlen( catalog.getLocaleCom( 61 ) ) + 1 );
     sprintf( tstr, catalog.getLocale( 293 ), catalog.getLocaleCom( 61 ) );
-    win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+    win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
     win->create();
     _freesafe( tstr );
 
