@@ -1,6 +1,6 @@
 /* condparser.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2004-2006,2011-2012 Ralf Hoffmann.
+ * Copyright (C) 2004-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -42,6 +42,7 @@ struct CondParser::deftokens CondParser::keywords[] = {
   { "isSock", CondParser::TOK_ISSOCK },
   { "isFIFO", CondParser::TOK_ISFIFO },
   { "isSymlink", CondParser::TOK_ISSYMLINK },
+  { "isBrokenSymlink", CondParser::TOK_ISBROKENSYMLINK },
   { "size", CondParser::TOK_SIZE },
   { "perm", CondParser::TOK_PERM },
   { "toLower", CondParser::TOK_TOLOWER },
@@ -62,6 +63,7 @@ struct CondParser::flaghelp CondParser::flags[] = {
   { "isSock()", 580, -1 },
   { "isFIFO()", 581, -1 },
   { "isSymlink()", 582, -1 },
+  { "isBrokenSymlink()", 1036, -1 },
   { "isLocal()", 626, -1 },
   { "size()", 583, -1 },
   { "perm(mask)", 584, -1 },
@@ -698,6 +700,20 @@ CondParser::erg_t *CondParser::factor()
       e->strval = (char*)_allocsafe( A_BYTESFORNUMBER( e->numval ) );
       sprintf( e->strval, "%d", e->numval );
       break;
+    case TOK_ISBROKENSYMLINK:
+      if ( match( TOK_ISBROKENSYMLINK ) ) return NULL;
+      if ( match( '(' ) ) return NULL;
+      if ( match( ')' ) ) return NULL;
+      e = (erg_t*)_allocsafe( sizeof( erg_t ) );
+      e->wasstring = 0;
+      if ( fe == NULL ) {
+        e->numval = 0;
+      } else {
+        e->numval = ( fe->isLink == true && fe->isCorrupt == true ) ? 1 : 0;
+      }
+      e->strval = (char*)_allocsafe( A_BYTESFORNUMBER( e->numval ) );
+      sprintf( e->strval, "%d", e->numval );
+      break;
     case TOK_SIZE:
       if ( match( TOK_SIZE ) ) return NULL;
       if ( match( '(' ) ) return NULL;
@@ -1078,7 +1094,7 @@ const char *CondParser::requestFlag()
   const char *returnstr;
   int i, trow, nr;
   
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, catalog.getLocale( 338 ) );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, catalog.getLocale( 338 ), AWindow::AWINDOW_DIALOG );
   win->create();
 
   AContainer *ac1 = win->setContainer( new AContainer( win, 1, 3 ), true );
