@@ -1,6 +1,6 @@
 /* configtokens.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2003-2013 Ralf Hoffmann.
+ * Copyright (C) 2003-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -399,7 +399,12 @@ struct mytoken tokenlist[]={
   { APPLY_WINDOW_DIALOG_TYPE_WCP, "applywindowdialogtype" },
   { FLAGS_ARE_RELATIVE_TO_BASEDIR_WCP, "flagsarerelativetobasedir" },
   { COMMANDMENUOP_WCP, "commandmenuop" },
-  { SEARCHMODEONKEYPRESS_WCP, "searchmodeonkeypress" }
+  { SEARCHMODEONKEYPRESS_WCP, "searchmodeonkeypress" },
+  { PATHENTRYONTOP_WCP, "pathentryontop" },
+  { PATHJUMPSETS_WCP, "pathjumpsets" },
+  { STORE_FILES_ALWAYS_WCP, "storefilesalways" },
+  { SHOWDOTDOT_WCP, "showdotdot" },
+  { SHOWBREADCRUMB_WCP, "showbreadcrumb" }
 };
 
 ArrayList *tokenlist_sorted = NULL;
