@@ -1,6 +1,6 @@
 /* copycore.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2013 Ralf Hoffmann.
+ * Copyright (C) 2013-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -29,11 +29,15 @@
 #include "verzeichnis.hh"
 #include <aguix/lowlevelfunc.h>
 #include <aguix/util.h>
+#include <chrono>
+#include <ratio>
 
-CopyCore::CopyCore( struct copyorder *copyorder ) :
-    m_copyorder( copyorder ),
+CopyCore::CopyCore( std::shared_ptr< struct copyorder > co ) :
+    m_copyorder( co ),
     m_cancel( false ),
-    m_bytes_to_copy( 0 )
+    m_bytes_to_copy( 0 ),
+    m_finished( false ),
+    m_ask_to_cancel( false )
 {
     buf = _allocsafe( BUFSIZE );
 }
@@ -49,7 +53,7 @@ CopyCore::nm_newname_t CopyCore::getNewName4Dir( const FileEntry *oldfe, const c
     char *oldname=oldfe->name;
     char *newname=dupstring(oldname);
     bool nameok=false,skip=false,cancel=false;
-    char *return_str=NULL,*buttonstr,*textstr,*newdest;
+    char *buttonstr,*textstr,*newdest;
     int erg;
     Datei::d_fe_t se,lse;
     int round=1;
@@ -78,17 +82,20 @@ CopyCore::nm_newname_t CopyCore::getNewName4Dir( const FileEntry *oldfe, const c
                     catalog.getLocale(8));
             textstr=(char*)_allocsafe(strlen(catalog.getLocale(148))+strlen(oldname)+1);
             sprintf(textstr,catalog.getLocale(148),oldname);
+
+            std::string return_str;
+
             erg = m_copyorder->cowin->string_request( catalog.getLocale( 149 ),
-                                                    textstr,
-                                                    newname,
-                                                    buttonstr,
-                                                    &return_str,
-                                                    Requester::REQUEST_SELECTALL );
+                                                      textstr,
+                                                      newname,
+                                                      buttonstr,
+                                                      return_str,
+                                                      Requester::REQUEST_SELECTALL );
             _freesafe(buttonstr);
             _freesafe(textstr);
-            if(return_str!=NULL) {
+            if ( ! return_str.empty() ) {
                 _freesafe(newname);
-                newname=return_str;
+                newname = dupstring( return_str.c_str() );
             }
             if(erg==1) {
                 skip=true;
@@ -335,7 +342,7 @@ CopyCore::nm_newname_t CopyCore::getNewName4File( const FileEntry *oldfe, const
     char *oldname=oldfe->name;
     char *newname=dupstring(oldname);
     bool nameok=false,skip=false,cancel=false;
-    char *return_str=NULL,*buttonstr,*textstr,*newdest;
+    char *buttonstr,*textstr,*newdest;
     int erg;
     Datei::d_fe_t lse;
     int round=1;
@@ -368,17 +375,20 @@ CopyCore::nm_newname_t CopyCore::getNewName4File( const FileEntry *oldfe, const
                     catalog.getLocale(8));
             textstr=(char*)_allocsafe(strlen(catalog.getLocale(148))+strlen(oldname)+1);
             sprintf(textstr,catalog.getLocale(148),oldname);
+
+            std::string return_str;
+
             erg = m_copyorder->cowin->string_request( catalog.getLocale( 149 ),
                                                     textstr,
                                                     newname,
                                                     buttonstr,
-                                                    &return_str,
+                                                    return_str,
                                                     Requester::REQUEST_SELECTALL );
             _freesafe(buttonstr);
             _freesafe(textstr);
-            if(return_str!=NULL) {
+            if ( ! return_str.empty() ) {
                 _freesafe(newname);
-                newname=return_str;
+                newname = dupstring( return_str.c_str() );
             }
             if(erg==1) {
                 skip=true;
@@ -625,7 +635,7 @@ CopyCore::copydir( const FileEntry *fe, NM_CopyOp_Dir *cod, bool acceptrename, c
     char *newname=NULL;
     int id1;
     FileEntry *subfe;
-    bool isdir,skip,cancel;
+    bool isdir,skip;
     char *buttonstr,*textstr;
     int erg;
     CopyOpWin *cowin=m_copyorder->cowin;
@@ -633,16 +643,15 @@ CopyCore::copydir( const FileEntry *fe, NM_CopyOp_Dir *cod, bool acceptrename, c
     struct utimbuf utb;
     bool corrupt_entries_skipped = false;
 
-    skip = cancel = false;
+    skip = false;
     if ( cowin != NULL ) {
         // call newfile without destination so the user
         // can see the filename in the copyopwin when he will
         // be asked for the new name
-        cowin->newfile( fe->name, NULL );
-        if ( cowin->redraw() != 0 ) cancel = true;
+        cowin->newfile( fe->name, "" );
     }
 
-    if ( cancel == false ) {
+    if ( m_cancel == false ) {
         e1=getNewName4Dir(fe,destdir,&newname,acceptrename);
         if((e1==NM_NEWNAME_OK)||(e1==NM_NEWNAME_USE)) {
             // build new name
@@ -714,7 +723,7 @@ CopyCore::copydir( const FileEntry *fe, NM_CopyOp_Dir *cod, bool acceptrename, c
                             erg = m_copyorder->cowin->request( catalog.getLocale( 347 ), textstr, buttonstr );
                             _freesafe(buttonstr);
                             _freesafe(textstr);
-                            if(erg==1) cancel=true;
+                            if(erg==1) m_cancel=true;
                             nameok=false;
                             if(cowin!=NULL) cowin->conttimer();
                         }
@@ -733,7 +742,7 @@ CopyCore::copydir( const FileEntry *fe, NM_CopyOp_Dir *cod, bool acceptrename, c
                             } else {
                                 cod->error_counter+=cod1->error_counter+1;
                                 if(ce1==NM_COPY_CANCEL) {
-                                    cancel=true;
+                                    m_cancel=true;
                                     break;
                                 }
                             }
@@ -742,10 +751,13 @@ CopyCore::copydir( const FileEntry *fe, NM_CopyOp_Dir *cod, bool acceptrename, c
                         cod->subdirs->closeEnum(id1);
             
                         // next only if read this dir correctly
-                        if((cod->ok==true)&&(cancel==false)) {
+                        if((cod->ok==true)&&(m_cancel==false)) {
                             for ( Verzeichnis::verz_it subfe_it1 = cod->verz->begin();
-                                  subfe_it1 != cod->verz->end() && cancel == false;
+                                  subfe_it1 != cod->verz->end() && m_cancel == false;
                                   subfe_it1++ ) {
+
+                                checkAskToCancel();
+
                                 subfe = *subfe_it1;
 
                                 if ( subfe->isCorrupt == true && subfe->isLink == false ) {
@@ -757,7 +769,7 @@ CopyCore::copydir( const FileEntry *fe, NM_CopyOp_Dir *cod, bool acceptrename, c
                                     erg = m_copyorder->cowin->request( catalog.getLocale( 347 ), text.c_str(), button.c_str() );
                                     if ( erg == 0 ) {
                                     } else {
-                                        cancel = true;
+                                        m_cancel = true;
                                     }
                     
                                     if ( cowin != NULL ) cowin->conttimer();
@@ -780,7 +792,7 @@ CopyCore::copydir( const FileEntry *fe, NM_CopyOp_Dir *cod, bool acceptrename, c
                                         } else {
                                             cod->error_counter++;
                                             if(cf_res==NM_COPY_CANCEL) {
-                                                cancel=true;
+                                                m_cancel=true;
                                                 break;
                                             }
                                         }
@@ -810,7 +822,7 @@ CopyCore::copydir( const FileEntry *fe, NM_CopyOp_Dir *cod, bool acceptrename, c
         } else {
             switch(e1) {
                 case NM_NEWNAME_CANCEL:
-                    cancel=true;
+                    m_cancel=true;
                 default:
                     skip=true;
                     break;
@@ -828,10 +840,9 @@ CopyCore::copydir( const FileEntry *fe, NM_CopyOp_Dir *cod, bool acceptrename, c
 
     if(cowin!=NULL) {
         cowin->dir_finished();
-        if(cowin->redraw()!=0) cancel=true;
     }
     if(newname!=NULL) _freesafe(newname);
-    if(cancel==true) return NM_COPY_CANCEL;
+    if(m_cancel==true) return NM_COPY_CANCEL;
     else if(skip==true) {
         cod->error_counter++;
         return NM_COPY_SKIP;
@@ -857,7 +868,7 @@ CopyCore::copyfile( const FileEntry *fe, bool acceptrename, const char *destdir,
     char *newfullname;
     Datei::d_fe_t de1;
     char *newname=NULL;
-    bool useregcopy, skip, cancel;
+    bool useregcopy, skip;
     int erg;
     CopyOpWin *cowin=m_copyorder->cowin;
     bool docopy,failedmove=false,opok=false;
@@ -874,13 +885,12 @@ CopyCore::copyfile( const FileEntry *fe, bool acceptrename, const char *destdir,
         bytesToCopy = fe->size();
     }
 
-    skip = cancel = false;
+    skip = false;
     if ( cowin != NULL ) {
         // call newfile without destination so the user
         // can see the filename in the copyopwin when he will
         // be asked for the new name
-        cowin->newfile( fe->name, NULL );
-        if ( cowin->redraw() != 0 ) cancel = true;
+        cowin->newfile( fe->name, "" );
     }
 
     if ( fe->isLink == false && fe->isCorrupt == true ) {
@@ -893,13 +903,13 @@ CopyCore::copyfile( const FileEntry *fe, bool acceptrename, const char *destdir,
         if ( erg == 0 ) {
             skip = true;
         } else {
-            cancel = true;
+            m_cancel = true;
         }
 
         if ( cowin != NULL ) cowin->conttimer();
     }
 
-    if ( cancel == false && skip == false ) {
+    if ( m_cancel == false && skip == false ) {
         e1=getNewName4File(fe,destdir,&newname,acceptrename);
     
         if((e1==NM_NEWNAME_OK)||(e1==NM_NEWNAME_OVERWRITE)) {
@@ -984,7 +994,7 @@ CopyCore::copyfile( const FileEntry *fe, bool acceptrename, const char *destdir,
                                 skip = true;
                             } else {
                                 // everything else (including ERROR) is cancel
-                                cancel = true;
+                                m_cancel = true;
                             }
                         } else {
                             // no regular copy
@@ -1003,7 +1013,7 @@ CopyCore::copyfile( const FileEntry *fe, bool acceptrename, const char *destdir,
                                     if ( erg == 0 ) {
                                         skip = true;
                                     } else {
-                                        cancel = true;
+                                        m_cancel = true;
                                     }
                                     if ( cowin != NULL ) cowin->conttimer();
                                 }
@@ -1021,7 +1031,7 @@ CopyCore::copyfile( const FileEntry *fe, bool acceptrename, const char *destdir,
                                 modeCheck = fe->mode();
                             }
               
-                            if ( ( skip == false ) && ( cancel == false ) ) {
+                            if ( ( skip == false ) && ( m_cancel == false ) ) {
                                 if ( ( S_ISCHR( modeCheck ) ) ||
 #ifdef S_ISSOCK
                                      ( S_ISSOCK( modeCheck ) ) ||
@@ -1045,7 +1055,7 @@ CopyCore::copyfile( const FileEntry *fe, bool acceptrename, const char *destdir,
                                             if ( erg == 0 ) {
                                                 skip = true;
                                             } else {
-                                                cancel = true;
+                                                m_cancel = true;
                                             }
                                             if ( cowin != NULL ) cowin->conttimer();
                                         } else opok = true;
@@ -1058,7 +1068,7 @@ CopyCore::copyfile( const FileEntry *fe, bool acceptrename, const char *destdir,
                                         if ( erg == 0 ) {
                                             skip = true;
                                         } else {
-                                            cancel = true;
+                                            m_cancel = true;
                                         }
                                         if ( cowin != NULL ) cowin->conttimer();
                                     }
@@ -1073,7 +1083,7 @@ CopyCore::copyfile( const FileEntry *fe, bool acceptrename, const char *destdir,
                                         if ( erg == 0 ) {
                                             skip = true;
                                         } else {
-                                            cancel = true;
+                                            m_cancel = true;
                                         }
                                         if ( cowin != NULL ) cowin->conttimer();
                                     } else opok = true;
@@ -1090,7 +1100,7 @@ CopyCore::copyfile( const FileEntry *fe, bool acceptrename, const char *destdir,
                                             if ( erg == 0 ) {
                                                 skip = true;
                                             } else {
-                                                cancel = true;
+                                                m_cancel = true;
                                             }
                                             if ( cowin != NULL ) cowin->conttimer();
                                         } else opok = true;
@@ -1104,7 +1114,7 @@ CopyCore::copyfile( const FileEntry *fe, bool acceptrename, const char *destdir,
                                         if ( erg == 0 ) {
                                             skip = true;
                                         } else {
-                                            cancel = true;
+                                            m_cancel = true;
                                         }
                                         if ( cowin != NULL ) cowin->conttimer();
                                     }
@@ -1117,7 +1127,7 @@ CopyCore::copyfile( const FileEntry *fe, bool acceptrename, const char *destdir,
                                     if ( erg == 0 ) {
                                         skip = true;
                                     } else {
-                                        cancel = true;
+                                        m_cancel = true;
                                     }
                                     if ( cowin != NULL ) cowin->conttimer();
                                 }
@@ -1166,7 +1176,7 @@ CopyCore::copyfile( const FileEntry *fe, bool acceptrename, const char *destdir,
         } else {
             switch(e1) {
                 case NM_NEWNAME_CANCEL:
-                    cancel=true;
+                    m_cancel=true;
                 default:
                     skip=true;
                     break;
@@ -1183,10 +1193,9 @@ CopyCore::copyfile( const FileEntry *fe, bool acceptrename, const char *destdir,
         } else {
             cowin->file_finished();
         }
-        if(cowin->redraw()!=0) cancel=true;
     }
     if(newname!=NULL) _freesafe(newname);
-    if(cancel==true) return NM_COPY_CANCEL;
+    if(m_cancel==true) return NM_COPY_CANCEL;
     else if(skip==true) return NM_COPY_SKIP;
     if(m_copyorder->move==true) {
         if((opok==true)&&(failedmove==true)) {
@@ -1318,6 +1327,9 @@ CopyCore::reg_copy_erg_t CopyCore::copyfile_reg( const char *sourcename,
         return return_value;
     }
 
+    loff_t total_bytes = 0;
+    bool asked_about_source_change = false;
+
     // now copy from fdr to fdw
     do {
         readbytes = worker_read( fdr, buf, BUFSIZE );
@@ -1355,10 +1367,12 @@ CopyCore::reg_copy_erg_t CopyCore::copyfile_reg( const char *sourcename,
 
             if ( writebytes > 0 && cowin != NULL ) {
                 cowin->add_curbytes_copied( writebytes );
+            }
 
-                //TODO if I use dynamic buffer I should skip some updates
-                if ( cowin->redraw() != 0 )
-                    return_value = REG_COPY_CANCEL;
+            checkAskToCancel();
+
+            if ( m_cancel ) {
+                return_value = REG_COPY_CANCEL;
             }
 
             if ( writebytes != bytes_left_to_write ) {
@@ -1398,10 +1412,38 @@ CopyCore::reg_copy_erg_t CopyCore::copyfile_reg( const char *sourcename,
 
             if ( return_value == REG_COPY_CANCEL || return_value == REG_COPY_SKIP ) break;
         }
+
+        total_bytes += bytes_written;
+
+        if ( total_bytes > bytesToCopy ) {
+            // more bytes read than initially known
+            // probably the source file has changed
+            // ask what to do
+
+            if ( ! asked_about_source_change ) {
+
+                text = AGUIXUtils::formatStringToString( catalog.getLocale( 1042 ), sourcename );
+                button = std::string( catalog.getLocale( 629 ) ) + "|" + catalog.getLocale( 225 ) + "|" + catalog.getLocale( 8 );
+                erg = cowin->request( catalog.getLocale( 347 ), text.c_str(), button.c_str() );
+	
+                if ( erg == 0 ) {
+                } else if ( erg == 1 ) {
+                    return_value = REG_COPY_SKIP;
+                } else {
+                    return_value = REG_COPY_CANCEL;
+                }
+
+                asked_about_source_change = true;
+            }
+        }
+
     } while ( ( return_value != REG_COPY_CANCEL ) && ( return_value != REG_COPY_SKIP ) );
 
     cowin->update_file_status();
-    if ( cowin->redraw() != 0 ) return_value = REG_COPY_CANCEL;
+
+    if ( m_cancel ) {
+        return_value = REG_COPY_CANCEL;
+    }
 
     worker_close( fdr );
     if ( worker_close( fdw ) != 0 ) {
@@ -1569,7 +1611,7 @@ int CopyCore::buildCopyDatabase()
 
     cowin->starttimer();
 
-    if ( cowin->redraw() != 0 ) m_cancel = true;
+    if ( cowin->redraw() & 1 ) m_cancel = true;
 
     // create the NM_CopyOp_Dir for each dir in copylist
   
@@ -1610,7 +1652,7 @@ int CopyCore::buildCopyDatabase()
 
             cowin->set_files_to_copy( gf );
             cowin->set_dirs_to_copy( gd );
-            if ( cowin->redraw() != 0 ) m_cancel = true;
+            if ( cowin->redraw() & 1 ) m_cancel = true;
         } else {
             // is not dir (mostly a file but can also be links ...)
             files++;
@@ -1641,9 +1683,16 @@ int CopyCore::buildCopyDatabase()
 
 int CopyCore::executeCopy()
 {
-    CopyOpWin *cowin = m_copyorder->cowin;
+    m_copy_thread = std::thread( [this]() { executeCopyThread(); } );
 
+    return 0;
+}
+
+int CopyCore::executeCopyThread()
+{
     for ( auto &cbe : m_copy_list ) {
+        checkAskToCancel();
+
         if ( m_cancel ) break;
 
         NM_CopyOp_Dir *tcod;
@@ -1651,14 +1700,22 @@ int CopyCore::executeCopy()
         tcod = cbe.m_cod;
 
         if ( m_pre_cb ) {
-            m_pre_cb( cbe.entry(), cbe.m_row, cbe.m_cod );
+            loff_t size = 0;
+
+            if ( cbe.entry().isLink && ! cbe.entry().isCorrupt ) {
+                size = cbe.entry().dsize();
+            } else {
+                size = cbe.entry().size();
+            }
+
+            m_pre_cb( size, cbe.m_row, cbe.m_cod != NULL ? true : false );
         }
 
         if ( tcod != NULL ) {
             nm_copy_t ce1;
             std::string target_fullname;
 
-            ce1 = copydir( &cbe.entry(), tcod, true, m_copyorder->destdir, &target_fullname );
+            ce1 = copydir( &cbe.entry(), tcod, true, m_copyorder->destdir.c_str(), &target_fullname );
             if ( ce1 == NM_COPY_OK || ce1 == NM_COPY_NEED_DELETE ) {
                 // success
             } else if ( ce1 == NM_COPY_CANCEL ) {
@@ -1666,14 +1723,17 @@ int CopyCore::executeCopy()
             }
 
             if ( m_post_cb ) {
-                m_post_cb( cbe.entry(), cbe.m_row,
-                           ce1, cbe.m_cod, target_fullname );
+                m_post_cb( cbe.entry().fullname, cbe.m_row,
+                           ce1,
+                           cbe.m_cod != NULL ? true : false,
+                           cbe.m_cod != NULL ? cbe.m_cod->error_counter : 0,
+                           target_fullname );
             }
         } else {
             nm_copy_t ce1;
             std::string target_fullname;
 
-            ce1 = copyfile( &cbe.entry(), true, m_copyorder->destdir, &target_fullname );
+            ce1 = copyfile( &cbe.entry(), true, m_copyorder->destdir.c_str(), &target_fullname );
 
             if ( ce1 == NM_COPY_OK || ce1 == NM_COPY_NEED_DELETE ) {
                 // success
@@ -1682,24 +1742,28 @@ int CopyCore::executeCopy()
             }
 
             if ( m_post_cb ) {
-                m_post_cb( cbe.entry(), cbe.m_row,
-                           ce1, NULL, target_fullname );
+                m_post_cb( cbe.entry().fullname, cbe.m_row,
+                           ce1,
+                           cbe.m_cod != NULL ? true : false,
+                           cbe.m_cod != NULL ? cbe.m_cod->error_counter : 0,
+                           target_fullname );
             }
         }
-
-        if ( cowin->redraw() != 0 ) m_cancel = true;
     }
 
+    setFinished();
+
     return 0;
 }
 
-void CopyCore::setPreCopyCallback( std::function< void( const FileEntry &fe, int row, const NM_CopyOp_Dir *cod ) > cb )
+void CopyCore::setPreCopyCallback( std::function< void( loff_t size, int row, bool is_dir ) > cb )
 {
     m_pre_cb = cb;
 }
 
-void CopyCore::setPostCopyCallback( std::function< void( const FileEntry &fe, int row,
-                                                         nm_copy_t res, const NM_CopyOp_Dir *cod,
+void CopyCore::setPostCopyCallback( std::function< void( const std::string &source_fullname, int row,
+                                                         nm_copy_t res,
+                                                         bool is_dir, unsigned long dir_error_counter,
                                                          const std::string &target_fullname ) > cb )
 {
     m_post_cb = cb;
@@ -1787,3 +1851,67 @@ int CopyCore::tryApplyOwnerPerm( const char *filename,
     }
     return erg;
 }
+
+std::shared_ptr< struct copyorder > CopyCore::getCopyOrder()
+{
+    return m_copyorder;
+}
+
+bool CopyCore::finished()
+{
+    std::unique_lock< std::mutex> l( m_finished_mutex );
+
+    return m_finished;
+}
+
+void CopyCore::join()
+{
+    m_copy_thread.join();
+}
+
+void CopyCore::checkAskToCancel()
+{
+    if ( m_ask_to_cancel ) {
+        m_ask_to_cancel = false;
+
+        std::string str1;
+        int erg;
+    
+        // user canceled -> request to be sure
+        str1 = catalog.getLocale( 11 );
+        str1 += "|";
+        str1 += catalog.getLocale( 544 );
+        erg = m_copyorder->cowin->request( catalog.getLocale( 123 ),
+                                           catalog.getLocale( 543 ),
+                                           str1.c_str() );
+        if ( erg == 0 ) {
+            m_cancel = true;
+        }
+    }
+}
+
+void CopyCore::setAskToCancel()
+{
+    m_ask_to_cancel = true;
+}
+
+bool CopyCore::timed_wait_for_finished()
+{
+    std::unique_lock< std::mutex> l( m_finished_mutex );
+
+    if ( ! m_finished ) {
+        m_finished_cond.wait_for( l,
+                                  std::chrono::duration<long long, std::milli>( 1000 / 25 ) );
+    }
+
+    return m_finished;
+}
+
+void CopyCore::setFinished()
+{
+    std::unique_lock< std::mutex> l( m_finished_mutex );
+
+    m_finished = true;
+
+    m_finished_cond.notify_all();
+}
