@@ -1,6 +1,6 @@
 /* copyop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2006,2008,2011,2013 Ralf Hoffmann.
+ * Copyright (C) 2001-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -138,8 +138,6 @@ CopyOp::getDescription()
 int
 CopyOp::normalmodecopy( ActionMessage *am )
 {
-  struct copyorder copyorder;
-  NM_specialsourceExt *specialsource=NULL;
   bool do_request,cont=true;
   std::string destination_directory;
   ListerMode *start_lm = NULL, *dest_lm = NULL;
@@ -173,24 +171,24 @@ CopyOp::normalmodecopy( ActionMessage *am )
   }
   
   if(cont==true) {
-    memset( &copyorder, 0, sizeof( copyorder ) );
-    copyorder.ignoreLosedAttr = false;
+    std::shared_ptr< struct copyorder > co;
+
+    co.reset( new copyorder );
+
+    co->ignoreLosedAttr = false;
     // detect what to copy
     if(am->mode==am->AM_MODE_ONLYACTIVE)
-      copyorder.source=copyorder.COPY_ONLYACTIVE;
+      co->source=co->COPY_ONLYACTIVE;
     else if(am->mode==am->AM_MODE_DNDACTION) {
       // insert DND-element into list
-      copyorder.source=copyorder.COPY_SPECIAL;
-      copyorder.sources=new std::list<NM_specialsourceExt*>;
-      specialsource = new NM_specialsourceExt( am->dndmsg->getFE() );
-      copyorder.sources->push_back(specialsource);
+      co->source=co->COPY_SPECIAL;
+      co->sources.push_back( new NM_specialsourceExt( am->dndmsg->getFE() ) );
     } else if(am->mode==am->AM_MODE_SPECIAL) {
-      copyorder.source=copyorder.COPY_SPECIAL;
-      copyorder.sources=new std::list<NM_specialsourceExt*>;
-      specialsource = new NM_specialsourceExt( am->getFE() );
-      copyorder.sources->push_back(specialsource);
-    } else
-      copyorder.source=copyorder.COPY_ALLENTRIES;
+      co->source=co->COPY_SPECIAL;
+      co->sources.push_back( new NM_specialsourceExt( am->getFE() ) );
+    } else {
+      co->source=co->COPY_ALLENTRIES;
+    }
 
     /* now source defined
        next find destination
@@ -280,37 +278,29 @@ CopyOp::normalmodecopy( ActionMessage *am )
         }
         /* now also destination determined
            next describing what to do */
-        copyorder.move=tmove;
-        copyorder.follow_symlinks=tfollow_symlinks;
-        copyorder.preserve_attr=tpreserve_attr;
-        copyorder.do_rename=trename;
+        co->move=tmove;
+        co->follow_symlinks=tfollow_symlinks;
+        co->preserve_attr=tpreserve_attr;
+        co->do_rename=trename;
         switch(toverwrite) {
             case COPYOP_OVERWRITE_ALWAYS:
-                copyorder.overwrite=copyorder.COPY_OVERWRITE_ALWAYS;
+                co->overwrite=co->COPY_OVERWRITE_ALWAYS;
                 break;
             case COPYOP_OVERWRITE_NEVER:
-                copyorder.overwrite=copyorder.COPY_OVERWRITE_NEVER;
+                co->overwrite=co->COPY_OVERWRITE_NEVER;
                 break;
             default:
-                copyorder.overwrite=copyorder.COPY_OVERWRITE_NORMAL;
+                co->overwrite=co->COPY_OVERWRITE_NORMAL;
         }
-        copyorder.destdir = destination_directory.c_str();
+        co->destdir = destination_directory.c_str();
         // now start copy process
-        copyorder.cowin=new CopyOpWin( am->getWorker()->getAGUIX(),
-                                       copyorder.move );
+        co->cowin=new CopyOpWin( am->getWorker()->getAGUIX(),
+                                       co->move );
 
         if ( VirtualDirMode *vdm = dynamic_cast< VirtualDirMode * >( start_lm ) ) {
-            vdm->copy( &copyorder );
+            vdm->copy( co );
         }
-
-        delete copyorder.cowin;
     }
-
-    if(copyorder.source==copyorder.COPY_SPECIAL) {
-      if ( specialsource != NULL ) delete specialsource;
-      delete copyorder.sources;
-    }
-
   }
   return 0;
 }
