@@ -1,6 +1,6 @@
 /* copyopwin.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2006,2010-2011,2013 Ralf Hoffmann.
+ * Copyright (C) 2001-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -27,16 +27,16 @@
 #include <aguix/button.h>
 #include <aguix/bevelbox.h>
 #include <aguix/solidbutton.h>
+#include <aguix/choosebutton.h>
 #include "worker_locale.h"
 #include "datei.h"
+#include "pers_kvp.hh"
 
 #define HUMAN_READABLE_DIALOG 1
 
 CopyOpWin::CopyOpWin( AGUIX*taguix, bool _move )
 {
   this->aguix=taguix;
-  source=dupstring("");
-  dest=dupstring("");
   copied_bytes_curfile=0;
   bytes_curfile=0;
   copied_bytes=0;
@@ -76,12 +76,27 @@ CopyOpWin::CopyOpWin( AGUIX*taguix, bool _move )
   
   gettimeofday( &last_update_time, NULL );
   skipped_updates = 0;
+
+  m_master_thread_id = std::this_thread::get_id();
+
+  m_finished = false;
+  m_detached = false;
+
+  int v = Worker::getKVPStore().getIntValue( "copy-keep-window" );
+
+  // 0 is default for non-existing entry, 2 means enabled
+  if ( v == 0 ||
+       v == 2 ) {
+      m_keep_window = true;
+  } else {
+      m_keep_window = false;
+  }
+
+  m_keep_window_cb = nullptr;
 }
 
 CopyOpWin::~CopyOpWin()
 {
-  if(source!=NULL) _freesafe(source);
-  if(dest!=NULL) _freesafe(dest);
   if(window!=NULL) close();
   delete lencalc;
 }
@@ -100,7 +115,7 @@ CopyOpWin::open()
 			10, 10,
 			w, h,
 			0, ( move == true ) ? catalog.getLocale( 299 ) : catalog.getLocaleCom( 8 ),
-                        AWindow::AWINDOW_DIALOG );
+                        AWindow::AWINDOW_NORMAL );
   window->create();
   tx=5;
   ty=5;
@@ -155,8 +170,18 @@ CopyOpWin::open()
   tx=5;
   bb4=(BevelBox*)window->add(new BevelBox(aguix,tx,ty,w-2*tx,2,0));
   ty+=bb4->getHeight()+5;
+
+  m_detach_b = (Button*)window->add( new Button( aguix, tx, ty, catalog.getLocale( 1039 ), 1, 0, 0 ) );
+
+  m_keep_window_cb = (ChooseButton*)window->add( new ChooseButton( aguix, tx, ty,
+                                                                   m_keep_window,
+                                                                   catalog.getLocale( 1047 ),
+                                                                   LABEL_RIGHT, 1, 0 ) );
+  m_keep_window_cb->hide();
+
   cb = (Button*)window->add( new Button( aguix, tx, ty, catalog.getLocale( 8 ), 1, 0, 0 ) );
-  cb->move(w/2-cb->getWidth()/2,cb->getY());
+  cb->move( w - 5 - cb->getWidth(), cb->getY() );
+
   ty+=cb->getHeight()+5;
   h=ty;
   window->setDoTabCycling( true );
@@ -186,13 +211,17 @@ CopyOpWin::close()
 void
 CopyOpWin::starttimer()
 {
-  gettimeofday( &starttime, NULL );
-  lasttime_global = starttime;
+    std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
+
+    gettimeofday( &starttime, NULL );
+    lasttime_global = starttime;
 }
 
 void
 CopyOpWin::set_files_to_copy(long nfiles)
 {
+    std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
+
   if(nfiles>0) files=nfiles;
   else files=0;
   update_files2gotext=true;
@@ -201,6 +230,8 @@ CopyOpWin::set_files_to_copy(long nfiles)
 void
 CopyOpWin::set_dirs_to_copy(long ndirs)
 {
+    std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
+
   if(ndirs>0) dirs=ndirs;
   else dirs=0;
   update_dirs2gotext=true;
@@ -209,6 +240,8 @@ CopyOpWin::set_dirs_to_copy(long ndirs)
 void
 CopyOpWin::set_bytes_to_copy(loff_t nbytes)
 {
+    std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
+
   if(nbytes>0) bytes=nbytes;
   else bytes=0;
   update_globtext=true;
@@ -217,6 +250,8 @@ CopyOpWin::set_bytes_to_copy(loff_t nbytes)
 void
 CopyOpWin::set_bytes_to_copy_curfile( loff_t nbytes )
 {
+    std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
+
   if ( nbytes > 0 ) bytes_curfile = nbytes;
   else bytes_curfile = 0;
   update_filetext = true;
@@ -225,6 +260,8 @@ CopyOpWin::set_bytes_to_copy_curfile( loff_t nbytes )
 void
 CopyOpWin::dir_finished()
 {
+    std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
+
   copied_dirs++;
   update_dirs2gotext=true;
 }
@@ -232,6 +269,8 @@ CopyOpWin::dir_finished()
 void
 CopyOpWin::file_finished()
 {
+    std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
+
   // inc counter
   copied_files++;
   update_files2gotext=true;
@@ -244,6 +283,8 @@ CopyOpWin::file_finished()
 void
 CopyOpWin::add_curbytes_copied( loff_t nbytes )
 {
+    std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
+
   if ( nbytes > 0 ) {
     copied_bytes_curfile += nbytes;
     copied_bytes += nbytes;
@@ -262,18 +303,15 @@ CopyOpWin::add_curbytes_copied( loff_t nbytes )
 }
 
 void
-CopyOpWin::newfile(char *name,char *ndest)
+CopyOpWin::newfile( const std::string &name,
+                    const std::string &ndest )
 {
-  if(source!=NULL) _freesafe(source);
-  if(name!=NULL)
-    source=dupstring(name);
-  else
-    source=dupstring("");
-  if(dest!=NULL) _freesafe(dest);
-  if(ndest!=NULL)
-    dest=dupstring(ndest);
-  else
-    dest=dupstring("");
+    std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
+
+    source = name;
+
+    dest = ndest;
+
   gettimeofday( &filestarttime, NULL );
   update_sourcetext=true;
   update_desttext=true;
@@ -288,7 +326,7 @@ CopyOpWin::newfile(char *name,char *ndest)
 }
 
 int
-CopyOpWin::redraw()
+CopyOpWin::redraw( bool skip_message_handling )
 { /* this functions will redraw the window AND process events for this window
    * returns values:
    *  0 for regular exit
@@ -312,40 +350,40 @@ CopyOpWin::redraw()
   
   char *tstr3;
 
+    std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
+
   timerclear( &curtime );
-  if ( ( update_sourcetext == true ) && ( ismessage[0] == false ) ) {
-    if ( source != NULL ) {
-      tstr3 = Datei::shrinkFilename( source, filenamespace[0], *lencalc );
-      if ( tstr3 == NULL)
-        tstr3 = dupstring( source );
-      tstr2 = (char*)_allocsafe( strlen( catalog.getLocale( ( move == true ) ? 594 : 116 ) ) + strlen( tstr3 ) + 1 );
-      sprintf( tstr2, catalog.getLocale( ( move == true ) ? 594 : 116 ), tstr3 );
-      _freesafe( tstr3 );
-    } else {
-      tstr2 = (char*)_allocsafe( strlen( catalog.getLocale( ( move == true ) ? 594 : 116 ) ) + 1 );
-      sprintf( tstr2, catalog.getLocale( ( move == true ) ? 594 : 116 ), "" );
-    }
-    sourcetext->setText(tstr2);
-    _freesafe(tstr2);
-    /* no need for explicit redraw */
-    update_sourcetext=false;
+  if ( update_sourcetext == true ) {
+      if ( ismessage[0] == false ) {
+          tstr3 = Datei::shrinkFilename( source.c_str(), filenamespace[0], *lencalc );
+          if ( tstr3 == NULL)
+              tstr3 = dupstring( source.c_str() );
+          tstr2 = (char*)_allocsafe( strlen( catalog.getLocale( ( move == true ) ? 594 : 116 ) ) + strlen( tstr3 ) + 1 );
+          sprintf( tstr2, catalog.getLocale( ( move == true ) ? 594 : 116 ), tstr3 );
+          _freesafe( tstr3 );
+          sourcetext->setText(tstr2);
+          _freesafe(tstr2);
+          /* no need for explicit redraw */
+      } else {
+          sourcetext->setText( source.c_str() );
+      }
+      update_sourcetext = false;
   }
-  if ( ( update_desttext == true ) && ( ismessage[1] == false ) ) {
-    if ( dest != NULL ) {
-      tstr3 = Datei::shrinkFilename( dest, filenamespace[1], *lencalc );
-      if ( tstr3 == NULL)
-        tstr3 = dupstring( dest );
-      tstr2 = (char*)_allocsafe( strlen( catalog.getLocale( 117 ) ) + strlen( tstr3 ) + 1 );
-      sprintf( tstr2, catalog.getLocale( 117 ), tstr3 );
-      _freesafe(tstr3);
-    } else {
-      tstr2 = (char*)_allocsafe( strlen( catalog.getLocale( 117 ) ) + 1 );
-      sprintf(tstr2,catalog.getLocale(117),"");
-    }
-    desttext->setText(tstr2);
-    _freesafe(tstr2);
-    // no need for explicit redraw
-    update_desttext=false;
+  if ( update_desttext == true ) {
+      if ( ismessage[1] == false ) {
+          tstr3 = Datei::shrinkFilename( dest.c_str(), filenamespace[1], *lencalc );
+          if ( tstr3 == NULL)
+              tstr3 = dupstring( dest.c_str() );
+          tstr2 = (char*)_allocsafe( strlen( catalog.getLocale( 117 ) ) + strlen( tstr3 ) + 1 );
+          sprintf( tstr2, catalog.getLocale( 117 ), tstr3 );
+          _freesafe(tstr3);
+          desttext->setText(tstr2);
+          _freesafe(tstr2);
+          // no need for explicit redraw
+      } else {
+          desttext->setText( dest.c_str() );
+      }
+      update_desttext = false;
   }
   if(update_filetext==true) {
     if ( ! timerisset( &curtime ) ) gettimeofday( &curtime, NULL );
@@ -578,29 +616,74 @@ CopyOpWin::redraw()
 
   //window->redraw();
   aguix->Flush();
-  /* update complete
-     now check for X-messages */
-  AGMessage *msg;
-  do {
-    msg=aguix->GetMessage( window );
+
+  if ( ! skip_message_handling ) {
+      /* update complete
+         now check for X-messages */
+      AGMessage *msg;
+      do {
+          msg=aguix->GetMessage( window );
+
+          returnvalue |= handleMessage( msg );
+
+          aguix->ReplyMessage(msg);
+      } while(msg!=NULL);
+  }
+  
+  return returnvalue;
+}
+
+int CopyOpWin::handleMessage( AGMessage *msg )
+{
+    int returnvalue=0;
+
     if(msg!=NULL) {
       switch(msg->type) {
         case AG_CLOSEWINDOW:
-          if(msg->closewindow.window==window->getWindow()) returnvalue=1;
+            if ( msg->closewindow.window == window->getWindow() ) {
+                if ( m_finished ) {
+                    returnvalue = 4;
+                } else {
+                    returnvalue = 1;
+                }
+            }
           break;
         case AG_BUTTONCLICKED:
-          if(msg->button.button==cb) returnvalue=1;
+            if ( msg->button.button == cb ) {
+                if ( m_finished ) {
+                    returnvalue = 4;
+                } else {
+                    returnvalue = 1;
+                }
+            } else if ( msg->button.button == m_detach_b ) {
+                returnvalue = 2;
+            }
           break;
+        case AG_CHOOSECLICKED:
+            if ( msg->choose.button == m_keep_window_cb ) {
+                m_keep_window = m_keep_window_cb->getState();
+
+                Worker::getKVPStore().setIntValue( "copy-keep-window",
+                                                   ( m_keep_window ? 2 : 1 ) );
+            }
+            break;
         case AG_SIZECHANGED:
           if ( msg->size.window == window->getWindow() ) {
+            std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
+
             bb1->resize( msg->size.neww - 2 * window->getBorderWidth(), bb1->getHeight() );
             sbw = bb1->getWidth() - 2;
             bb2->resize( msg->size.neww - 2 * window->getBorderWidth(), bb2->getHeight() );
             bb3->resize( msg->size.neww - 2 * window->getBorderWidth(), bb3->getHeight() );
             bb4->resize( msg->size.neww - 2 * window->getBorderWidth(), bb4->getHeight() );
-            cb->move( msg->size.neww / 2 - cb->getWidth() / 2,
+            cb->move( msg->size.neww -5 - cb->getWidth(),
                       msg->size.newh - window->getBorderWidth() - cb->getHeight() );
-            
+
+            m_detach_b->move( 5,
+                              msg->size.newh - window->getBorderWidth() - m_detach_b->getHeight() );
+            m_keep_window_cb->move( 5,
+                                    msg->size.newh - window->getBorderWidth() - m_keep_window_cb->getHeight() );
+
             filenamespace[0] = msg->size.neww - window->getBorderWidth() - sourcetext->getX();
             filenamespace[0] -= aguix->getTextWidth( catalog.getLocale( ( move == true ) ? 594 : 116 ) ) - 2 * aguix->getTextWidth( " " );
             filenamespace[1] = msg->size.neww - window->getBorderWidth() - desttext->getX();
@@ -611,60 +694,56 @@ CopyOpWin::redraw()
             // now gsb and fsb are not scaled for new width
             // but next call of redraw they will so this shouldn't hurt anybody
             // but a solution would be to first react for messages and then redraw
+
+            if ( m_finished ) {
+                double rate = 0.0;
+
+                if ( bytes_curfile > 0 ) {
+                    rate = (double)copied_bytes_curfile / (double)bytes_curfile;
+                }
+
+                fsb->resize( w_max( (int)( sbw * rate ), 2 ), fsb->getHeight() );
+
+                if ( bytes > 0 ) {
+                    rate = (double)copied_bytes / (double)bytes;
+                } else {
+                    rate=0.0;
+                }
+
+                gsb->resize( w_max( (int)( sbw * rate ), 2 ), gsb->getHeight() );
+            }
           }
           break;
       }
-      aguix->ReplyMessage(msg);
     }
-  } while(msg!=NULL);
-  
-  if ( returnvalue == 1 ) {
-    std::string str1;
-    int erg;
-    
-    stoptimer();
-
-    // user canceled -> request to be sure
-    str1 = catalog.getLocale( 11 );
-    str1 += "|";
-    str1 += catalog.getLocale( 544 );
-    erg = window->request( catalog.getLocale( 123 ),
-                           catalog.getLocale( 543 ),
-                           str1.c_str() );
-    if ( erg != 0 ) returnvalue = 0;
-
-    conttimer();
-  }
   
   return returnvalue;
 }
 
 void
-CopyOpWin::setmessage( const char *msg, int line )
+CopyOpWin::setmessage( const std::string &msg, int line )
 {
-  switch(line) {
-    case 0:
-      if(msg!=NULL) sourcetext->setText(msg);
-      else sourcetext->setText("");
-      ismessage[0] = true;
-      break;
-    case 1:
-      if(msg!=NULL) desttext->setText(msg);
-      else desttext->setText("");
-      ismessage[1] = true;
-      break;
-  }
-}
+    std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
 
-void
-CopyOpWin::forceredraw()
-{
-  window->redraw();
+    switch(line) {
+        case 0:
+            source = msg;
+            ismessage[0] = true;
+            update_sourcetext = true;
+            break;
+        case 1:
+            dest = msg;
+            ismessage[1] = true;
+            update_desttext = true;
+            break;
+    }
 }
 
 void
 CopyOpWin::dec_file_counter(unsigned long f)
 {
+    std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
+
   files-=f;
   update_files2gotext=true;
 }
@@ -672,6 +751,8 @@ CopyOpWin::dec_file_counter(unsigned long f)
 void
 CopyOpWin::dec_dir_counter(unsigned long d)
 {
+    std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
+
   dirs-=d;
   update_dirs2gotext=true;
 }
@@ -679,6 +760,8 @@ CopyOpWin::dec_dir_counter(unsigned long d)
 void
 CopyOpWin::dec_byte_counter(loff_t b)
 {
+    std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
+
   bytes-=b;
   update_globtext=true;
 }
@@ -686,6 +769,8 @@ CopyOpWin::dec_byte_counter(loff_t b)
 void
 CopyOpWin::stoptimer()
 {
+    std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
+
   gettimeofday( &stoptime, NULL );
   timer_stopped = true;
 }
@@ -696,6 +781,8 @@ CopyOpWin::conttimer()
   struct timeval curtime;
   long s, us;
 
+    std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
+
   if ( timer_stopped == false ) return;
   
   gettimeofday( &curtime, NULL );
@@ -728,10 +815,10 @@ CopyOpWin::conttimer()
   }
 }
 
-int CopyOpWin::request( const char *title,
-                        const char *text,
-                        const char *buttons,
-                        Requester::request_flags_t flags )
+int CopyOpWin::request_int( const char *title,
+                            const char *text,
+                            const char *buttons,
+                            Requester::request_flags_t flags )
 {
   if ( window != NULL ) {
     return window->request( title, text, buttons, flags );
@@ -740,12 +827,12 @@ int CopyOpWin::request( const char *title,
   }
 }
 
-int CopyOpWin::string_request( const char *title,
-                               const char *lines,
-                               const char *default_str,
-                               const char *buttons,
-                               char **return_str,
-                               Requester::request_flags_t flags )
+int CopyOpWin::string_request_int( const char *title,
+                                   const char *lines,
+                                   const char *default_str,
+                                   const char *buttons,
+                                   char **return_str,
+                                   Requester::request_flags_t flags )
 {
   if ( window != NULL ) {
     return window->string_request( title, lines, default_str, buttons, return_str, flags );
@@ -756,6 +843,8 @@ int CopyOpWin::string_request( const char *title,
 
 void CopyOpWin::file_skipped( loff_t byteSum, bool subCopied )
 {
+    std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
+
   if ( files > 0 ) files--;
   if ( byteSum > 0 ) bytes -= byteSum;
   if ( subCopied == true ) copied_bytes -= copied_bytes_curfile;
@@ -765,8 +854,154 @@ void CopyOpWin::file_skipped( loff_t byteSum, bool subCopied )
 
 void CopyOpWin::update_file_status()
 {
+    std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
+
   if ( skipped_updates > 0 ) {
     update_filetext = true;
     update_globtext = true;
   }
 }
+
+void CopyOpWin::master_process_request()
+{
+    std::unique_lock<std::mutex> lck( m_request_lock );
+
+    if ( m_request ) {
+        process_request( *m_request );
+
+        std::unique_lock<std::mutex> lck( m_response_lock );
+
+        m_response = std::move( m_request );
+
+        m_response_cond.notify_one();
+    }
+}
+
+void CopyOpWin::push_request( const BGCopyRequestMessage &msg )
+{
+    std::unique_lock<std::mutex> lck( m_request_lock );
+
+    m_request = std::unique_ptr< BGCopyRequestMessage >( new BGCopyRequestMessage( msg ) );
+}
+
+BGCopyRequestMessage CopyOpWin::wait_for_response()
+{
+    std::unique_lock<std::mutex> lck( m_response_lock );
+
+    while ( ! m_response ) {
+        m_response_cond.wait( lck );
+    }
+
+    BGCopyRequestMessage res = *m_response;
+
+    m_response.reset();
+
+    return res;
+}
+
+int CopyOpWin::request( const std::string &title,
+                        const std::string &text,
+                        const std::string &buttons,
+                        Requester::request_flags_t flags )
+{
+    BGCopyRequestMessage msg( title, text, buttons );
+
+    if ( std::this_thread::get_id() == m_master_thread_id ) {
+        process_request( msg );
+
+        return msg.getRes();
+    } else {
+        push_request( msg );
+
+        return wait_for_response().getRes();
+    }
+}
+
+int CopyOpWin::string_request( const std::string &title,
+                               const std::string &lines,
+                               const std::string &default_str,
+                               const std::string &buttons,
+                               std::string &return_str,
+                               Requester::request_flags_t flags )
+{
+    BGCopyRequestMessage msg( title, lines, default_str, buttons, flags );
+
+    if ( std::this_thread::get_id() == m_master_thread_id ) {
+        process_request( msg );
+
+        return_str = msg.getResultString();
+
+        return msg.getRes();
+    } else {
+        push_request( msg );
+
+        auto res = wait_for_response();
+
+        return_str = res.getResultString();
+
+        return res.getRes();
+    }
+}
+
+void CopyOpWin::process_request( BGCopyRequestMessage &msg )
+{
+    if ( msg.getType() == BGCopyRequestMessage::REGULAR_REQUEST ) {
+        msg.setRes( request_int( msg.getTitle().c_str(),
+                                 msg.getText().c_str(),
+                                 msg.getButtons().c_str() ) );
+    } else if ( msg.getType() == BGCopyRequestMessage::STRING_REQUEST ) {
+        char *result_string = NULL;
+        int res = string_request_int( msg.getTitle().c_str(),
+                                      msg.getText().c_str(),
+                                      msg.getDefaultString().c_str(),
+                                      msg.getButtons().c_str(),
+                                      &result_string,
+                                      msg.getFlags() );
+
+        if ( result_string ) {
+            msg.setResultString( result_string );
+
+            _freesafe( result_string );
+        }
+        msg.setRes( res );
+    } else {
+        throw 1;
+    }
+}
+
+void CopyOpWin::hide_detach_button()
+{
+    m_detach_b->hide();
+
+    m_keep_window_cb->show();
+
+    m_detached = true;
+}
+
+bool CopyOpWin::getKeepWindow() const
+{
+    if ( ! m_detached ) return false;
+
+    return m_keep_window;
+}
+
+void CopyOpWin::setFinished()
+{
+    m_finished = true;
+
+    if ( getKeepWindow() ) {
+        cb->setText( 0, catalog.getLocale( 633 ) );
+
+        int tw = cb->getMaximumWidth();
+
+        if ( tw > cb->getWidth() ) {
+            cb->resize( tw, cb->getHeight() );
+            cb->move( window->getWidth() - 5 - cb->getWidth(), cb->getY() );
+        }
+
+        setmessage( catalog.getLocale( 1046 ), 0 );
+        setmessage( "", 1 );
+
+        redraw();
+    }
+}
