@@ -1,6 +1,6 @@
 /* createsymlinkop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2004,2008,2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2004,2008,2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,7 +21,6 @@
 
 #include "createsymlinkop.h"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "nmspecialsourceext.hh"
 #include "datei.h"
@@ -29,6 +28,8 @@
 #include "worker_locale.h"
 #include <aguix/button.h>
 #include <aguix/choosebutton.h>
+#include "createsymlinkorder.hh"
+#include "virtualdirmode.hh"
 
 const char *CreateSymlinkOp::name="CreateSymlinkOp";
 
@@ -77,9 +78,13 @@ CreateSymlinkOp::run( WPUContext *wpu, ActionMessage *msg )
       startlister=l1;
       endlister = msg->getWorker()->getOtherLister(startlister);
       lm1=startlister->getActiveMode();
-      if(lm1!=NULL)
-        if(lm1->isType("NormalMode")==true)
-          normalmodecreatesl( msg );
+      if(lm1!=NULL) {
+          if(dynamic_cast< VirtualDirMode *>( lm1 ) ) {
+              normalmodecreatesl( msg );
+          } else {
+              lm1->not_supported();
+          }
+      }
     }
   } else normalmodecreatesl( msg );
   return 0;
@@ -104,100 +109,97 @@ CreateSymlinkOp::getDescription()
 int
 CreateSymlinkOp::normalmodecreatesl( ActionMessage *am )
 {
-  NormalMode *nm1=NULL,
-             *nm2=NULL;
-  ListerMode *lm1=NULL;
-  NM_specialsourceExt *specialsource=NULL;
-  bool cont=true;
-  char *dest=NULL;
-  struct NM_createsymlinkorder cslorder;
-  bool takesamedir;
+    ListerMode *lm1 = NULL, *lm2 = NULL;
+    NM_specialsourceExt *specialsource = NULL;
+    bool cont = true;
+    char *dest = NULL;
+    struct NM_createsymlinkorder cslorder;
+    bool takesamedir;
   
-  if(am->mode==am->AM_MODE_DNDACTION) {
-    lm1 = am->dndmsg->getSourceMode();
-    if(lm1==NULL) return 1;
-    if(lm1->isType("NormalMode")!=true) return 1;
-    nm1=(NormalMode*)lm1;
-    lm1 = am->dndmsg->getDestMode();
-    if(lm1!=NULL)
-      if(lm1->isType("NormalMode")==true)
-        nm2=(NormalMode*)lm1;
-  } else {
-    if(startlister==NULL) return 1;
-    lm1=startlister->getActiveMode();
-    if(lm1==NULL) return 1;
-    if(lm1->isType("NormalMode")!=true) return 1;
-    nm1=(NormalMode*)lm1;
-    if(endlister!=NULL) {
-      lm1=endlister->getActiveMode();
-      if(lm1!=NULL)
-        if(lm1->isType("NormalMode")==true)
-          nm2=(NormalMode*)lm1;
-    }
-  }
+    if ( am->mode == am->AM_MODE_DNDACTION ) {
+        lm1 = am->dndmsg->getSourceMode();
+        if ( lm1 == NULL ) return 1;
 
-  if(requestflags==true) {
-    if(doconfigure(1)!=0) cont=false;
-  } else {
-    // set values in t* variables
-    tsame_dir=same_dir;
-    tlocal=local;
-  }
-  
-  if(cont==true) {
-    memset( &cslorder, 0, sizeof( cslorder ) );
-    // detect what to copy
-    if(am->mode==am->AM_MODE_ONLYACTIVE)
-      cslorder.source=cslorder.NM_ONLYACTIVE;
-    else if(am->mode==am->AM_MODE_DNDACTION) {
-      // insert DND-element into list
-      cslorder.source=cslorder.NM_SPECIAL;
-      cslorder.sources=new std::list<NM_specialsourceExt*>;
-      specialsource = new NM_specialsourceExt( am->dndmsg->getFE() );
-      cslorder.sources->push_back(specialsource);
-    } else if(am->mode==am->AM_MODE_SPECIAL) {
-      cslorder.source=cslorder.NM_SPECIAL;
-      cslorder.sources=new std::list<NM_specialsourceExt*>;
-      specialsource = new NM_specialsourceExt( am->getFE() );
-      cslorder.sources->push_back(specialsource);
-    } else
-      cslorder.source=cslorder.NM_ALLENTRIES;
+        lm2 = am->dndmsg->getDestMode();
+    } else {
+        if ( startlister == NULL ) return 1;
+        lm1 = startlister->getActiveMode();
+        if ( lm1 == NULL ) return 1;
 
-    takesamedir=true;
-    if(am->mode==am->AM_MODE_DNDACTION) {
-      const char *ddstr = am->dndmsg->getDestDir();
-      if(ddstr!=NULL) dest=dupstring(ddstr);
-      else dest=NULL;
-      takesamedir=false;
-    } else if(tsame_dir==false) {
-      if(nm2!=NULL) {
-        const char *tstr = nm2->getCurrentDir();
-        if(tstr!=NULL) {
-          dest=dupstring(tstr);
-          takesamedir=false;
+        if ( endlister != NULL ) {
+            lm2 = endlister->getActiveMode();
         }
-      }
     }
 
-    if(takesamedir==true) {
-      const char *tstr = nm1->getCurrentDir();
-      if(tstr!=NULL)
-        dest=dupstring(tstr);
+    if ( requestflags == true ) {
+        if ( doconfigure( 1 ) != 0 ) cont = false;
+    } else {
+        // set values in t* variables
+        tsame_dir = same_dir;
+        tlocal = local;
     }
+  
+    if ( cont == true ) {
+        memset( &cslorder, 0, sizeof( cslorder ) );
+        // detect what to copy
+        if ( am->mode==am->AM_MODE_ONLYACTIVE ) {
+            cslorder.source = cslorder.NM_ONLYACTIVE;
+        } else if ( am->mode == am->AM_MODE_DNDACTION ) {
+            // insert DND-element into list
+            cslorder.source = cslorder.NM_SPECIAL;
+            cslorder.sources = new std::list<NM_specialsourceExt*>;
+            specialsource = new NM_specialsourceExt( am->dndmsg->getFE() );
+            cslorder.sources->push_back( specialsource );
+        } else if ( am->mode == am->AM_MODE_SPECIAL ) {
+            cslorder.source = cslorder.NM_SPECIAL;
+            cslorder.sources = new std::list<NM_specialsourceExt*>;
+            specialsource = new NM_specialsourceExt( am->getFE() );
+            cslorder.sources->push_back( specialsource );
+        } else {
+            cslorder.source = cslorder.NM_ALLENTRIES;
+        }
+
+        takesamedir = true;
+        if ( am->mode == am->AM_MODE_DNDACTION ) {
+            std::string ddstr = am->dndmsg->getDestDir();
+            if ( ! ddstr.empty() ) dest = dupstring( ddstr.c_str() );
+            else dest = NULL;
+
+            takesamedir = false;
+        } else if ( tsame_dir == false ) {
+            if ( lm2 != NULL ) {
+                std::string tstr = lm2->getCurrentDirectory();
+                if ( ! tstr.empty() ) {
+                    dest = dupstring( tstr.c_str() );
+                    takesamedir = false;
+                }
+            }
+        }
+
+        if ( takesamedir == true ) {
+            std::string tstr = lm1->getCurrentDirectory();
+            if ( ! tstr.empty() ) {
+                dest = dupstring( tstr.c_str() );
+            }
+        }
     
-    if(dest!=NULL) {
-      cslorder.local=tlocal;
-      cslorder.destdir=dest;
-      // now start copy process
-      nm1->createsymlink(&cslorder);
+        if ( dest != NULL ) {
+            cslorder.local = tlocal;
+            cslorder.destdir = dest;
+
+            if ( auto vdm = dynamic_cast< VirtualDirMode * >( lm1 ) ) {
+                vdm->createsymlink( &cslorder );
+            }
+        }
+
+        _freesafe( dest );
+
+        if ( cslorder.source == cslorder.NM_SPECIAL ) {
+            if ( specialsource != NULL ) delete specialsource;
+            delete cslorder.sources;
+        }
     }
-    if(dest!=NULL) _freesafe(dest);
-    if(cslorder.source==cslorder.NM_SPECIAL) {
-      if ( specialsource != NULL ) delete specialsource;
-      delete cslorder.sources;
-    }
-  }
-  return 0;
+    return 0;
 }
 
 int
@@ -227,7 +229,7 @@ CreateSymlinkOp::doconfigure(int mode)
 
   tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(36))+1);
   sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(36));
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe(tstr);
 
