@@ -1,6 +1,6 @@
 /* datei.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2007,2009-2010,2012 Ralf Hoffmann.
+ * Copyright (C) 2001-2007,2009-2010,2012-2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -27,6 +27,7 @@
 #include <aguix/utf8.hh>
 #include "nwc_os.hh"
 #include "nwc_path.hh"
+#include "worker.h"
 
 Datei::Datei()
 {
@@ -74,6 +75,9 @@ int Datei::open( const char *name, const char *mode )
   } else {
     return 1;
   }
+
+  p = p & ~Worker::getUMask();
+
   fd = ::worker_open( name, m, p );
   if ( fd < 0 ) return 1;
   iseof = false;
