@@ -1,6 +1,6 @@
 /* deleteop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2004,2008,2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2004,2008,2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -20,7 +20,6 @@
  */
 
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "deleteop.h"
 #include "nmspecialsourceext.hh"
@@ -29,6 +28,8 @@
 #include <aguix/choosebutton.h>
 #include <aguix/solidbutton.h>
 #include "datei.h"
+#include "deleteorder.hh"
+#include "virtualdirmode.hh"
 
 const char *DeleteOp::name="DeleteOp";
 
@@ -72,9 +73,13 @@ DeleteOp::run( WPUContext *wpu, ActionMessage *msg )
     if(l1!=NULL) {
       startlister=l1;
       lm1=startlister->getActiveMode();
-      if(lm1!=NULL)
-        if(lm1->isType("NormalMode")==true)
-          normalmodedelete( msg );
+      if(lm1!=NULL) {
+          if(dynamic_cast< VirtualDirMode * >( lm1 ) ) {
+              normalmodedelete( msg );
+          } else {
+              lm1->not_supported();
+          }
+      }
     }
   }
   return 0;
@@ -89,36 +94,43 @@ DeleteOp::getDescription()
 int
 DeleteOp::normalmodedelete( ActionMessage *am )
 {
-  struct NM_deleteorder delorder;
-  NormalMode *nm1=NULL;
-  ListerMode *lm1=NULL;
-  NM_specialsourceExt *specialsource=NULL;
+    struct deleteorder delorder;
+    ListerMode *lm1 = NULL;
+    VirtualDirMode *vdm = nullptr;
+    NM_specialsourceExt *specialsource = NULL;
   
-  if(startlister==NULL) return 1;
-  lm1=startlister->getActiveMode();
-  if(lm1==NULL) return 1;
-  if(lm1->isType("NormalMode")!=true) return 1;
-  nm1=(NormalMode*)lm1;
-  
-  if(am->mode==am->AM_MODE_NORMAL) {
-    memset( &delorder, 0, sizeof( delorder ) );
-    if ( also_active == true ) {
-      delorder.source=delorder.NM_ALLENTRIES;
-    } else {
-      delorder.source=delorder.NM_ONLYSELECTED;
-    }
+    if ( startlister == NULL ) return 1;
+    lm1 = startlister->getActiveMode();
 
-    delorder.delmode=delorder.NM_DELMODE_NORMAL;
-    delorder.dirdel=delorder.NM_DIRDELETE_NORMAL;
-    delorder.dowin=new DeleteOpWin( Worker::getAGUIX() );
-    nm1->deletef(&delorder);
-    delete delorder.dowin;
-    if(delorder.source==delorder.NM_SPECIAL) {
-      if ( specialsource != NULL ) delete specialsource;
-      delete delorder.sources;
+    if ( lm1 == NULL ) return 1;
+
+    vdm = dynamic_cast< VirtualDirMode * >( lm1 );
+  
+    if ( am->mode == am->AM_MODE_NORMAL ) {
+        memset( &delorder, 0, sizeof( delorder ) );
+        if ( also_active == true ) {
+            delorder.source = delorder.NM_ALLENTRIES;
+        } else {
+            delorder.source = delorder.NM_ONLYSELECTED;
+        }
+
+        delorder.delmode = delorder.NM_DELMODE_NORMAL;
+        delorder.dirdel = delorder.NM_DIRDELETE_NORMAL;
+
+        delorder.dowin = new DeleteOpWin( Worker::getAGUIX() );
+
+        if ( vdm ) {
+            vdm->deletef( &delorder );
+        }
+
+        delete delorder.dowin;
+
+        if ( delorder.source == delorder.NM_SPECIAL ) {
+            if ( specialsource != NULL ) delete specialsource;
+            delete delorder.sources;
+        }
     }
-  }
-  return 0;
+    return 0;
 }
 
 int DeleteOp::configure()
@@ -139,7 +151,7 @@ int DeleteOp::configure()
 
   tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(25))+1);
   sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(25));
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe(tstr);
   
@@ -248,7 +260,8 @@ DeleteOpWin::open()
   close();
   w=400;
   h=10;
-  window = new AWindow( aguix, 10, 10, w, h, 0, catalog.getLocaleCom( 25 ) );
+  window = new AWindow( aguix, 10, 10, w, h, 0, catalog.getLocaleCom( 25 ),
+                        AWindow::AWINDOW_DIALOG );
   window->create();
   tx=5;
   ty=5;
