@@ -1,6 +1,6 @@
 /* dirbookmarkui.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2007-2008,2010,2012 Ralf Hoffmann.
+ * Copyright (C) 2007-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -36,16 +36,17 @@
 #include <algorithm>
 #include "prefixdb.hh"
 #include "worker.h"
+#include "stringmatcher_flexibleregex.hh"
 
 DirBookmarkUI::DirBookmarkUI( AGUIX &aguix, BookmarkDBProxy &data  ) : m_aguix( aguix ),
                                                                        m_data( data )
 {
-    m_filtered_data = std::auto_ptr<BookmarkDBFilter>( new BookmarkDBFilter( data ) );
+    m_filtered_data = std::unique_ptr<BookmarkDBFilter>( new BookmarkDBFilter( data ) );
 
-    m_win = std::auto_ptr<AWindow>( new AWindow( &m_aguix,
-                                                 0, 0,
-                                                 400, 400,
-                                                 0, catalog.getLocaleCom( 51 ) ) );
+    m_win = std::unique_ptr<AWindow>( new AWindow( &m_aguix,
+                                                   0, 0,
+                                                   400, 400,
+                                                   0, catalog.getLocaleCom( 51 ) ) );
     m_win->create();
 
     m_co1 = m_win->setContainer( new AContainer( m_win.get(), 1, 5 ), true );
@@ -420,8 +421,8 @@ int DirBookmarkUI::mainLoop()
                   it1 != e.end();
                   it1++, row-- ) {
                 if ( row == 0 ) {
-                    m_selected_entry = std::auto_ptr<BookmarkDBEntry>( new BookmarkDBEntry( *it1 ) );
-		    pdb.pushAccess( m_filtered_data->getInfixFilter(), it1->getName() );
+                    m_selected_entry = std::unique_ptr<BookmarkDBEntry>( new BookmarkDBEntry( *it1 ) );
+                    pdb.pushAccess( m_filtered_data->getInfixFilter(), it1->getName() );
                 }
             }
         }
@@ -434,6 +435,12 @@ int DirBookmarkUI::mainLoop()
 
 void DirBookmarkUI::showData( const std::string &cat )
 {
+    StringMatcherFlexibleRegEx matcher;
+
+    matcher.setMatchFlexible( false );
+
+    matcher.setMatchString( m_filtered_data->getInfixFilter() );
+
     m_infixtext->setText( m_filtered_data->getInfixFilter().c_str() );
 
     int act_row = m_lv->getActiveRow();
@@ -452,6 +459,14 @@ void DirBookmarkUI::showData( const std::string &cat )
         
         if ( row == act_row )
             m_lv->setActiveRow( row );
+
+        if ( ! m_filtered_data->getInfixFilter().empty() ) {
+            std::vector< size_t > segments = matcher.getMatchSegments( it1->getName() );
+
+            if ( segments.size() > 1 ) {
+                m_lv->setHighlightSegments( row, 0, segments );
+            }
+        }
     }
 
     if ( m_lv->isValidRow( m_lv->getActiveRow() ) == false ) {
@@ -473,6 +488,11 @@ BookmarkDBEntry DirBookmarkUI::getSelectedEntry()
     throw 0;
 }
 
+std::string DirBookmarkUI::getFilterString()
+{
+    return m_filtered_data->getInfixFilter();
+}
+
 void DirBookmarkUI::setCurrentDirname( const std::string &dirname )
 {
     m_dirname = dirname;
