@@ -1,6 +1,6 @@
 /* enterdirop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2009,2011-2012 Ralf Hoffmann.
+ * Copyright (C) 2009-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,7 +21,6 @@
 
 #include "enterdirop.hh"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "wpucontext.h"
 #include "ownop.h"
@@ -31,8 +30,12 @@
 #include <aguix/cyclebutton.h>
 #include <aguix/button.h>
 #include <aguix/stringgadget.h>
+#include <aguix/request.h>
+#include <aguix/acontainer.h>
+#include <aguix/awindow.h>
 #include "datei.h"
 #include "nwc_path.hh"
+#include "argclass.hh"
 
 /* Bei normalen Aufruf: aktiven Eintrag benutzen
    Bei DND: Element (muesste aktives sein) in Ziel (muss erst erkannt werden) darstellen
@@ -78,26 +81,20 @@ int EnterDirOp::run( WPUContext *wpu, ActionMessage *msg )
 {
     Lister *l1;
     ListerMode *lm1;
-    NormalMode *nm1;
-    int erg;
-    char *buttonstr, *return_str, *tstr;
-    const char *textstr;
 
     if ( msg->mode == msg->AM_MODE_DNDACTION ) {
         l1 = msg->dndmsg->getDestLister();
         if ( l1 != NULL ) {
-            // Element in l1 darstellen
-            l1->switch2Mode( 0 );
             lm1 = l1->getActiveMode();
             if ( lm1 != NULL ) {
-                if ( lm1->isType( "NormalMode" ) == true ) {
-                    nm1 = (NormalMode*)lm1;
-                    FileEntry *fe = msg->dndmsg->getFE();
-                    if ( fe != NULL && fe->fullname != NULL ) {
-                        tstr = dupstring( fe->fullname );
-                        nm1->enterPath( tstr );
-                        _freesafe( tstr );
-                    }
+
+                const FileEntry *fe = msg->dndmsg->getFE();
+            
+                if ( fe != NULL && fe->fullname != NULL ) {
+                    std::list< RefCount< ArgClass > > args;
+
+                    args.push_back( new StringArg( fe->fullname ) );
+                    lm1->runCommand( "enter_dir", args );
                 }
             }
         }
@@ -106,21 +103,23 @@ int EnterDirOp::run( WPUContext *wpu, ActionMessage *msg )
             if ( strlen( dir ) > 0 ) {
                 l1 = msg->getWorker()->getActiveLister();
                 if ( l1 != NULL ) {
-                    l1->switch2Mode( 0 );
                     lm1 = l1->getActiveMode();
                     if ( lm1 != NULL ) {
-                        if ( lm1->isType( "NormalMode" ) == true ) {
-                            nm1 = (NormalMode*)lm1;
-              
-                            std::string res_str;
-                            if ( wpu->parse( dir, res_str, a_max( EXE_STRING_LEN - 1024, 256 ), false ) == NormalMode::PARSE_SUCCESS ) {
-                                nm1->enterPath( res_str.c_str() );
-                            }
+                        std::string res_str;
+                        if ( wpu->parse( dir, res_str, a_max( EXE_STRING_LEN - 1024, 256 ), false ) == WPUContext::PARSE_SUCCESS ) {
+                            std::list< RefCount< ArgClass > > args;
+
+                            args.push_back( new StringArg( res_str ) );
+                            lm1->runCommand( "enter_dir", args );
                         }
                     }
                 }
             }
         } else if ( enterdirmode == ENTERDIROP_REQUEST ) {
+            char *buttonstr, *return_str;
+            const char *textstr;
+            int erg;
+
             Requester *req = new Requester( msg->getWorker()->getAGUIX() );
             textstr = catalog.getLocale( 290 );
             buttonstr = (char*)_allocsafe( strlen( catalog.getLocale( 11 ) ) + 1 +
@@ -132,38 +131,52 @@ int EnterDirOp::run( WPUContext *wpu, ActionMessage *msg )
             if ( erg == 0 && strlen( return_str) > 0 ) {
                 l1 = msg->getWorker()->getActiveLister();
                 if ( l1 != NULL ) {
-                    l1->switch2Mode( 0 );
                     lm1 = l1->getActiveMode();
                     if ( lm1 != NULL ) {
-                        if ( lm1->isType( "NormalMode" ) == true ) {
-                            nm1 = (NormalMode*)lm1;
-
-                            //TODO show a choosebutton in the requester to enable env replacing?
-                            std::string tstr1 = return_str;
-                            if ( tstr1[0] == '$' ) {
-                                char *tstr2 = NWC::Path::handlePathExt( tstr1.c_str() );
-                                tstr1 = tstr2;
-                                _freesafe( tstr2 );
-                            }
-                            nm1->enterPath( tstr1.c_str() );
+                        //TODO show a choosebutton in the requester to enable env replacing?
+                        std::string tstr1 = return_str;
+                        if ( tstr1[0] == '$' ) {
+                            char *tstr2 = NWC::Path::handlePathExt( tstr1.c_str() );
+                            tstr1 = tstr2;
+                            _freesafe( tstr2 );
                         }
+
+                        std::list< RefCount< ArgClass > > args;
+
+                        args.push_back( new StringArg( tstr1 ) );
+                        lm1->runCommand( "enter_dir", args );
                     }
                 }
             }
             _freesafe( return_str );
             delete req;
         } else {
-            // here we need the help of NormalMode
             l1 = msg->getWorker()->getActiveLister();
             if ( l1 != NULL ) {
                 lm1 = l1->getActiveMode();
+
                 if ( lm1 != NULL ) {
-                    if ( lm1->isType( "NormalMode" ) == true ) {
-                        nm1 = (NormalMode*)lm1;
+                    std::list< NM_specialsourceExt > extfilelist;
+                    std::string entry_fullname;
+
+                    lm1->getSelFiles( extfilelist, ListerMode::LM_GETFILES_ONLYACTIVE );
+                    for ( auto &ss1 : extfilelist ) {
+                        entry_fullname = ss1.entry()->fullname;
+                    }
+
+                    if ( ! entry_fullname.empty() ) {
+                        std::list< RefCount< ArgClass > > args;
+
+                        args.push_back( new StringArg( entry_fullname ) );
+
                         if ( enterdirmode == ENTERDIROP_ACTIVE ) {
-                            nm1->enterDirActive();
+                            lm1->runCommand( "enter_active" );
                         } else {
-                            nm1->setActiveDir2Other();
+                            ListerMode *nonactivemode = msg->getWorker()->getOtherLister( msg->getWorker()->getActiveLister() )->getActiveMode();
+
+                            if ( nonactivemode ) {
+                                nonactivemode->runCommand( "enter_dir", args );
+                            }
                         }
                     }
                 }
@@ -198,7 +211,7 @@ int EnterDirOp::configure()
     tstr = (char*)_allocsafe( strlen( catalog.getLocale( 293 ) ) +
                               strlen( catalog.getLocaleCom( 18 ) ) + 1 );
     sprintf( tstr, catalog.getLocale( 293 ), catalog.getLocaleCom( 18 ) );
-    win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+    win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
     win->create();
     _freesafe( tstr );
 
