@@ -1,6 +1,6 @@
 /* enterpathop.cc
- * This file belongs to Worker, a filemanager for UNIX/X11.
- * Copyright (C) 2001-2004 Ralf Hoffmann.
+ * This file belongs to Worker, a file manager for UN*X/X11.
+ * Copyright (C) 2001-2004,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -18,16 +18,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
-/* $Id: enterpathop.cc,v 1.16 2005/10/19 18:05:39 ralf Exp $ */
 
 #include "enterpathop.h"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "worker_locale.h"
 #include "datei.h"
 #include <aguix/button.h>
 #include <aguix/cyclebutton.h>
+#include <aguix/acontainer.h>
+#include <aguix/lowlevelfunc.h>
+#include <aguix/awindow.h>
 
 const char *EnterPathOp::name="EnterPathOp";
 
@@ -64,18 +65,12 @@ EnterPathOp::getName()
 int
 EnterPathOp::run( WPUContext *wpu, ActionMessage *msg )
 {
-  ListerMode *lm1;
-  if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1 = msg->getWorker()->getActiveLister();
-    if(l1!=NULL) {
-      startlister=l1;
-      lm1=startlister->getActiveMode();
-      if(lm1!=NULL)
-        if(lm1->isType("NormalMode")==true)
-          normalmodeenterpath( msg );
+    if ( msg->mode != msg->AM_MODE_DNDACTION ) {
+        startlister = msg->getWorker()->getActiveLister();
+
+        normalmodeenterpath( msg );
     }
-  }
-  return 0;
+    return 0;
 }
 
 bool
@@ -108,7 +103,6 @@ EnterPathOp::getDescription()
 void
 EnterPathOp::normalmodeenterpath( ActionMessage *am )
 {
-  NormalMode *nm1=NULL;
   ListerMode *lm1=NULL;
   Lister *ll,*lr,*tl;
   
@@ -134,10 +128,7 @@ EnterPathOp::normalmodeenterpath( ActionMessage *am )
   if(tl!=NULL) {
     lm1=tl->getActiveMode();
     if(lm1!=NULL) {
-      if(lm1->isType("NormalMode")==true) {
-        nm1=(NormalMode*)lm1;
-        nm1->enterpath();
-      }
+        lm1->runCommand( "activate_path_input" );
     }
   }
 }
@@ -161,7 +152,7 @@ EnterPathOp::configure()
   
   tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(34))+1);
   sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(34));
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe(tstr);
 
