@@ -1,6 +1,6 @@
 /* fileentry.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2008,2010-2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2008,2010-2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -718,7 +718,7 @@ dev_t FileEntry::ddev() const
   return dstatbuf.dev;
 }
 
-bool FileEntry::checkAccess( int m )
+bool FileEntry::checkAccess( int m ) const
 {
   if ( fullname == NULL ) return false;
   return ( worker_access( fullname, m ) == 0 ) ? true : false;
@@ -965,6 +965,60 @@ FileEntry::FileEntry( const FileEntry &other )
   }
 }
 
+FileEntry::FileEntry( const NWC::FSEntry &other )
+{
+    fullname = dupstring( other.getFullname().c_str() );
+    name = dupstring( other.getBasename().c_str() );
+    filetype = nullptr;
+    use = true;
+    isHidden = other.isHiddenEntry();
+    isCorrupt = other.isBrokenLink();
+
+    reclistQueued = false;
+
+    memset( &statbuf, 0, sizeof( statbuf ) );
+    memset( &dstatbuf, 0, sizeof( dstatbuf ) );
+    
+    statbuf.size = other.stat_size();
+    statbuf.lastaccess = other.stat_lastaccess();
+    statbuf.lastmod = other.stat_lastmod();
+    statbuf.lastchange = other.stat_lastchange();
+    statbuf.mode = other.stat_mode();
+    statbuf.userid = other.stat_userid();
+    statbuf.groupid = other.stat_groupid();
+    statbuf.inode = other.stat_inode();
+    statbuf.nlink = other.stat_nlink();
+    statbuf.blocks = other.stat_blocks();
+    statbuf.rdev = other.stat_rdev();
+    statbuf.dev = other.stat_dev();
+
+    dstatbuf.size = other.stat_dest_size();
+    dstatbuf.lastaccess = other.stat_dest_lastaccess();
+    dstatbuf.lastmod = other.stat_dest_lastmod();
+    dstatbuf.lastchange = other.stat_dest_lastchange();
+    dstatbuf.mode = other.stat_dest_mode();
+    dstatbuf.userid = other.stat_dest_userid();
+    dstatbuf.groupid = other.stat_dest_groupid();
+    dstatbuf.inode = other.stat_dest_inode();
+    dstatbuf.nlink = other.stat_dest_nlink();
+    dstatbuf.blocks = other.stat_dest_blocks();
+    dstatbuf.rdev = other.stat_dest_rdev();
+    dstatbuf.dev = other.stat_dest_dev();
+
+    dirsize = 0;
+    select = false;
+    isLink = other.isLink();
+
+    contentBuf.content = NULL;
+    contentBuf.size = 0;
+  
+    outputBuf = NULL;
+
+    _ID = -1;
+
+    m_bookmark_info = NULL;
+}
+
 /*
  * Method to test for equal fileentry
  *
