@@ -1,6 +1,6 @@
 /* filereq.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2007,2011-2012 Ralf Hoffmann.
+ * Copyright (C) 2001-2007,2011-2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -73,7 +73,7 @@ int FileRequester::request( const char *title,
   const int cmin = AContainer::ACONT_MINH +
                    AContainer::ACONT_MINW;
 
-  AWindow *win = new AWindow( aguix, 10, 10, 300, 200, 0, title );
+  AWindow *win = new AWindow( aguix, 10, 10, 300, 200, 0, title, AWindow::AWINDOW_DIALOG );
   win->create();
 
   AContainer *ac1 = win->setContainer( new AContainer( win, 1, 3 ), true );
@@ -390,7 +390,7 @@ int FileRequester::request_entry( const char *title,
     }
   } else use_dir = dupstring( actdir );
 
-  win = new AWindow( aguix, 10, 10, 300, 200, 0, title );
+  win = new AWindow( aguix, 10, 10, 300, 200, 0, title, AWindow::AWINDOW_DIALOG );
   win->create();
 
   AContainer *ac1 = win->setContainer( new AContainer( win, 1, 3 ), true );
