@@ -1,6 +1,6 @@
 /* fontreq.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2008,2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2008,2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -146,7 +146,7 @@ int FontRequester::request(const char *xname,char**return_xname)
   const int cmin = AContainer::ACONT_MINH +
                    AContainer::ACONT_MINW;
 
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, catalog.getLocale( 204 ) );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, catalog.getLocale( 204 ), AWindow::AWINDOW_DIALOG );
   win->create();
 
   ac1 = win->setContainer( new AContainer( win, 1, 7 ), true );
