@@ -139,7 +139,7 @@ bool ActionMessage::getKeyAction() const
 
 const char *FunctionProto::getDescription()
 {
-  return "Prototype einer Aktion";
+    return "prototype of an action";
 }
 
 int FunctionProto::configureWhenAvail()
@@ -196,6 +196,11 @@ FunctionProto::command_categories_t FunctionProto::getCategory() const
     return m_category;
 }
 
+std::string FunctionProto::getStringRepresentation()
+{
+    return getDescription();
+}
+
 void ActionMessage::setEntriesToConsider( RefCount< NWC::VirtualDir > entries )
 {
     m_entries_to_consider = entries;
@@ -206,3 +211,27 @@ RefCount< NWC::VirtualDir > ActionMessage::getEntriesToConsider()
 {
     return m_entries_to_consider;
 }
+
+std::string getStringReprForList( List *coms )
+{
+    std::string res;
+    FunctionProto *fp;
+    int id =coms->initEnum();
+
+    fp = (FunctionProto*)coms->getFirstElement( id );
+
+    while ( fp != NULL ) {
+
+        if ( ! res.empty() ) {
+            res += "; ";
+        }
+
+        res += fp->getStringRepresentation();
+
+        fp = (FunctionProto*)coms->getNextElement( id );
+    }
+
+    coms->closeEnum(id);
+
+    return res;
+}
