@@ -1,6 +1,6 @@
 /* goftpop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2005-2007,2010-2011 Ralf Hoffmann.
+ * Copyright (C) 2005-2007,2010-2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,7 +21,6 @@
 
 #include "goftpop.h"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "avfssupport.hh"
 #include "worker_locale.h"
@@ -30,6 +29,8 @@
 #include <aguix/choosebutton.h>
 #include <aguix/cyclebutton.h>
 #include <aguix/button.h>
+#include "argclass.hh"
+#include "virtualdirmode.hh"
 
 const char *GoFTPOp::name = "GoFTPOp";
 
@@ -81,7 +82,6 @@ GoFTPOp::run( WPUContext *wpu, ActionMessage *msg )
   Requester *req = Worker::getRequester();
   std::string str1;
   bool cont = true;
-  NormalMode *nm1=NULL;
   ListerMode *lm1=NULL;
   Lister *l1;
 
@@ -135,13 +135,17 @@ GoFTPOp::run( WPUContext *wpu, ActionMessage *msg )
 
 	l1 = msg->getWorker()->getActiveLister();
 	if ( l1 != NULL ) {
-	  l1->switch2Mode( 0 );
-	  lm1 = l1->getActiveMode();
-	  if ( ( lm1 != NULL ) &&
-	       ( lm1->isType( "NormalMode" ) == true ) ) {
-	    nm1=(NormalMode*)lm1;
-	    nm1->enterDir( str1.c_str() );
-	  }
+            if ( l1->getActiveMode() == NULL ||
+                 ! ( dynamic_cast< VirtualDirMode *>( l1->getActiveMode() ) ) ) {
+                l1->switch2Mode( 0 );
+            }
+            lm1 = l1->getActiveMode();
+            if ( lm1 != NULL ) {
+                std::list< RefCount< ArgClass > > args;
+
+                args.push_back( new StringArg( str1 ) );
+                lm1->runCommand( "enter_dir", args );
+            }
 	}
       }
     }
@@ -205,7 +209,7 @@ GoFTPOp::doconfigure(int mode)
 
   tstr = (char*)_allocsafe( strlen( catalog.getLocale( 293 ) ) + strlen( catalog.getLocaleCom( 48 ) ) + 1 );
   sprintf( tstr, catalog.getLocale( 293 ), catalog.getLocaleCom( 48 ) );
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe( tstr );
 
