@@ -420,6 +420,16 @@ void HWVolumeManager::update( HWVolume &vol )
             vol.setIsAvailable( false );
         }
 
+        try {
+#ifdef HAVE_HAL_DBUS
+#elif defined(HAVE_DBUS)
+            bool avail = getBoolProperty( vol.getUDI(),
+                                          "DeviceIsMediaAvailable" );
+            vol.setIsAvailable( avail );
+#endif
+        } catch ( int ) {
+        }
+
         if ( m_fstab_table.containsDevice( vol.getDevice() ) ) {
             vol.setIsInFStab( true );
         } else {
@@ -560,8 +570,7 @@ std::list<HWVolume> HWVolumeManager::getNewVolumes( bool store_new )
     for ( std::list<HWVolume>::iterator it1 = l.begin();
           it1 != l.end();
           it1++ ) {
-        if ( ! it1->getUDI().empty() ||
-             it1->isAvailable() == true ) {
+        if ( it1->isAvailable() == true ) {
             if ( m_known_devices.count( it1->getDevice() ) == 0 ) {
                 newl.push_back( *it1 );
             }
