@@ -1,6 +1,6 @@
 /* informationmode.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2009,2012 Ralf Hoffmann.
+ * Copyright (C) 2001-2009,2012-2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -20,7 +20,7 @@
  */
 
 #include "informationmode.h"
-#include "normalmode.h"
+#include "listermode.h"
 #include "worker_locale.h"
 #include "wconfig.h"
 #include "worker.h"
@@ -31,6 +31,7 @@
 #include "nmspecialsourceext.hh"
 #include <aguix/acontainer.h>
 #include <aguix/fieldlistview.h>
+#include <aguix/awindow.h>
 #include "datei.h"
 #include "nwc_path.hh"
 
@@ -156,16 +157,6 @@ int InformationMode::load()
   return 0;
 }
 
-int InformationMode::loadBin(Datei *fh)
-{
-  int chunksize=fh->getInt();
-  while(chunksize>0) {
-    fh->getUChar();
-    chunksize--;
-  }
-  return 0;
-}
-
 bool InformationMode::save(Datei *fh)
 {
   return false;
@@ -176,7 +167,7 @@ void InformationMode::setName()
   parentlister->setName(catalog.getLocale(175));
 }
 
-void InformationMode::reconfig()
+void InformationMode::relayout()
 {
     int side = parentlister->getSide();
     lv->setFont( wconfig->getFont( 2 + side ).c_str() );
@@ -184,36 +175,26 @@ void InformationMode::reconfig()
 
 void InformationMode::update(bool force)
 {
-  NormalMode *nm1=NULL;
   ListerMode *lm1=NULL;
   Lister *ol=NULL;
-  std::list<NM_specialsourceExt*> *files;
-  std::list<NM_specialsourceExt*>::iterator it1;
-  NM_specialsourceExt *ss1;
   const FileEntry *fe;
   
   ol=parentlister->getWorker()->getOtherLister(parentlister);
   if(ol!=NULL) {
     lm1=ol->getActiveMode();
-    if(lm1!=NULL)
-      if(lm1->isType("NormalMode")==true)
-        nm1=(NormalMode*)lm1;
   }
   
   if ( ( force == true ) ||
        ( nospaceinfo == true ) )
     clearLastActiveFE();
   
-  if(nm1!=NULL) {
-    files=new std::list<NM_specialsourceExt*>;
+  if(lm1!=NULL) {
+    std::list< NM_specialsourceExt > files;
     
-    nm1->getSelFiles( files, NormalMode::NM_GETFILES_ONLYACTIVE );
-    if ( files->begin() != files->end() )
-      ss1 = *( files->begin() );
-    else
-      ss1 = NULL;
-    if(ss1!=NULL) {
-      fe = ss1->entry();
+    lm1->getSelFiles( files, ListerMode::LM_GETFILES_ONLYACTIVE );
+
+    if ( ! files.empty() ) {
+      fe = files.begin()->entry();
       if ( fe->equals( lastactivefe ) == false ) {
 	clearLastActiveFE();
 	if ( fe != NULL ) {
@@ -230,12 +211,6 @@ void InformationMode::update(bool force)
 	}
       }
     }
-    
-    for ( it1 = files->begin(); it1 != files->end(); it1++ ) {
-      delete *it1;
-    }
-    
-    delete files;
   }
 }
 
