@@ -1,6 +1,6 @@
 /* intviewop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2005-2006,2008,2011 Ralf Hoffmann.
+ * Copyright (C) 2005-2006,2008,2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,7 +21,6 @@
 
 #include "intviewop.h"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "ownop.h"
 #include "wpucontext.h"
@@ -35,6 +34,7 @@
 #include <aguix/button.h>
 #include <aguix/choosebutton.h>
 #include <aguix/stringgadget.h>
+#include <aguix/awindow.h>
 #include "dnd.h"
 
 const char *InternalViewOp::name = "InternalViewOp";
@@ -79,10 +79,7 @@ InternalViewOp::run( WPUContext *wpu, ActionMessage *msg )
 {
   std::string str1;
   bool cont = true;
-  NormalMode *nm1 = NULL;
   ListerMode *lm1 = NULL;
-  std::list<NM_specialsourceExt*> extfilelist;
-  std::list<NM_specialsourceExt*>::iterator it1;
   std::list<std::string> filelist;
 
   if ( _request_flags == true ) {
@@ -101,9 +98,6 @@ InternalViewOp::run( WPUContext *wpu, ActionMessage *msg )
       lm1 = msg->dndmsg->getSourceMode();
     }
     
-    if ( ( lm1 != NULL ) && ( lm1->isType( "NormalMode" ) == true ) )
-      nm1 = (NormalMode*)lm1;
-    
     if ( msg->mode == msg->AM_MODE_ONLYACTIVE )
       _tmode = SHOW_ACTIVE_FILE;
     else if ( msg->mode == msg->AM_MODE_DNDACTION ) {
@@ -114,16 +108,14 @@ InternalViewOp::run( WPUContext *wpu, ActionMessage *msg )
 
     if ( filelist.size() < 1 ) {
       if ( _tmode == SHOW_ACTIVE_FILE ) {
-	if ( nm1 != NULL ) {
-	  nm1->getSelFiles( &extfilelist, NormalMode::NM_GETFILES_ONLYACTIVE );
-	  for ( it1 = extfilelist.begin(); it1 != extfilelist.end(); it1++ ) {
-	    filelist.push_back( (*it1)->entry()->fullname );
-	  }
-	  NormalMode::freeSelFiles( &extfilelist );
-	}
+          std::list< NM_specialsourceExt > extfilelist;
+          lm1->getSelFiles( extfilelist, ListerMode::LM_GETFILES_ONLYACTIVE );
+          for ( auto &ss1 : extfilelist ) {
+              filelist.push_back( ss1.entry()->fullname );
+          }
 	/*} else if ( _tmode == SHOW_SELECTED_FILES ) {
 	  if ( nm1 != NULL ) {
-	  nm1->getSelFiles( &extfilelist, NormalMode::NM_GETFILES_SELORACT, true );
+	  nm1->getSelFiles( &extfilelist, ListerMode::LM_GETFILES_SELORACT, true );
 	  for ( it1 = extfilelist.begin(); it1 != extfilelist.end(); it1++ ) {
 	  filelist.push_back( (*it1)->entry()->fullname );
 	  }
@@ -134,7 +126,7 @@ InternalViewOp::run( WPUContext *wpu, ActionMessage *msg )
           if ( wpu->parse( _tcustom_files.c_str(),
                            res_str,
                            a_max( EXE_STRING_LEN - 1024, 256 ),
-                           true ) == NormalMode::PARSE_SUCCESS ) {
+                           true ) == WPUContext::PARSE_SUCCESS ) {
               char **argvlist;
               char *unquoted_str;
               int n, i;
@@ -153,8 +145,13 @@ InternalViewOp::run( WPUContext *wpu, ActionMessage *msg )
     }
 
     if ( filelist.size() > 0 ) {
-	FileViewer fv;
-	fv.view( filelist, NULL );
+
+        for ( auto &f : filelist ) {
+            msg->getWorker()->storePathPersIfInProgress( f );
+        }
+
+        FileViewer fv;
+        fv.view( filelist, NULL );
     }
   }
   return 0;
@@ -216,7 +213,7 @@ InternalViewOp::doconfigure(int mode)
   
   tstr = (char*)_allocsafe( strlen( catalog.getLocale( 293 ) ) + strlen( catalog.getLocaleCom( 49 ) ) + 1 );
   sprintf( tstr, catalog.getLocale( 293 ), catalog.getLocaleCom( 49 ) );
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe( tstr );
 
