@@ -1,6 +1,6 @@
 /* lister.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2005,2009,2011-2013 Ralf Hoffmann.
+ * Copyright (C) 2001-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -110,7 +110,19 @@ void Lister::messageHandler(AGMessage *msg)
         break;
     }
   }
-  if(subHandle==true) if(activemode!=NULL) activemode->messageHandler(msg);
+
+  if ( subHandle == true ) {
+      if ( activemode != NULL ) {
+          activemode->messageHandler( msg );
+      }
+
+      int j = parent->getMaxModeNr();
+      for ( int i = 0; i < j; i++ ) {
+          if ( modes[i] != NULL && modes[i] != activemode ) {
+              modes[i]->messageHandlerInactive( msg );
+          }
+      }
+  }
 }
 
 AGUIX *Lister::getAGUIX()
@@ -359,7 +371,16 @@ Worker *Lister::getWorker()
 
 void Lister::cyclicfunc(cyclicfunc_mode_t mode)
 {
-  if(activemode!=NULL) activemode->cyclicfunc(mode);
+    if ( activemode != NULL ) {
+        activemode->cyclicfunc( mode );
+    }
+
+    int j = parent->getMaxModeNr();
+    for ( int i = 0; i < j; i++ ) {
+        if ( modes[i] != NULL && modes[i] != activemode ) {
+            modes[i]->cyclicfuncInactive( mode );
+        }
+    }
 }
 
 int Lister::saveState(Datei *fh)
@@ -589,3 +610,11 @@ bool Lister::pathsChanged( const std::set< std::string > changed_paths )
     if ( activemode != NULL ) return activemode->pathsChanged( changed_paths );
     return false;
 }
+
+void Lister::finalizeBGOps()
+{
+    int j = parent->getMaxModeNr();
+    for ( int i = 0; i < j; i++ ) {
+        modes[i]->finalizeBGOps();
+    }
+}
