@@ -396,7 +396,13 @@ void MenuTreeUI::show()
                                     fullname += "/";
                                 }
                                 fullname += m_lv->getText( row, 0 );
-                                if ( best_hit == fullname ) {
+
+                                std::string fullname_with_help = fullname;
+                                fullname += ";";
+                                fullname += m_lv_help->getText( row, 0 );;
+
+                                if ( best_hit == fullname ||
+                                     best_hit == fullname_with_help ) {
                                     m_lv->setActiveRow( row );
                                     m_lv->redraw();
                                     break;
@@ -444,6 +450,10 @@ void MenuTreeUI::show()
                 }
                 fullname += m_lv->getText( row, 0 );
 
+                fullname += ";";
+
+                fullname += m_lv_help->getText( row, 0 );
+
                 if ( ! filter.empty() ) {
                     filter += "/";
                 }
