@@ -1,6 +1,6 @@
 /* modifytabsop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2009,2011 Ralf Hoffmann.
+ * Copyright (C) 2009,2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,7 +21,6 @@
 
 #include "modifytabsop.hh"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include <aguix/awindow.h>
 #include <aguix/text.h>
@@ -30,6 +29,8 @@
 #include <aguix/cyclebutton.h>
 #include "worker_locale.h"
 #include "datei.h"
+#include "argclass.hh"
+#include "virtualdirmode.hh"
 
 const char *ModifyTabsOp::name = "ModifyTabsOp";
 
@@ -72,11 +73,15 @@ ModifyTabsOp::run( WPUContext *wpu, ActionMessage *msg )
         if ( l1 != NULL ) {
             startlister = l1;
             lm1 = startlister->getActiveMode();
-            if ( lm1 != NULL )
-                if ( lm1->isType( "NormalMode" ) == true )
+            if ( lm1 != NULL ) {
+                if ( dynamic_cast< VirtualDirMode *>( lm1 ) ) {
                     nm_tab_action( msg );
+                } else {
+                    lm1->not_supported();
+                }
+            }
         }
-    };
+    }
     return 0;
 }
 
@@ -110,28 +115,27 @@ ModifyTabsOp::getDescription()
 void
 ModifyTabsOp::nm_tab_action( ActionMessage *am )
 {
-    NormalMode *nm1 = NULL;
     ListerMode *lm1 = NULL;
 
     if ( startlister != NULL ) {
         lm1 = startlister->getActiveMode();
         if ( lm1 != NULL ) {
-            if ( lm1->isType( "NormalMode" ) == true ) {
-                nm1 = dynamic_cast<NormalMode*>( lm1 );
-                switch ( m_tab_action ) {
-                    case CLOSE_CURRENT_TAB:
-                        nm1->closeCurrentTab();
-                        break;
-                    case NEXT_TAB:
-                        nm1->switchToTabDirection( 1 );
-                        break;
-                    case PREV_TAB:
-                        nm1->switchToTabDirection( -1 );
-                        break;
-                    default:
-                        nm1->newTab();
-                        break;
-                }
+            std::list< RefCount< ArgClass > > args;
+            switch ( m_tab_action ) {
+                case CLOSE_CURRENT_TAB:
+                    lm1->runCommand( "close_current_tab" );
+                    break;
+                case NEXT_TAB:
+                    args.push_back( new IntArg( 1 ) );
+                    lm1->runCommand( "switch_to_tab_direction", args );
+                    break;
+                case PREV_TAB:
+                    args.push_back( new IntArg( -1 ) );
+                    lm1->runCommand( "switch_to_tab_direction", args );
+                    break;
+                default:
+                    lm1->runCommand( "new_tab" );
+                    break;
             }
         }
     }
@@ -150,7 +154,7 @@ ModifyTabsOp::configure()
     tstr = (char*)_allocsafe( strlen( catalog.getLocale( 293 ) ) +
                               strlen( catalog.getLocaleCom( 56 ) ) + 1 );
     sprintf( tstr, catalog.getLocale( 293 ), catalog.getLocaleCom( 56 ) );
-    win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+    win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
     win->create();
     _freesafe(tstr);
     
