@@ -1,6 +1,6 @@
 /* nmcacheentry.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2008,2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2008,2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -28,12 +28,12 @@
 #include "fileentry.hh"
 #include "simplelist.hh"
 
-NMCacheEntry::NMCacheEntry( std::auto_ptr<Verzeichnis> verz,
+NMCacheEntry::NMCacheEntry( std::unique_ptr<Verzeichnis> verz,
                             DirFilterSettings *dir_filter_sets,
                             DirSortSettings *dir_sort_sets,
                             DirBookmarksSettings *dir_bookmarks_sets ) : _active_is_visible( false ),
                                                                          _last_active_row( -1 ),
-                                                                         _verz( verz ),
+                                                                         _verz( std::move( verz ) ),
                                                                          _dir_filter_sets( dir_filter_sets ),
                                                                          _dir_sort_sets( dir_sort_sets ),
                                                                          m_dir_bookmarks_sets( dir_bookmarks_sets ),
@@ -417,9 +417,9 @@ void NMCacheEntry::checkDirSettings( bool force )
     }
 }
 
-void NMCacheEntry::setVerz( std::auto_ptr<Verzeichnis> verz )
+void NMCacheEntry::setVerz( std::unique_ptr<Verzeichnis> verz )
 {
-    _verz = verz;
+    _verz = std::move( verz );
     
     _activefe = NULL;
 
