@@ -1,6 +1,6 @@
 /* nmextlist.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2008,2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2008,2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -20,7 +20,8 @@
  */
 
 #include "nmextlist.hh"
-#include "normalmode.h"
+#include "listermode.h"
+#include "functionproto.h"
 #include "verzeichnis.hh"
 #include "simplelist.hh"
 #include "fileentry.hh"
@@ -95,20 +96,16 @@ int NMExtList::removeElement( NM_extern_fe *efe )
   return 0;
 }
 
-void NMExtList::createExtList( NormalMode *nm, bool rec )
+void NMExtList::createExtList( ListerMode *lm, bool rec )
 {
-  if ( nm == NULL ) return;
+  if ( lm == NULL ) return;
 
-  std::list<NM_specialsourceExt*> *splist;
+  std::list< NM_specialsourceExt > splist;
 
-  splist = new std::list<NM_specialsourceExt*>;
-  nm->getSelFiles( splist, NormalMode::NM_GETFILES_SELORACT );
+  lm->getSelFiles( splist, ListerMode::LM_GETFILES_SELORACT );
 
   createExtList( splist, rec );
 
-  NormalMode::freeSelFiles( splist );
-  delete splist;
-
   files = getEntriesRealSize();
   dirs = extlist->size() - files;
 }
@@ -116,25 +113,36 @@ void NMExtList::createExtList( NormalMode *nm, bool rec )
 class ExtListDirWalk : public NWC::DirWalkCallBack
 {
 public:
-    ExtListDirWalk( NormalMode *nm,
-                    std::list<NM_specialsourceExt*> *splist ) : m_normalmode( nm ),
-                                                                m_splist( splist )
+    ExtListDirWalk( ListerMode *lm,
+                    std::list< NM_specialsourceExt > &splist ) : m_listermode( lm ),
+                                                                 m_splist( splist )
     {
     }
 
     int visit( NWC::File &file, NWC::Dir::WalkControlObj &cobj )
     {
-        NM_specialsourceExt *ss1 = m_normalmode->getSpecialsourceForFilename( file.getFullname() );
+        std::unique_ptr< NM_specialsourceExt > ss1;
+
+        FileEntry fe( file );
+
+        ss1 = std::unique_ptr< NM_specialsourceExt >( new NM_specialsourceExt( &fe ) );
+
         if ( ss1 != NULL ) {
-            m_splist->push_back( ss1 );
+            m_splist.push_back( *ss1 );
         }
         return 0;
     }
+
     int visitEnterDir( NWC::Dir &dir, NWC::Dir::WalkControlObj &cobj )
     {
-        NM_specialsourceExt *ss1 = m_normalmode->getSpecialsourceForFilename( dir.getFullname() );
+        std::unique_ptr< NM_specialsourceExt > ss1;
+
+        FileEntry fe( dir );
+
+        ss1 = std::unique_ptr< NM_specialsourceExt >( new NM_specialsourceExt( &fe ) );
+
         if ( ss1 != NULL ) {
-            m_splist->push_back( ss1 );
+            m_splist.push_back( *ss1 );
         }
         return 0;
     }
@@ -143,8 +151,8 @@ public:
                         std::list< RefCount< NWC::FSEntry > > &return_add_entries,
                         std::list< RefCount< NWC::FSEntry > > &return_skip_entries ) { return 0; }
 private:
-    NormalMode *m_normalmode;
-    std::list<NM_specialsourceExt*> *m_splist;
+    ListerMode *m_listermode;
+    std::list< NM_specialsourceExt > &m_splist;
 };
 
 /*
@@ -154,36 +162,37 @@ private:
  * of message is given.
  * At the moment it handles SPECIAL and DND
  */
-void NMExtList::createExtList( NormalMode *nm, ActionMessage *msg, bool rec )
+void NMExtList::createExtList( ListerMode *lm, ActionMessage *msg, bool rec )
 {
-  if ( nm == NULL ) return;
+  if ( lm == NULL ) return;
 
-  std::list<NM_specialsourceExt*> *splist;
-  std::list<NM_specialsourceExt*>::iterator ite1;
-  NM_specialsourceExt *ss1;
+  std::list< NM_specialsourceExt > splist;
+  std::unique_ptr<NM_specialsourceExt> ss1;
   const DNDMsg *dm;
 
-  splist = new std::list<NM_specialsourceExt*>;
-
   switch ( msg->mode ) {
     case ActionMessage::AM_MODE_SPECIAL:
-      ss1 = nm->getSpecialsourceForFE( msg->getFE() );
-      if ( ss1 != NULL ) splist->push_back( ss1 );
-      break;
+        ss1 = std::unique_ptr< NM_specialsourceExt >( new NM_specialsourceExt( msg->getFE() ) );
+
+        if ( ss1 != NULL ) {
+            splist.push_back( *ss1 );
+        }
+        break;
     case ActionMessage::AM_MODE_SPECIFIC_ENTRIES:
         {
             RefCount< NWC::VirtualDir > vd = msg->getEntriesToConsider();
             if ( vd.getVal() != NULL ) {
-                ExtListDirWalk w( nm, splist );
+                ExtListDirWalk w( lm, splist );
                 vd->walk( w );
             }
         }
         break;
     case ActionMessage::AM_MODE_DNDACTION:
-      dm = msg->dndmsg;
-      //      if ( dm->getSourceLister() == parentlister ) {
-        ss1 = nm->getSpecialsourceForFE( dm->getFE() );
-        if ( ss1 != NULL ) splist->push_back( ss1 );
+        dm = msg->dndmsg;
+        ss1 = std::unique_ptr< NM_specialsourceExt >( new NM_specialsourceExt( dm->getFE() ) );
+        if ( ss1 != NULL ) {
+            splist.push_back( *ss1 );
+        }
         //      }
       break;
     case ActionMessage::AM_MODE_ONLYACTIVE:
@@ -194,11 +203,6 @@ void NMExtList::createExtList( NormalMode *nm, ActionMessage *msg, bool rec )
 
   createExtList( splist, rec );
 
-  for ( ite1 = splist->begin(); ite1 != splist->end(); ite1++ ) {
-    delete *ite1;
-  }
-  delete splist;
-
   files = getEntriesRealSize();
   dirs = extlist->size() - files;
 }
@@ -234,18 +238,13 @@ int NMExtList::create_externfe_rec( const char *fullname, bool rec, const FileEn
   return 0;
 }
 
-int NMExtList::createExtList( std::list<NM_specialsourceExt*> *splist, bool rec )
+int NMExtList::createExtList( std::list< NM_specialsourceExt > &splist, bool rec )
 {
-  NM_specialsourceExt *ss1;
-  std::list<NM_specialsourceExt*>::iterator ite1;
+    for ( auto &ss1 : splist ) {
+        create_externfe_rec( ss1.entry()->fullname, rec, ss1.entry() );
+    }
 
-  if ( splist == NULL ) return -1;
-
-  for ( ite1 = splist->begin(); ite1 != splist->end(); ite1++ ) {
-    ss1 = *ite1;
-    create_externfe_rec( ss1->entry()->fullname, rec, ss1->entry() );
-  }
-  return 0;
+    return 0;
 }
 
 int NMExtList::getEntriesRealSize()
