@@ -1,6 +1,6 @@
 /* normalops.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2005,2010-2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,7 +21,6 @@
 
 #include "normalops.h"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "workerinitialsettings.hh"
 #include "nwc_path.hh"
@@ -32,6 +31,11 @@
 #include <aguix/stringgadget.h>
 #include <aguix/fieldlistview.h>
 #include "datei.h"
+#include "virtualdirmode.hh"
+#include "argclass.hh"
+#include "fileentry.hh"
+#include "stringmatcher_fnmatch.hh"
+#include "pers_kvp.hh"
 
 /******
   ChangeListerSetOp
@@ -110,7 +114,7 @@ int ChangeListerSetOp::configure()
   
   tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(19))+1);
   sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(19));
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe(tstr);
 
@@ -303,10 +307,10 @@ const char *SwitchListerOp::getDescription()
 
 const char *FilterSelectOp::name="FilterSelectOp";
 int FilterSelectOp::instance_counter = 0;
-size_t FilterSelectOp::history_size = 30;
+size_t FilterSelectOp::history_size = 1000;
 int FilterSelectOp::lastw = -1;
 int FilterSelectOp::lasth = -1;
-std::auto_ptr< PersistentStringList > FilterSelectOp::m_history;
+std::unique_ptr< PersistentStringList > FilterSelectOp::m_history;
 
 bool FilterSelectOp::isName(const char *str)
 {
@@ -329,6 +333,8 @@ FilterSelectOp::FilterSelectOp() : FunctionProto()
     instance_counter++;
     hasConfigure = true;
     m_category = FunctionProto::CAT_SELECTIONS;
+
+    m_mode = 0;
 }
 
 FilterSelectOp::~FilterSelectOp()
@@ -350,18 +356,25 @@ int FilterSelectOp::run( WPUContext *wpu, ActionMessage *msg )
 {
   Lister *l1;
   ListerMode *lm1;
-  NormalMode *nm1;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
     l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       lm1=l1->getActiveMode();
       if(lm1!=NULL) {
-        if(lm1->isType("NormalMode")==true) {
-          /* Requester oeffnen */
-          if(requestfilter()==true) {
-            nm1=(NormalMode*)lm1;
-            nm1->filterselect( current_filter.c_str(), 0 );
-          }
+        if ( auto vdm = dynamic_cast< VirtualDirMode * >( lm1 ) ) {
+            std::list< NM_specialsourceExt > extfilelist;
+            std::string entry_basename;
+
+            lm1->getSelFiles( extfilelist, ListerMode::LM_GETFILES_ONLYACTIVE );
+            for ( auto &ss1 : extfilelist ) {
+                entry_basename = ss1.entry()->name;
+            }
+
+            if ( requestfilter( entry_basename ) == true ) {
+                vdm->selectByFilter( current_filter, true );
+            }
+        } else {
+            lm1->not_supported();
         }
       }
     }
@@ -390,7 +403,7 @@ int FilterSelectOp::configure()
 
   tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(21))+1);
   sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(21));
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe(tstr);
   
@@ -459,7 +472,12 @@ bool FilterSelectOp::save(Datei *fh)
   return true;
 }
 
-bool FilterSelectOp::requestfilter()
+void FilterSelectOp::setMode( int nm )
+{
+    m_mode = nm;
+}
+
+bool FilterSelectOp::requestfilter( const std::string &fullname )
 {
   int w, h;
   StringGadget *sg;
@@ -481,14 +499,18 @@ bool FilterSelectOp::requestfilter()
   const int cmin = AContainer::ACONT_MINH +
                    AContainer::ACONT_MINW;
 
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, catalog.getLocale( 268 ) );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0,
+                     catalog.getLocale( ( m_mode == 1 ) ? 269 : 268 ),
+                     AWindow::AWINDOW_DIALOG );
   win->create();
 
   AContainer *ac1 = win->setContainer( new AContainer( win, 1, 4 ), true );
   ac1->setMinSpace( 5 );
   ac1->setMaxSpace( 5 );
 
-  ac1->add( new Text( aguix, 0, 0, catalog.getLocale( 270 ), 1 ), 0, 0, cincwnr );
+  ac1->add( new Text( aguix, 0, 0,
+                      catalog.getLocale( ( m_mode == 1 ) ? 271 : 270 ),
+                      1 ), 0, 0, cincwnr );
   ac1->add( new Text( aguix, 0, 0, catalog.getLocale( 272 ), 1 ), 0, 1, cincwnr );
 
   AContainer *ac1_1 = ac1->add( new AContainer( win, 1, 2 ), 0, 2 );
@@ -503,6 +525,9 @@ bool FilterSelectOp::requestfilter()
   lv->setDisplayFocus( true );
   lv->setAcceptFocus( true );
 
+  std::string default_value = current_filter.empty() ? filter : current_filter;
+  bool matched = false;
+
   std::list< std::string > cur_history = m_history->getList();
   for ( std::list< std::string >::const_iterator it1 = cur_history.begin();
         it1 != cur_history.end();
@@ -510,13 +535,40 @@ bool FilterSelectOp::requestfilter()
       row = lv->insertRow( 0 );
       lv->setText( row, 0, *it1 );
       lv->setPreColors( row, FieldListView::PRECOLOR_ONLYACTIVE );
+
+      if ( ! matched && ! fullname.empty() && *it1 != "*" ) {
+          StringMatcherFNMatch matcher;
+
+          matcher.setMatchString( *it1 );
+
+          if ( matcher.match( fullname ) ) {
+              default_value = *it1;
+
+              // the latest entry is at the beginning, so stop matching
+              // after the first hit
+              matched = true;
+          }
+      }
   }
   lv->showRow( cur_history.size() - 1 );
   lv->redraw();
 
+  if ( ! matched ) {
+      std::string::size_type dot_pos = fullname.rfind( "." );
+
+      if ( dot_pos != std::string::npos ) {
+          std::string suffix( fullname, dot_pos );
+
+          default_value = "*";
+          default_value += suffix;
+      }
+  }
+
   sg=(StringGadget*)ac1_1->add( new StringGadget( aguix, 0, 0, 100,
-						  current_filter.empty() ? filter.c_str() : current_filter.c_str(),
+						  default_value.c_str(),
 						  0 ), 0, 1, cincw );
+
+  sg->selectAll( false );
   
   AContainer *ac1_2 = ac1->add( new AContainer( win, 2, 1 ), 0, 3 );
   ac1_2->setMinSpace( 5 );
@@ -537,8 +589,15 @@ bool FilterSelectOp::requestfilter()
 						     0,
 						     0 ), 1, 0, cfix );
   win->contMaximize( true );
-  w = ( lastw > 0 ) ? lastw : win->getWidth();
-  h = ( lasth > 0 ) ? lasth : win->getHeight();
+
+  w = Worker::getKVPStore().getIntValue( "filterselect-width",
+                                         win->getWidth(),
+                                         10000 );
+  h = Worker::getKVPStore().getIntValue( "filterselect-height",
+                                         win->getHeight(),
+                                         10000 );
+  //w = ( lastw > 0 ) ? lastw : win->getWidth();
+  //h = ( lasth > 0 ) ? lasth : win->getHeight();
 
   win->setDoTabCycling( true );
   win->resize(w,h);
@@ -608,6 +667,12 @@ bool FilterSelectOp::requestfilter()
       current_filter = sg->getText();
       addHistoryItem( current_filter );
   }
+
+  Worker::getKVPStore().setIntValue( "filterselect-width",
+                                     win->getWidth() );
+  Worker::getKVPStore().setIntValue( "filterselect-height",
+                                     win->getHeight() );
+
   lastw = win->getWidth();
   lasth = win->getHeight();
   delete win;
@@ -621,7 +686,7 @@ void FilterSelectOp::initHistory()
     str1 = WorkerInitialSettings::getInstance().getConfigBaseDir();
     str1 = NWC::Path::join( str1, "filterselect-history" );
 
-    m_history = std::auto_ptr< PersistentStringList >( new PersistentStringList( str1 ) );
+    m_history = std::unique_ptr< PersistentStringList >( new PersistentStringList( str1 ) );
 }
 
 void FilterSelectOp::closeHistory()
@@ -673,6 +738,7 @@ const char *FilterUnSelectOp::getName()
 FilterUnSelectOp::FilterUnSelectOp() : FilterSelectOp()
 {
     m_category = FunctionProto::CAT_SELECTIONS;
+    setMode( 1 );
 }
 
 FilterUnSelectOp::~FilterUnSelectOp()
@@ -690,18 +756,25 @@ int FilterUnSelectOp::run( WPUContext *wpu, ActionMessage *msg )
 {
   Lister *l1;
   ListerMode *lm1;
-  NormalMode *nm1;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
     l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       lm1=l1->getActiveMode();
       if(lm1!=NULL) {
-        if(lm1->isType("NormalMode")==true) {
-          /* Requester oeffnen */
-          if(requestfilter()==true) {
-            nm1=(NormalMode*)lm1;
-            nm1->filterselect( current_filter.c_str(), 1 );
-          }
+        if ( auto vdm = dynamic_cast< VirtualDirMode * >( lm1 ) ) {
+            std::list< NM_specialsourceExt > extfilelist;
+            std::string entry_basename;
+
+            lm1->getSelFiles( extfilelist, ListerMode::LM_GETFILES_ONLYACTIVE );
+            for ( auto &ss1 : extfilelist ) {
+                entry_basename = ss1.entry()->name;
+            }
+
+            if ( requestfilter( entry_basename ) == true ) {
+                vdm->selectByFilter( current_filter, false );
+            }
+        } else {
+            lm1->not_supported();
         }
       }
     }
@@ -730,7 +803,7 @@ int FilterUnSelectOp::configure()
 
   tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(22))+1);
   sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(22));
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe(tstr);
   
@@ -793,156 +866,6 @@ int FilterUnSelectOp::configure()
   return (ende==1)?0:1;
 }
 
-bool FilterUnSelectOp::requestfilter()
-{
-  StringGadget *sg;
-  FieldListView *lv;
-  AGUIX *aguix = Worker::getAGUIX();
-  AWindow *win;
-  int row;
-  bool lvstill;
-  int lastrow;
-  const int cincw = AContainer::ACONT_MINH +
-                    AContainer::ACONT_MINW +
-                    AContainer::ACONT_MAXH;
-  const int cincwnr = cincw +
-                      AContainer::ACONT_NORESIZE;
-  const int cfix = AContainer::ACONT_MINH +
-                   AContainer::ACONT_MINW +
-                   AContainer::ACONT_MAXH +
-                   AContainer::ACONT_MAXW;
-  const int cmin = AContainer::ACONT_MINH +
-                   AContainer::ACONT_MINW;
-
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, catalog.getLocale( 269 ) );
-  win->create();
-
-  AContainer *ac1 = win->setContainer( new AContainer( win, 1, 4 ), true );
-  ac1->setMinSpace( 5 );
-  ac1->setMaxSpace( 5 );
-
-  ac1->add( new Text( aguix, 0, 0, catalog.getLocale( 271 ), 1 ), 0, 0, cincwnr );
-  ac1->add( new Text( aguix, 0, 0, catalog.getLocale( 272 ), 1 ), 0, 1, cincwnr );
-
-  AContainer *ac1_1 = ac1->add( new AContainer( win, 1, 2 ), 0, 2 );
-  ac1_1->setMinSpace( 0 );
-  ac1_1->setMaxSpace( 0 );
-  ac1_1->setBorderWidth( 0 );
-
-  lv = (FieldListView*)ac1_1->add( new FieldListView( aguix, 0, 0, 100,
-						      7 * aguix->getCharHeight(), 0 ), 0, 0, cmin );
-
-  lv->setHBarState( 2 );
-  lv->setVBarState( 2 );
-  lv->setDisplayFocus( true );
-  lv->setAcceptFocus( true );
-
-  std::list< std::string > cur_history = m_history->getList();
-  for ( std::list< std::string >::const_iterator it1 = cur_history.begin();
-        it1 != cur_history.end();
-        it1++ ) {
-      row = lv->insertRow( 0 );
-      lv->setText( row, 0, *it1 );
-      lv->setPreColors( row, FieldListView::PRECOLOR_ONLYACTIVE );
-  }
-  lv->showRow( cur_history.size() - 1 );
-  lv->redraw();
-
-  sg = (StringGadget*)ac1_1->add( new StringGadget( aguix, 0, 0, 100,
-                                                    current_filter.empty() ? filter.c_str() : current_filter.c_str(),
-						    0 ), 0, 1, cincw );
-  
-  AContainer *ac1_2 = ac1->add( new AContainer( win, 2, 1 ), 0, 3 );
-  ac1_2->setMinSpace( 5 );
-  ac1_2->setMaxSpace( -1 );
-  ac1_2->setBorderWidth( 0 );
-  Button *okb =(Button*)ac1_2->add( new Button( aguix,
-                                                0,
-                                                0,
-                                                catalog.getLocale( 11 ),
-                                                1,
-                                                0,
-                                                0 ), 0, 0, cfix );
-  Button *cancelb = (Button*)ac1_2->add( new Button( aguix,
-						     0,
-						     0,
-						     catalog.getLocale( 8 ),
-						     1,
-						     0,
-						     0 ), 1, 0, cfix );
-
-  win->setDoTabCycling( true );
-  win->contMaximize( true );
-  win->show();
-  AGMessage *msg;
-  while((msg=aguix->GetMessage(NULL))!=NULL) aguix->ReplyMessage(msg);
-  int ende=0;
-  sg->takeFocus();
-  lastrow = -1;
-  lvstill = false;
-  while(ende==0) {
-    msg=aguix->WaitMessage(win);
-    if(msg!=NULL) {
-      if(msg->type==AG_CLOSEWINDOW) {
-        if(msg->closewindow.window==win->getWindow()) ende=-1;
-      } else if(msg->type==AG_BUTTONCLICKED) {
-        if(msg->button.button==okb) ende=1;
-        else if(msg->button.button==cancelb) ende=-1;
-      } else if(msg->type==AG_STRINGGADGET_OK) {
-        if(msg->stringgadget.sg==sg) ende=1;
-      } else if(msg->type==AG_STRINGGADGET_CANCEL) {
-        if(msg->stringgadget.sg==sg) ende=-1;
-      } else if ( msg->type == AG_KEYPRESSED ) {
-        if ( win->isParent( msg->key.window, false ) == true ) {
-          switch ( msg->key.key ) {
-          case XK_Up:
-            if ( sg->isActive() == true ) {
-              lv->setActiveRow( lv->getElements() - 1 );
-              lv->takeFocus();
-              lv->showActive();
-              lastrow = lv->getActiveRow();
-              sg->setText( lv->getText( lastrow, 0 ).c_str() );
-            }
-            lvstill = false;
-            break;
-          case XK_Down:
-            if ( lv->getHasFocus() == true ) {
-              if ( ( lv->getActiveRow() == ( lv->getElements() - 1 ) ) &&
-                 ( lvstill == true ) ) {
-                lv->setActiveRow( -1 );
-                sg->takeFocus();
-              }
-            }
-            break;
-          case XK_Escape:
-            ende = -1;
-            break;
-          case XK_Return:
-            if ( cancelb->getHasFocus() == false )
-              ende = 1;
-            break;
-          }
-        }
-      } else if ( msg->type == AG_FIELDLV_ONESELECT ) {
-        row = lv->getActiveRow();
-        if ( row == lastrow ) lvstill = true;
-        else lvstill = false;
-        lastrow = row;
-        if ( lv->isValidRow( row ) == true ) {
-            sg->setText( lv->getText( row, 0 ).c_str() );
-        }
-      }
-      aguix->ReplyMessage(msg);
-    }
-  }
-  if(ende==1) {
-      current_filter = sg->getText();
-      addHistoryItem( current_filter );
-  }
-  delete win;
-  return (ende==1?true:false);
-}
-
 /*************
  Ende FilterUnSelectOp
 **************/
@@ -979,36 +902,37 @@ Path2OSideOp *Path2OSideOp::duplicate() const
 
 int Path2OSideOp::run( WPUContext *wpu, ActionMessage *msg )
 {
-  Lister *l1;
-  ListerMode *lm1;
-  NormalMode *nm1;
-  const char *dir=NULL;
-  if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    l1 = msg->getWorker()->getActiveLister();
-    if(l1!=NULL) {
-      lm1=l1->getActiveMode();
-      if(lm1!=NULL) {
-        if(lm1->isType("NormalMode")==true) {
-          nm1=(NormalMode*)lm1;
-          dir = nm1->getCurrentDir();
+    Lister *l1;
+    ListerMode *lm1;
+
+    std::string dir;
+
+    if(msg->mode!=msg->AM_MODE_DNDACTION) {
+        l1 = msg->getWorker()->getActiveLister();
+        if(l1!=NULL) {
+            lm1=l1->getActiveMode();
+            if(lm1!=NULL) {
+                dir = lm1->getCurrentDirectory();
+            }
         }
-      }
-    }
-    if(dir!=NULL) {
-      l1 = msg->getWorker()->getOtherLister( msg->getWorker()->getActiveLister());
-      if(l1!=NULL) {
-        l1->switch2Mode(0);
-        lm1=l1->getActiveMode();
-        if(lm1!=NULL) {
-          if(lm1->isType("NormalMode")==true) {
-            nm1=(NormalMode*)lm1;
-            nm1->enterDir(dir);
-          }
+        if ( ! dir.empty() ) {
+            l1 = msg->getWorker()->getOtherLister( msg->getWorker()->getActiveLister());
+            if(l1!=NULL) {
+                if ( l1->getActiveMode() == NULL ) {
+                    l1->switch2Mode( 0 );
+                }
+                lm1=l1->getActiveMode();
+                if(lm1!=NULL) {
+                    std::list< RefCount< ArgClass > > args;
+
+                    args.push_back( new StringArg( dir ) );
+                    lm1->runCommand( "enter_dir", args );
+                }
+            }
         }
-      }
     }
-  }
-  return 0;
+
+    return 0;
 }
 
 const char *Path2OSideOp::getDescription()
@@ -1082,7 +1006,7 @@ int QuitOp::configure()
   
   tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(24))+1);
   sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(24));
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe(tstr);
 
