@@ -1,6 +1,6 @@
 /* nwc_fsentry.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2006,2009,2013 Ralf Hoffmann.
+ * Copyright (C) 2006-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -495,4 +495,23 @@ namespace NWC
 
         return 0;
     }
+
+    void FSEntry::resetToDummy()
+    {
+        memset( &_dest_statbuf, 0, sizeof( _dest_statbuf ) );
+
+        _statbuf.lastaccess = 0;
+        _statbuf.lastmod = 0;
+        _statbuf.lastchange = 0;
+        _statbuf.userid = 0;
+        _statbuf.groupid = 0;
+        _statbuf.inode = 0;
+        _statbuf.nlink = 2;
+        _statbuf.rdev = 0;
+        _statbuf.dev = 0;
+
+        _is_link = false;
+        _is_corrupt = false;
+        _exists = true;
+    }
 }
