@@ -1,6 +1,6 @@
 /* nwc_path.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2006,2009-2010,2012-2013 Ralf Hoffmann.
+ * Copyright (C) 2006-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -343,5 +343,23 @@ namespace NWC {
             return res;
         }
 
+        bool is_prefix_dir( const std::string &path1,
+                            const std::string &path2 )
+        {
+            if ( path1.length() > path2.length() ) return false;
+
+            if ( path1 == path2 ) return true;
+
+            if ( ! AGUIXUtils::starts_with( path1, path2 ) ) return false;
+
+            if ( AGUIXUtils::ends_with( path1, "/" ) ) {
+                return true;
+            }
+
+            if ( path2[ path1.length() ] != '/' ) return false;
+
+            return true;
+        }
+
     }
 }
