@@ -414,6 +414,20 @@ std::string NWCEntrySelectionState::getStringRepresentation( WorkerTypes::listco
                                                              int file_name_skip ) const
 {
     std::string res;
+
+    const NWC::FSEntry *fse = getNWCEntry();
+
+    if ( ! fse ) return res;
+
+    return getStringRepresentation( type, file_name_skip, fse, m_filetype );
+}
+
+std::string NWCEntrySelectionState::getStringRepresentation( WorkerTypes::listcol_t type,
+                                                             int file_name_skip,
+                                                             const NWC::FSEntry *fse,
+                                                             const WCFiletype *filetype )
+{
+    std::string res;
     struct tm *timeptr;
     time_t timestamp;
     loff_t dissize, li;
@@ -421,8 +435,6 @@ std::string NWCEntrySelectionState::getStringRepresentation( WorkerTypes::listco
     int ost, x;
     char nbuf[ A_BYTESFORNUMBER( loff_t ) ];
 
-    const NWC::FSEntry *fse = getNWCEntry();
-
     if ( ! fse ) return res;
   
     switch ( type ) {
@@ -491,8 +503,8 @@ std::string NWCEntrySelectionState::getStringRepresentation( WorkerTypes::listco
             }
             break;
         case WorkerTypes::LISTCOL_TYPE:
-            if ( m_filetype != NULL ) {
-                res = m_filetype->getName();
+            if ( filetype != NULL ) {
+                res = filetype->getName();
             } else {
                 if ( fse->isDir( true ) == true ) {
                     // Dir
