@@ -1,6 +1,6 @@
 /* opencontextmenuop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2007,2011 Ralf Hoffmann.
+ * Copyright (C) 2007,2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,7 +21,7 @@
 
 #include "opencontextmenuop.hh"
 #include "listermode.h"
-#include "normalmode.h"
+#include "virtualdirmode.hh"
 #include "worker_locale.h"
 #include "worker.h"
 
@@ -58,16 +58,14 @@ int OpenContextMenuOp::run( WPUContext *wpu, ActionMessage *msg )
 {
     Lister *l1;
     ListerMode *lm1;
-    NormalMode *nm1;
     
     l1 = msg->getWorker()->getActiveLister();
     if ( l1 == NULL )
         return 1;
 
     lm1 = l1->getActiveMode();
-    if ( lm1 != NULL && lm1->isType( "NormalMode" ) == true ) {
-        nm1 = (NormalMode*)lm1;
-        nm1->openContextMenu();
+    if ( auto vdm = dynamic_cast< VirtualDirMode *>( lm1 ) ) {
+        vdm->openContextMenu();
     }
         
     return 0;
