@@ -1,6 +1,6 @@
 /* ownop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2005,2008,2010-2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -20,19 +20,22 @@
  */
 
 #include "ownop.h"
-#include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "filereq.h"
-#include "nmspecialsourceext.hh"
 #include "datei.h"
 #include <aguix/button.h>
 #include <aguix/fieldlistview.h>
 #include <aguix/stringgadget.h>
 #include <aguix/cyclebutton.h>
 #include <aguix/choosebutton.h>
+#include <aguix/acontainer.h>
+#include <aguix/awindow.h>
 #include "worker_locale.h"
 #include "dnd.h"
+#include "wpucontext.h"
+#include "processexitaction.hh"
+#include "execlass.h"
+#include "wconfig.h"
 
 const char *OwnOp::name="OwnOp";
 
@@ -52,6 +55,7 @@ OwnOp::OwnOp() : FunctionProto()
   dontcd = false;
 
   m_use_virtual_temp_copies = true;
+  m_flags_are_relative_to_base_dir = true;
 }
 
 OwnOp::~OwnOp()
@@ -70,6 +74,7 @@ OwnOp::duplicate() const
   ta->take_dirs = take_dirs;
   ta->dontcd = dontcd;
   ta->m_use_virtual_temp_copies = m_use_virtual_temp_copies;
+  ta->m_flags_are_relative_to_base_dir = m_flags_are_relative_to_base_dir;
 
   ta->ownstart=ownstart;
 
@@ -95,19 +100,8 @@ OwnOp::getName()
 int
 OwnOp::run( WPUContext *wpu, ActionMessage *msg )
 {
-  ListerMode *lm1;
-  use_wpu = wpu;
-  if(msg->mode!=msg->AM_MODE_DNDACTION) {
-    Lister *l1 = msg->getWorker()->getActiveLister();
-    if(l1!=NULL) {
-      startlister=l1;
-      endlister = msg->getWorker()->getOtherLister(startlister);
-      lm1=startlister->getActiveMode();
-      if(lm1!=NULL)
-        if(lm1->isType("NormalMode")==true)
-          normalmodeown( msg );
-    }
-  } else normalmodeown( msg );
+  externprog( wpu, msg );
+
   return 0;
 }
 
@@ -145,6 +139,10 @@ OwnOp::save(Datei *fh)
   if ( m_use_virtual_temp_copies != true ) {
       fh->configPutPairBool( "usevirtualtempcopies", m_use_virtual_temp_copies );
   }
+
+  if ( m_flags_are_relative_to_base_dir != true ) {
+      fh->configPutPairBool( "flagsarerelativetobasedir", m_flags_are_relative_to_base_dir );
+  }
   return true;
 }
 
@@ -154,94 +152,13 @@ OwnOp::getDescription()
   return catalog.getLocaleCom(28);
 }
 
-int
-OwnOp::normalmodeown( ActionMessage *am )
+static bool isCorrectViewProg( const char *str )
 {
-  class NM_externorder extorder;
-  NormalMode *nm1=NULL,
-             *nm2=NULL;
-  ListerMode *lm1=NULL;
-  NM_specialsourceExt *specialsource=NULL;
-  
-  if(am->mode==am->AM_MODE_DNDACTION) {
-    lm1 = am->dndmsg->getSourceMode();
-    if(lm1==NULL) return 1;
-    if(lm1->isType("NormalMode")!=true) return 1;
-    nm1=(NormalMode*)lm1;
-    lm1 = am->dndmsg->getDestMode();
-    if(lm1!=NULL)
-      if(lm1->isType("NormalMode")==true)
-        nm2=(NormalMode*)lm1;
-  } else {
-    if(startlister==NULL) return 1;
-    lm1=startlister->getActiveMode();
-    if(lm1==NULL) return 1;
-    if(lm1->isType("NormalMode")!=true) return 1;
-    nm1=(NormalMode*)lm1;
-    if(endlister!=NULL) {
-      lm1=endlister->getActiveMode();
-      if(lm1!=NULL)
-        if(lm1->isType("NormalMode")==true)
-          nm2=(NormalMode*)lm1;
-    }
-  }
-  
-  extorder.wpu = use_wpu;
-  extorder.destmode=NULL;
-  if(am->mode==am->AM_MODE_ONLYACTIVE)
-    extorder.source=extorder.NM_ONLYACTIVE;
-  else if(am->mode==am->AM_MODE_DNDACTION) {
-    // insert DND-element into list
-    extorder.source=extorder.NM_SPECIAL;
-    extorder.sources=new std::list<NM_specialsourceExt*>;
-    specialsource = new NM_specialsourceExt( am->dndmsg->getFE() );
-    extorder.sources->push_back(specialsource);
-    extorder.destmode=nm2;
-  } else if(am->mode==am->AM_MODE_SPECIAL) {
-    extorder.source=extorder.NM_SPECIAL;
-    extorder.sources=new std::list<NM_specialsourceExt*>;
-    specialsource = new NM_specialsourceExt( am->getFE() );
-    extorder.sources->push_back(specialsource);
-  } else
-    extorder.source=extorder.NM_ALLENTRIES;
-
-  extorder.separate_each_entry=separate_each_entry;
-  extorder.recursive=recursive;
-  extorder.inbackground = inbackground;
-  extorder.take_dirs = take_dirs;
-  switch(ownstart) {
-    case OWNOP_START_IN_TERMINAL:
-      extorder.extstart=extorder.NM_EXT_START_IN_TERMINAL;
-      break;
-    case OWNOP_START_IN_TERMINAL_AND_WAIT4KEY:
-      extorder.extstart=extorder.NM_EXT_START_IN_TERMINAL_AND_WAIT4KEY;
-      break;
-    case OWNOP_SHOW_OUTPUT:
-      extorder.extstart=extorder.NM_EXT_SHOW_OUTPUT;
-      break;
-    case OWNOP_SHOW_OUTPUT_INT:
-      extorder.extstart = extorder.NM_EXT_SHOW_OUTPUT_INT;
-      break;
-    default:
-      extorder.extstart=extorder.NM_EXT_START_NORMAL;
-      break;
-  }
-  extorder.com_str=NULL;
-  extorder.view_str=NULL;
-  if(strlen(com_str)>0) extorder.com_str=com_str;
-  if(strlen(view_str)>0) extorder.view_str=view_str;
-
-  extorder.dontcd = dontcd;
-  extorder.m_use_virtual_temp_copies = m_use_virtual_temp_copies;
-
-  nm1->externprog(&extorder);
-
-  if(extorder.source==extorder.NM_SPECIAL) {
-    if ( specialsource != NULL ) delete specialsource;
-    delete extorder.sources;
-  }
-
-  return 0;
+    const char *pos;
+    if ( strlen( str ) < 1 ) return true;
+    pos = strstr( str, "%s" );
+    if ( pos != NULL ) return true;
+    return false;
 }
 
 int
@@ -276,11 +193,11 @@ OwnOp::configure()
 
   tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(28))+1);
   sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(28));
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe(tstr);
 
-  AContainer *ac1 = win->setContainer( new AContainer( win, 1, 10 ), true );
+  AContainer *ac1 = win->setContainer( new AContainer( win, 1, 11 ), true );
   ac1->setMinSpace( 5 );
   ac1->setMaxSpace( 5 );
 
@@ -299,6 +216,9 @@ OwnOp::configure()
   ChooseButton* virtualcb = (ChooseButton*)ac1->add( new ChooseButton( aguix, 0, 0, ( m_use_virtual_temp_copies == true ) ? 1 : 0,
                                                                        catalog.getLocale( 964 ), LABEL_RIGHT, 1, 0 ), 0, 4, cincwnr );
 
+  ChooseButton* relflagscb = (ChooseButton*)ac1->add( new ChooseButton( aguix, 0, 0, ( m_flags_are_relative_to_base_dir == true ) ? 1 : 0,
+                                                                        catalog.getLocale( 1001 ), LABEL_RIGHT, 1, 0 ), 0, 5, cincwnr );
+
   ibcb = (ChooseButton*)ac1->add( new ChooseButton( aguix,
                                                     0,
                                                     0,
@@ -306,9 +226,9 @@ OwnOp::configure()
                                                     catalog.getLocale( 435 ),
                                                     LABEL_RIGHT,
                                                     1,
-                                                    0 ), 0, 5, cincwnr );
+                                                    0 ), 0, 6, cincwnr );
 
-  AContainer *ac1_1 = ac1->add( new AContainer( win, 2, 1 ), 0, 6 );
+  AContainer *ac1_1 = ac1->add( new AContainer( win, 2, 1 ), 0, 7 );
   ac1_1->setMinSpace( 5 );
   ac1_1->setMaxSpace( 5 );
   ac1_1->setBorderWidth( 0 );
@@ -340,7 +260,7 @@ OwnOp::configure()
       break;
   }
   
-  AContainer *ac1_2 = ac1->add( new AContainer( win, 5, 1 ), 0, 7 );
+  AContainer *ac1_2 = ac1->add( new AContainer( win, 5, 1 ), 0, 8 );
   ac1_2->setMinSpace( 0 );
   ac1_2->setMaxSpace( 0 );
   ac1_2->setBorderWidth( 0 );
@@ -355,7 +275,7 @@ OwnOp::configure()
   
   fb = (Button*)ac1_2->add( new Button( aguix, 0, 0, "O", 1, 0, 0 ), 4, 0, cfix );
   
-  AContainer *ac1_3 = ac1->add( new AContainer( win, 4, 1 ), 0, 8 );
+  AContainer *ac1_3 = ac1->add( new AContainer( win, 4, 1 ), 0, 9 );
   ac1_3->setMinSpace( 0 );
   ac1_3->setMaxSpace( 0 );
   ac1_3->setBorderWidth( 0 );
@@ -376,7 +296,7 @@ OwnOp::configure()
       view_text->hide();
   }
   
-  AContainer *ac1_4 = ac1->add( new AContainer( win, 2, 1 ), 0, 9 );
+  AContainer *ac1_4 = ac1->add( new AContainer( win, 2, 1 ), 0, 10 );
   ac1_4->setMinSpace( 5 );
   ac1_4->setMaxSpace( -1 );
   ac1_4->setBorderWidth( 0 );
@@ -407,7 +327,7 @@ OwnOp::configure()
           break;
         case AG_BUTTONCLICKED:
           if(msg->button.button==okb) {
-            if(NormalMode::isCorrectViewProg(sg[1]->getText())==false) {
+            if ( isCorrectViewProg( sg[1]->getText() ) == false ) {
               // show requester
               textstr=catalog.getLocale(312);
               buttonstr=(char*)_allocsafe(strlen(catalog.getLocale(313))+1+
@@ -484,7 +404,7 @@ OwnOp::configure()
           break;
 	case AG_STRINGGADGET_DEACTIVATE:
           if(msg->stringgadget.sg==sg[1]) {
-            if(NormalMode::isCorrectViewProg(sg[1]->getText())==false) {
+            if ( isCorrectViewProg( sg[1]->getText() ) == false ) {
               // show requester
               textstr=catalog.getLocale(312);
               buttonstr=(char*)_allocsafe(strlen(catalog.getLocale(313))+1+
@@ -527,6 +447,7 @@ OwnOp::configure()
     take_dirs = tdcb->getState();
     dontcd = dcdcb->getState();
     m_use_virtual_temp_copies = virtualcb->getState();
+    m_flags_are_relative_to_base_dir = relflagscb->getState();
     inbackground = ibcb->getState();
     _freesafe(com_str);
     _freesafe(view_str);
@@ -581,7 +502,7 @@ OwnOp::getFlag()
                    AContainer::ACONT_MAXH +
                    AContainer::ACONT_MAXW;
   
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, catalog.getLocale( 338 ) );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, catalog.getLocale( 338 ), AWindow::AWINDOW_DIALOG );
   win->create();
 
   AContainer *ac1 = win->setContainer( new AContainer( win, 1, 3 ), true );
@@ -713,3 +634,166 @@ void OwnOp::setUseVirtualTempCopies( bool nv )
 {
     m_use_virtual_temp_copies = nv;
 }
+
+void OwnOp::setFlagsAreRelativeToBaseDir( bool nv )
+{
+    m_flags_are_relative_to_base_dir = nv;
+}
+
+void OwnOp::externprog( WPUContext *wpu, ActionMessage *msg )
+{
+    char *tmpname, *tmpoutput, *tstr;
+    Datei *fp;
+    int lastsize[2];
+    bool useint;
+    bool cancel = false;
+    bool oldrec, oldtd;
+    std::string string1;
+    bool removeTMPFiles;
+
+    if ( com_str == NULL ) return;  // nothing to do
+    if ( wpu == NULL ) return;
+
+    oldrec = wpu->getRecursive();
+    oldtd = wpu->getTakeDirs();
+    wpu->setRecursive( recursive );
+    wpu->setTakeDirs( take_dirs );
+    wpu->setUseVirtualTempCopies( m_use_virtual_temp_copies );
+    wpu->setUseExtendedBasename( m_flags_are_relative_to_base_dir );
+
+    tmpname = Datei::createTMPName();
+    tmpoutput = Datei::createTMPName();
+
+    if ( tmpname != NULL && tmpoutput != NULL ) {
+        // runCommand will remove the tmpfiles but if we do runCommand
+        // (because of an error) remove them here
+        removeTMPFiles = true;
+
+        std::string tmpoutput_lnk;
+
+        RefCount< GenericCallbackArg< void, int > > pea( NULL );
+      
+        wpu->ref();
+
+        if ( ownstart == OWNOP_SHOW_OUTPUT_INT ) {
+            tmpoutput_lnk = Datei::createTMPHardlink( tmpoutput );
+            pea = new ProcessExitAction( true,
+                                         tmpoutput_lnk,
+                                         msg->getWorker(), wpu );
+        } else {
+            pea = new ProcessExitAction( false,
+                                         "",
+                                         msg->getWorker(), wpu );
+        }
+
+        fp = new Datei();
+        if ( fp->open( tmpname, "w" ) == 0 ) {
+            fp->putString( "#! /bin/sh\n" );
+            if ( wpu->getBaseDir() != NULL ) {
+                if ( ( worker_islocal( wpu->getBaseDir() ) == 1 ) &&
+                     ( dontcd == false ) ) {
+                    tstr = AGUIX_catTrustedAndUnTrusted( "cd ", wpu->getBaseDir() );
+                    fp->putString( tstr );
+                    _freesafe( tstr );
+                    fp->putString( "\n" );
+                }
+            }
+            lastsize[0] = wpu->filelistRealSize( false );
+            lastsize[1] = wpu->filelistRealSize( true );
+
+            for ( int loopnr = 1;; loopnr++ ) {
+                std::string res_str;
+                if ( wpu->parse( com_str, res_str, EXE_STRING_LEN - strlen( tmpoutput ) - 3 ) == WPUContext::PARSE_SUCCESS ) {
+                    string1 = Worker_secureCommandForShell( res_str.c_str() );
+                    if ( string1.length() > 0 ) {
+                        if ( ownstart == OWNOP_SHOW_OUTPUT ||
+                             ownstart == OWNOP_SHOW_OUTPUT_INT ) {
+                            tstr = (char*)_allocsafe( strlen( string1.c_str() ) + 3 + strlen( tmpoutput ) + 1 );
+                            if ( loopnr == 1 ) sprintf( tstr, "%s >%s", string1.c_str(), tmpoutput );
+                            else sprintf( tstr, "%s >>%s", string1.c_str(), tmpoutput );
+                            fp->putString( tstr);
+                            _freesafe( tstr );
+                        } else {
+                            fp->putString( string1.c_str() );
+                        }
+                        fp->putString( "\n" );
+                    }
+                    if ( separate_each_entry == false ) break;
+                    if ( wpu->filelistRealSize( false ) == 0 ) break;
+                    if ( wpu->filelistRealSize( false ) == lastsize[0] ) {
+                        // endless loop
+                        //TODO: requester
+                        break;
+                    }
+                } else {
+                    cancel = true;
+                    break;
+                }
+                lastsize[0] = wpu->filelistRealSize( false );
+            }
+            if(  cancel == false ) {
+                std::string exestr;
+
+                if ( ( ownstart == OWNOP_START_IN_TERMINAL_AND_WAIT4KEY ) ||
+                     ( ownstart == OWNOP_START_IN_TERMINAL ) ) {
+                    /* in terminal starten */
+                    if ( ownstart == OWNOP_START_IN_TERMINAL_AND_WAIT4KEY ) {
+                        fp->putStringExt( "echo %s\n", catalog.getLocale( 201 ) );
+                        fp->putString( "read x\n" );
+                    }
+                    exestr = AGUIXUtils::formatStringToString( wconfig->getTerminalStr(),
+                                                               tmpname );
+                } else {
+                    exestr = "/bin/sh ";
+                    exestr += tmpname;
+                }
+                if ( ownstart == OWNOP_SHOW_OUTPUT ) {
+                    useint = true;
+                    if ( view_str != NULL ) {
+                        if ( strlen( view_str ) > 2 ) {
+                            if ( strstr( view_str, "%s" ) != NULL ) useint = false;
+                        }
+                    }
+                    if ( useint == true ) fp->putStringExt( OUTPUT_BIN, tmpoutput );
+                    else fp->putStringExt( view_str, tmpoutput );
+                    fp->putString( "\n" );
+                }
+                fp->close();
+      
+                if ( fp->errors() == 0 ) {
+                    worker_chmod( tmpname, 0700 );
+                    removeTMPFiles = false;
+                    msg->getWorker()->runCommand( exestr.c_str(), tmpname, tmpoutput, inbackground, pea );
+                } else {
+                    std::string text = AGUIXUtils::formatStringToString( catalog.getLocale( 647 ), tmpname );
+                    std::string button = catalog.getLocale( 11 );
+                    Worker::getRequester()->request( catalog.getLocale( 347 ), text.c_str(), button.c_str() );
+                }
+            } else fp->close();
+        }
+        delete fp;
+        if ( removeTMPFiles == true ) {
+            worker_unlink( tmpname );
+            worker_unlink( tmpoutput );
+
+            if ( pea.getVal() != NULL ) {
+                pea->callback( 0 );
+            }
+        }
+    } else {
+        std::string text = AGUIXUtils::formatStringToString( catalog.getLocale( 647 ), "" );
+        std::string button = catalog.getLocale( 11 );
+        Worker::getRequester()->request( catalog.getLocale( 347 ), text.c_str(), button.c_str() );
+    }
+  
+    if ( tmpname != NULL ) _freesafe(tmpname);
+    if ( tmpoutput != NULL ) _freesafe(tmpoutput);
+
+    wpu->setRecursive( oldrec );
+    wpu->setTakeDirs( oldtd );
+}
+
+std::string OwnOp::getStringRepresentation()
+{
+    return com_str;
+}
