@@ -1,6 +1,6 @@
 /* pathjumpui.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2012 Ralf Hoffmann.
+ * Copyright (C) 2012-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -40,6 +40,9 @@
 #include "prefixdb.hh"
 #include "wconfig.h"
 #include "stringmatcher_flexiblematch.hh"
+#include "pers_kvp.hh"
+
+int PathJumpUI::s_pathjump_number = 1;
 
 PathJumpUI::PathJumpUI( AGUIX &aguix, DeepPathStore &path_store, Worker &worker )
   : m_aguix( aguix ),
@@ -128,13 +131,14 @@ PathJumpUI::PathJumpUI( AGUIX &aguix, DeepPathStore &path_store, Worker &worker
     // add one for the initial slash
     max_path_components++;
 
-    m_win = std::auto_ptr<AWindow>( new AWindow( &m_aguix,
-                                                 0, 0,
-                                                 400, 400,
-                                                 0, catalog.getLocaleCom( 60 ) ) );
+    m_win = std::unique_ptr<AWindow>( new AWindow( &m_aguix,
+                                                   0, 0,
+                                                   400, 400,
+                                                   0, catalog.getLocaleCom( 60 ),
+                                                   AWindow::AWINDOW_DIALOG ) );
     m_win->create();
 
-    m_co1 = m_win->setContainer( new AContainer( m_win.get(), 1, 6 ), true );
+    m_co1 = m_win->setContainer( new AContainer( m_win.get(), 1, 7 ), true );
     m_co1->setMaxSpace( 5 );
     
     AContainer *co1_2 = m_co1->add( new AContainer( m_win.get(), 2, 1 ), 0, 0 );
@@ -146,7 +150,19 @@ PathJumpUI::PathJumpUI( AGUIX &aguix, DeepPathStore &path_store, Worker &worker
                                                   1, 0, AContainer::CO_INCW ) );
 
 
-    AContainer *co1_4 = m_co1->add( new AContainer( m_win.get(), 2, 1 ), 0, 1 );
+    m_hold_co = m_co1->add( new AContainer( m_win.get(), 4, 1 ), 0, 1 );
+    m_hold_co->setMaxSpace( 5 );
+    m_hold_co->setBorderWidth( 0 );
+    m_hold_co->add( new Text( &m_aguix, 0, 0, catalog.getLocale( 1029 ), 1 ),
+                0, 0, AContainer::CO_FIX );
+    m_hold_status_b = m_hold_co->addWidget( new Button( &m_aguix, 0, 0, catalog.getLocale( 1027 ), 1, 0, 0 ),
+                                            1, 0, AContainer::CO_FIX );
+    m_hold_co->add( new Text( &m_aguix, 0, 0, catalog.getLocale( 1030 ), 1 ),
+                2, 0, AContainer::CO_FIX );
+    m_hold_filter_text = m_hold_co->addWidget( new Text( &m_aguix, 0, 0, "", 1 ),
+                                           3, 0, AContainer::CO_INCW );
+
+    AContainer *co1_4 = m_co1->add( new AContainer( m_win.get(), 2, 1 ), 0, 2 );
     co1_4->setMaxSpace( 5 );
     co1_4->setBorderWidth( 0 );
     co1_4->add( new Text( &m_aguix, 0, 0, catalog.getLocale( 980 ), 1 ),
@@ -166,9 +182,9 @@ PathJumpUI::PathJumpUI( AGUIX &aguix, DeepPathStore &path_store, Worker &worker
                                                                  false,
                                                                  catalog.getLocale( 977 ),
                                                                  LABEL_RIGHT, 1, 0 ),
-                                               0, 2, AContainer::CO_INCWNR );
+                                               0, 3, AContainer::CO_INCWNR );
 
-    m_breadcrumb_co = m_co1->add( new AContainer( m_win.get(), max_path_components + 1, 1 ), 0, 3 );
+    m_breadcrumb_co = m_co1->add( new AContainer( m_win.get(), max_path_components + 1, 1 ), 0, 4 );
     m_breadcrumb_co->setMaxSpace( 5 );
 
     for ( int i = 0; i < max_path_components; i++ ) {
@@ -184,10 +200,15 @@ PathJumpUI::PathJumpUI( AGUIX &aguix, DeepPathStore &path_store, Worker &worker
         b->hide();
     }
 
-    m_lv = static_cast<FieldListView*>( m_co1->add( new FieldListView( &m_aguix, 
-                                                                     0, 0, 
-                                                                     400, 300, 0 ),
-                                                  0, 4, AContainer::CO_MIN ) );
+    AContainer *co1_5 = m_co1->add( new AContainer( m_win.get(), 2, 1 ), 0, 5 );
+    co1_5->setMaxSpace( 5 );
+    co1_5->setBorderWidth( 0 );
+
+    m_lv = co1_5->addWidget( new FieldListView( &m_aguix, 
+                                                0, 0, 
+                                                400, 300, 0 ),
+                             0, 0, AContainer::CO_MIN );
+
     m_lv->setNrOfFields( 2 );
     m_lv->setShowHeader( true );
     m_lv->setFieldText( 1, catalog.getLocale( 809 ) );
@@ -198,7 +219,28 @@ PathJumpUI::PathJumpUI( AGUIX &aguix, DeepPathStore &path_store, Worker &worker
     m_lv->setFieldText( 0, catalog.getLocale( 978 ) );
     m_lv->setGlobalFieldSpace( 5 );
 
-    AContainer *co1_3 = m_co1->add( new AContainer( m_win.get(), 3, 1 ), 0, 5 );
+    m_class_counts_lv = co1_5->addWidget( new FieldListView( &m_aguix, 
+                                                             0, 0, 
+                                                             100, 100, 0 ),
+                                          1, 0, AContainer::CO_INCH );
+    m_class_counts_lv->setNrOfFields( 3 );
+    m_class_counts_lv->setShowHeader( true );
+    m_class_counts_lv->setFieldText( 0, catalog.getLocale( 978 ) );
+    m_class_counts_lv->setFieldText( 1, catalog.getLocale( 1031 ) );
+    m_class_counts_lv->setFieldText( 2, catalog.getLocale( 1032 ) );
+
+    m_class_counts_lv->setHBarState( 2 );
+    m_class_counts_lv->setVBarState( 2 );
+    m_class_counts_lv->setAcceptFocus( true );
+    m_class_counts_lv->setDisplayFocus( true );
+    m_class_counts_lv->setGlobalFieldSpace( 5 );
+    m_class_counts_lv->setConsiderHeaderWidthForDynamicWidth( true );
+
+    m_class_counts_lv->maximizeX();
+
+    co1_5->readLimits();
+
+    AContainer *co1_3 = m_co1->add( new AContainer( m_win.get(), 4, 1 ), 0, 6 );
     co1_3->setMinSpace( 5 );
     co1_3->setMaxSpace( -1 );
     co1_3->setBorderWidth( 0 );
@@ -209,34 +251,30 @@ PathJumpUI::PathJumpUI( AGUIX &aguix, DeepPathStore &path_store, Worker &worker
                                              1,
                                              0,
                                              0 ), 0, 0, AContainer::CO_FIX );
+    m_panelize_b = co1_3->addWidget( new Button( &m_aguix,
+                                                 0,
+                                                 0,
+                                                 catalog.getLocale( 1034 ),
+                                                 1,
+                                                 0,
+                                                 0 ), 1, 0, AContainer::CO_FIX );
     m_cleanupb = (Button*)co1_3->add( new Button( &m_aguix,
                                                   0,
                                                   0,
                                                   catalog.getLocale( 984 ),
                                                   1,
                                                   0,
-                                             0 ), 1, 0, AContainer::CO_FIX );
+                                             0 ), 2, 0, AContainer::CO_FIX );
     m_cancelb = (Button*)co1_3->add( new Button( &m_aguix,
                                                  0,
                                                  0,
                                                  catalog.getLocale( 633 ),
                                                  1,
                                                  0,
-                                                 0 ), 2, 0, AContainer::CO_FIX );
+                                                 0 ), 3, 0, AContainer::CO_FIX );
 
     m_win->contMaximize( true );
     m_win->setDoTabCycling( true );
-
-    std::string cfgfile = Worker::getWorkerConfigDir();
-#ifdef USEOWNCONFIGFILES
-    cfgfile = NWC::Path::join( cfgfile, "directory-history.prob2" );
-#else
-    cfgfile = NWC::Path::join( cfgfile, "directory-history.prob" );
-#endif
-
-    // make this instance static in this class or worker to avoid
-    // reparsing it
-    m_pdb = std::auto_ptr< PrefixDB >( new PrefixDB( cfgfile ) );
 }
 
 PathJumpUI::~PathJumpUI()
@@ -252,6 +290,15 @@ int PathJumpUI::mainLoop()
     highlight_best_hit( m_dirname );
 
     maximizeWin();
+
+    int tw = Worker::getKVPStore().getIntValue( "pathjumpwin-width",
+                                                m_win->getWidth(),
+                                                10000 );
+    int th = Worker::getKVPStore().getIntValue( "pathjumpwin-height",
+                                                m_win->getHeight(),
+                                                10000 );
+    m_win->resize( tw, th );
+
     m_win->show();
 
     m_lv->takeFocus();
@@ -272,8 +319,16 @@ int PathJumpUI::mainLoop()
                   } else if ( msg->button.button == m_cleanupb ) {
                       cleanup();
                       endmode = -1;
+                  } else if ( msg->button.button == m_panelize_b ) {
+                      endmode = 2;
                   } else if ( msg->button.button == m_cancelb ) {
                       endmode = -1;
+                  } else if ( msg->button.button == m_hold_status_b ) {
+                      if ( m_hold_status.active ) {
+                          resetHeldEntriesAndUpdateUI();
+                      } else {
+                          showHoldHelp();
+                      }
                   } else {
                       for ( unsigned int i = 0; i < m_breadcrumb_buttons.size(); i++ ) {
                           if ( m_breadcrumb_buttons[i] == msg->button.button ) {
@@ -360,13 +415,22 @@ int PathJumpUI::mainLoop()
                               IsFunctionKey( msg->key.key ) ||
                               IsMiscFunctionKey( msg->key.key ) ||
                               ( ( msg->key.key >= XK_BackSpace ) && ( msg->key.key <= XK_Escape ) ) ) {
+                  } else if ( msg->key.key >= XK_0 && msg->key.key <= XK_9 &&
+                              KEYSTATEMASK( msg->key.keystate ) == ControlMask ) {
+                      if ( msg->key.key == XK_0 ) {
+                          resetHeldEntriesAndUpdateUI();
+                      } else {
+                          holdCurrentEntriesAndUpdateUI( msg->key.key - XK_0 );
+                      }
                   } else if ( strlen( msg->key.keybuf ) > 0 ) {
                       if ( msg->key.key == XK_a &&
                            KEYSTATEMASK( msg->key.keystate ) == ControlMask ) {
-                          m_show_hidden_entries = ! m_show_hidden_entries;
-                          m_show_all_cb->setState( m_show_hidden_entries );
-                          apply_filter( m_filter );
-                          showData();
+                          if ( ! m_hold_status.active ) {
+                              m_show_hidden_entries = ! m_show_hidden_entries;
+                              m_show_all_cb->setState( m_show_hidden_entries );
+                              apply_filter( m_filter );
+                              showData();
+                          }
                       } else if ( msg->key.key == XK_f &&
                            KEYSTATEMASK( msg->key.keystate ) == ControlMask ) {
                           switch ( m_entry_filter_mode ) {
@@ -400,17 +464,16 @@ int PathJumpUI::mainLoop()
                   if ( msg->fieldlv.lv == m_lv ) {
                       // double click in lv, actual element is unimportant here
                       endmode = 1;
+                  } else if ( msg->fieldlv.lv == m_class_counts_lv ) {
+                      holdCurrentEntriesAndUpdateUI( msg->fieldlv.row + 1 );
                   }
                   break;
                 case AG_FIELDLV_ONESELECT:
                 case AG_FIELDLV_MULTISELECT:
                     if ( msg->fieldlv.lv == m_lv ) {
-                        try {
-                            m_current_entry = m_entry_fullnames.at( m_lv->getActiveRow() );
-                            set_entry( m_current_entry );
-                            showData();
-                        } catch (...) {
-                        }
+                        handleMainLVRowChange( m_lv->getActiveRow() );
+                    } else if ( msg->fieldlv.lv == m_class_counts_lv ) {
+                        handleMatchClassRow( msg->fieldlv.row );
                     }
                     break;
                 case AG_CHOOSECLICKED:
@@ -444,7 +507,8 @@ int PathJumpUI::mainLoop()
 
     m_selected_path = "";
 
-    if ( endmode == 1 ) {
+    if ( endmode == 2 ) {
+    } else if ( endmode == 1 ) {
         std::string res = "";
 
         if ( m_current_depth <= 0 ) {
@@ -469,13 +533,20 @@ int PathJumpUI::mainLoop()
                   it1 != l.end();
                   it1++ ) {
                 if ( AGUIXUtils::starts_with( m_selected_path, *it1 ) ) {
-                    m_pdb->pushAccess( m_filter, m_selected_path );
+                    m_worker.getDirHistPrefixDB().pushAccess( m_filter, m_selected_path );
                     break;
                 }
             }
         }
+
+        m_worker.setPathJumpInProgress( m_filter );
     }
 
+    Worker::getKVPStore().setIntValue( "pathjumpwin-width",
+                                       m_win->getWidth() );
+    Worker::getKVPStore().setIntValue( "pathjumpwin-height",
+                                       m_win->getHeight() );
+
     m_win->hide();
     
     return endmode;
@@ -483,6 +554,10 @@ int PathJumpUI::mainLoop()
 
 void PathJumpUI::showData()
 {
+    StringMatcherFlexibleMatch matcher;
+
+    matcher.setMatchString( m_filter );
+
     m_infixtext->setText( m_filter.c_str() );
 
     if ( m_lv_dirty ||
@@ -518,22 +593,32 @@ void PathJumpUI::showData()
               it1++, pos++ ) {
             if ( ( it1->m_type == PATH_SHOW ||
                    it1->m_type == BOOKMARK_SHOW ) &&
-                 it1->m_post_filtering ) {
+                 it1->m_post_filtering &&
+                 it1->m_always_ignore == false ) {
                 int row = m_lv->addRow();
 
                 m_entry_fullnames.push_back( it1->m_path );
 
+                std::string visibile_name = it1->m_path;
+
                 if ( m_entry_filter_mode == SHOW_ONLY_DIRECT_SUBDIRS ) {
                     size_t p2 = it1->m_path.find( '/', l + 1 );
                     if ( p2 != std::string::npos ) {
                         std::string s( it1->m_path, 0, p2 );
 
-                        m_lv->setText( row, 1, s );
-                    } else {
-                        m_lv->setText( row, 1, it1->m_path );
+                        visibile_name = s;
+                    }
+                }
+
+                m_lv->setText( row, 1, visibile_name );
+
+                if ( ! m_filter.empty() ) {
+                    std::vector< size_t > segments;
+                    matcher.countNonMatchingBlocks( visibile_name, &segments );
+
+                    if ( segments.size() > 1 ) {
+                        m_lv->setHighlightSegments( row, 1, segments );
                     }
-                } else {
-                    m_lv->setText( row, 1, it1->m_path );
                 }
 
                 if ( it1->m_blockcount >= 0 && ! m_filter.empty() ) {
@@ -563,6 +648,46 @@ void PathJumpUI::showData()
             }
         }
 
+        calculateMatchClasses();
+
+        m_class_counts_lv->setSize( 0 );
+
+        for ( auto &e : m_match_classes ) {
+            int row = m_class_counts_lv->addRow();
+
+            std::string t1;
+
+            if ( e.first >= 0 ) {
+                t1 = AGUIXUtils::formatStringToString( "%3.0f%%",
+                                                       100.0 / (double)( e.first + 1 ) );
+
+                m_class_counts_lv->setText( row, 0, t1 );
+            } else {
+                m_class_counts_lv->setText( row, 0, "100%" );
+            }
+
+            if ( row == 0 ) {
+                t1 = AGUIXUtils::formatStringToString( "%d",
+                                                       e.second );
+            } else {
+                t1 = AGUIXUtils::formatStringToString( "+%d",
+                                                       e.second );
+            }
+
+            m_class_counts_lv->setText( row, 1, t1 );
+
+            if ( row < 9 ) {
+                t1 = AGUIXUtils::formatStringToString( "Ctrl-%d",
+                                                       row + 1 );
+
+                m_class_counts_lv->setText( row, 2, t1 );
+            }
+
+            m_class_counts_lv->setPreColors( row, FieldListView::PRECOLOR_ONLYACTIVE );
+        }
+
+        m_class_counts_lv->redraw();
+
         m_lv->showActive();
         m_lv->redraw();
     }
@@ -608,14 +733,14 @@ void PathJumpUI::maximizeWin()
     m_co1->rearrange();
 
     if ( my_w < m_lv->getWidth() ) {
-        m_co1->setMinWidth( my_w, 0, 4 );
+        m_co1->setMinWidth( my_w, 0, 5 );
     } else {
-        m_co1->setMinWidth( m_lv->getWidth(), 0, 4 );
+        m_co1->setMinWidth( m_lv->getWidth(), 0, 5 );
     }
     if ( my_h < m_lv->getHeight() ) {
-        m_co1->setMinHeight( my_h, 0, 4 );
+        m_co1->setMinHeight( my_h, 0, 5 );
     } else {
-        m_co1->setMinHeight( m_lv->getHeight(), 0, 4 );
+        m_co1->setMinHeight( m_lv->getHeight(), 0, 5 );
     }
     m_win->contMaximize( true );
 }
@@ -656,9 +781,10 @@ int PathJumpUI::apply_filter( const std::string &filter )
     for ( std::list< pathjump_entry >::iterator it1 = m_entries.begin();
           it1 != m_entries.end();
           it1++ ) {
+        if ( it1->m_always_ignore ) continue;
 
         if ( use_flexible ) {
-            it1->m_blockcount = flmatch.countNonMatchingBlocks( it1->m_path );
+            it1->m_blockcount = flmatch.countNonMatchingBlocks( it1->m_path, NULL );
         } else {
             it1->m_blockcount = match.match( it1->m_path ) ? 0 : -1;
         }
@@ -687,6 +813,8 @@ int PathJumpUI::apply_filter( const std::string &filter )
     for ( std::list< pathjump_entry >::iterator it1 = m_entries.begin();
           it1 != m_entries.end();
           it1++ ) {
+        if ( it1->m_always_ignore ) continue;
+
         if ( it1->m_type == PATH_SHOW ||
              it1->m_type == PATH_HIDE ) {
             if ( it1->m_blockcount == 0 ||
@@ -699,7 +827,8 @@ int PathJumpUI::apply_filter( const std::string &filter )
             }
         } else {
             if ( ( path_matches == 0 && ! filter.empty() ) ||
-                 m_show_hidden_entries == true ) {
+                 m_show_hidden_entries == true ||
+                 m_hold_status.active == true ) {
                 if ( it1->m_blockcount == 0 ||
                      ( it1->m_blockcount > 0 &&
                        ( ! prefer_exact_matches || ( prefer_exact_matches && exact_bookmark_matches == 0 ) ) ) ) {
@@ -728,7 +857,7 @@ int PathJumpUI::find_entry( const std::string &name, const std::string &filter )
     std::string best_hit;
 
     if ( ! filter.empty() ) {
-        best_hit = entry_to_find = m_pdb->getBestHit( filter );
+        best_hit = entry_to_find = m_worker.getDirHistPrefixDB().getBestHit( filter );
     }
 
     if ( entry_to_find.empty() ) {
@@ -738,6 +867,8 @@ int PathJumpUI::find_entry( const std::string &name, const std::string &filter )
     for ( std::list< pathjump_entry >::iterator it1 = m_entries.begin();
           it1 != m_entries.end();
           it1++ ) {
+        if ( it1->m_always_ignore ) continue;
+
         if ( it1->m_type == PATH_SHOW ||
              it1->m_type == BOOKMARK_SHOW ) {
             if ( AGUIXUtils::starts_with( it1->m_path,
@@ -785,6 +916,8 @@ int PathJumpUI::set_entry( const std::string &name )
     for ( std::list< pathjump_entry >::iterator it1 = m_entries.begin();
           it1 != m_entries.end();
           it1++ ) {
+        if ( it1->m_always_ignore ) continue;
+
         if ( ( it1->m_type == PATH_SHOW ||
                it1->m_type == BOOKMARK_SHOW ) &&
              it1->m_path == name ) {
@@ -824,6 +957,8 @@ int PathJumpUI::post_filtering()
         for ( std::list< pathjump_entry >::iterator it1 = m_entries.begin();
               it1 != m_entries.end();
               it1++ ) {
+            if ( it1->m_always_ignore ) continue;
+
             if ( it1->m_type == PATH_SHOW ||
                  it1->m_type == BOOKMARK_SHOW ) {
 
@@ -858,6 +993,7 @@ int PathJumpUI::post_filtering()
     for ( std::list< pathjump_entry >::iterator it1 = m_entries.begin();
           it1 != m_entries.end();
           it1++ ) {
+        if ( it1->m_always_ignore ) continue;
 
         bool new_state = false;
 
@@ -1109,3 +1245,213 @@ int PathJumpUI::cleanup()
 
     return 0;
 }
+
+int PathJumpUI::resetHeldEntries()
+{
+    for ( auto &e : m_entries ) {
+        e.m_always_ignore = false;
+    }
+
+    return 0;
+}
+
+void PathJumpUI::resetHeldEntriesAndUpdateUI()
+{
+    resetHeldEntries();
+
+    m_show_all_cb->setState( m_hold_status.base_show_hidden_state );
+    m_show_all_cb->show();
+
+    m_show_hidden_entries = m_hold_status.base_show_hidden_state;
+    m_hold_status.active = false;
+    m_hold_status.current_filter = "";
+
+    m_hold_status_b->setText( 0, catalog.getLocale( 1027 ) );
+    m_hold_filter_text->setText( "" );
+
+    m_hold_co->readLimits();
+
+    m_win->updateCont();
+
+    apply_filter( m_filter );
+    showData();
+}
+
+int PathJumpUI::calculateMatchClasses()
+{
+    std::set< int > match_counts;
+    std::list< int > sorted_counts;
+    std::map< int, int > entries_per_match;
+
+    for ( auto &e : m_entries ) {
+        if ( ( e.m_type == PATH_SHOW ||
+               e.m_type == BOOKMARK_SHOW ) &&
+             e.m_post_filtering &&
+             e.m_always_ignore == false ) {
+            match_counts.insert( e.m_blockcount );
+            entries_per_match[e.m_blockcount]++;
+        }
+    }
+
+    for ( auto &c : match_counts ) {
+        sorted_counts.push_back( c );
+    }
+
+    sorted_counts.sort();
+
+    m_match_classes.clear();
+
+    for ( auto &c : sorted_counts ) {
+        m_match_classes.push_back( std::make_pair( c, entries_per_match[c] ) );
+    }
+
+    return 0;
+}
+
+int PathJumpUI::holdCurrentEntries( int top_x )
+{
+    std::list< std::pair< int, int > > sorted_counts;
+
+    if ( top_x < 1 ) return 1;
+
+    sorted_counts = m_match_classes;
+
+    if ( top_x < (int)sorted_counts.size() ) {
+        sorted_counts.resize( top_x );
+    }
+
+    for ( auto &e : m_entries ) {
+        if ( ( e.m_type == PATH_SHOW ||
+               e.m_type == BOOKMARK_SHOW ) &&
+             e.m_post_filtering &&
+             e.m_always_ignore == false ) {
+            if ( std::find_if( sorted_counts.begin(),
+                               sorted_counts.end(),
+                               [&e]( std::pair< int, int > &mc ) {
+                                   if ( mc.first == e.m_blockcount ) return true;
+                                   return false;
+                               } ) != sorted_counts.end() ) {
+                // blockcount accepted, continue wiht next entry
+                continue;
+            }
+        }
+
+        e.m_always_ignore = true;
+    }
+
+    return 0;
+}
+
+int PathJumpUI::holdCurrentEntriesAndUpdateUI( int top_x )
+{
+    if ( ! m_hold_status.active ) {
+        m_hold_status.base_show_hidden_state = m_show_hidden_entries;
+        m_hold_status.active = true;
+
+        m_show_all_cb->hide();
+
+        m_hold_status_b->setText( 0, catalog.getLocale( 1028 ) );
+        m_hold_status_b->resize( m_hold_status_b->getMaximumWidth(),
+                                 m_hold_status_b->getHeight() );
+    }
+
+    if ( ! m_filter.empty() ) {
+        if ( ! m_hold_status.current_filter.empty() ) {
+            m_hold_status.current_filter += "+";
+        }
+
+        m_hold_status.current_filter += m_filter;
+    }
+
+    m_hold_filter_text->setText( m_hold_status.current_filter.c_str() );
+
+    m_hold_co->readLimits();
+
+    m_win->updateCont();
+
+    int res = holdCurrentEntries( top_x );
+
+    m_filter = "";
+
+    apply_filter( m_filter );
+    showData();
+
+    return res;
+}
+
+int PathJumpUI::handleMatchClassRow( int row )
+{
+    if ( row == 0 ) {
+        m_lv->setActiveRow( 0 );
+
+        handleMainLVRowChange( 0 );
+
+        m_lv->centerActive();
+        m_lv->redraw();
+
+        return 0;
+    }
+
+    std::string s1 = m_class_counts_lv->getText( row, 0 );
+
+    for ( int mainrow = 0; mainrow < m_lv->getElements(); mainrow++ ) {
+        if ( m_lv->getText( mainrow, 0 ) == s1 ) {
+            m_lv->setActiveRow( mainrow );
+
+            handleMainLVRowChange( mainrow );
+
+            m_lv->centerActive();
+            m_lv->redraw();
+            break;
+        }
+    }
+
+    return 0;
+}
+
+int PathJumpUI::handleMainLVRowChange( int row )
+{
+    try {
+        m_current_entry = m_entry_fullnames.at( row );
+        set_entry( m_current_entry );
+        showData();
+    } catch (...) {
+    }
+
+    return 0;
+}
+
+void PathJumpUI::showHoldHelp()
+{
+    RefCount<AFontWidth> lencalc( new AFontWidth( Worker::getAGUIX(), NULL ) );
+
+    std::string message = catalog.getLocale( 1033 );
+
+    TextStorageString ts( message, lencalc );
+
+    std::string buttons = catalog.getLocale( 11 );
+
+    Worker::getRequester()->request( catalog.getLocale( 124 ), ts, buttons.c_str() );
+}
+
+std::unique_ptr< NWC::Dir > PathJumpUI::getResultsAsDir()
+{
+    std::string name = AGUIXUtils::formatStringToString( "pathjump%d", s_pathjump_number++ );
+
+    if ( s_pathjump_number < 0 ) {
+        // avoid negative and zero number
+        s_pathjump_number = 1;
+    }
+
+    std::unique_ptr< NWC::Dir > d( new NWC::VirtualDir( name ) );
+
+    for ( auto &e : m_entry_fullnames ) {
+        NWC::FSEntry fse( e );
+
+        if ( fse.entryExists() ) {
+            d->add( fse );
+        }
+    }
+
+    return d;
+}
