@@ -1,6 +1,6 @@
 /* run_custom_action.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2007-2008,2011 Ralf Hoffmann.
+ * Copyright (C) 2007-2008,2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,7 +21,6 @@
 
 #include "run_custom_action.hh"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "wcfiletype.hh"
 #include "fileentry.hh"
@@ -35,6 +34,7 @@
 #include <aguix/stringgadget.h>
 #include <aguix/fieldlistview.h>
 #include <aguix/button.h>
+#include <aguix/awindow.h>
 #include "datei.h"
 
 const char *RunCustomAction::name = "RunCustomAction";
@@ -69,16 +69,7 @@ RunCustomAction *RunCustomAction::duplicate() const
 
 int RunCustomAction::run( WPUContext *wpu, ActionMessage *msg )
 {
-    ListerMode *lm1;
-    if ( msg->mode != msg->AM_MODE_DNDACTION ) {
-        Lister *l1 = msg->getWorker()->getActiveLister();
-        if ( l1 != NULL ) {
-            lm1=l1->getActiveMode();
-            if ( lm1 != NULL )
-                if ( lm1->isType( "NormalMode" ) == true )
-                    normalmoderuncustomaction( msg );
-        }
-    } else normalmoderuncustomaction( msg );
+    normalmoderuncustomaction( msg );
     return 0;
 }
 
@@ -90,11 +81,9 @@ const char *RunCustomAction::getDescription()
 void RunCustomAction::normalmoderuncustomaction( ActionMessage *am )
 {
     ListerMode *lm1;
-    NormalMode *nm = NULL;
     ActionMessage amsg( am->getWorker() );
     Lister *l1;
     const FileEntry *fe = NULL;
-    NM_specialsourceExt *ss1;
     
     switch ( am->mode ) {
       case ActionMessage::AM_MODE_DNDACTION:
@@ -151,24 +140,22 @@ void RunCustomAction::normalmoderuncustomaction( ActionMessage *am )
       case ActionMessage::AM_MODE_ONLYACTIVE:
       default:
           l1 = am->getWorker()->getActiveLister();
+          lm1 = NULL;
           if ( l1 != NULL ) {
               lm1 = l1->getActiveMode();
-              if ( lm1 != NULL )
-                  if ( lm1->isType( "NormalMode" ) == true ) nm = (NormalMode*)lm1;
           }
-          if ( nm != NULL ) {
-              std::list<NM_specialsourceExt*> *files = new std::list<NM_specialsourceExt*>;
-              std::list<NM_specialsourceExt*>::iterator it1;
-              nm->getSelFiles( files,
-                               ( am->mode == ActionMessage::AM_MODE_ONLYACTIVE ) ? NormalMode::NM_GETFILES_ONLYACTIVE :
-                               NormalMode::NM_GETFILES_SELORACT,
-                               false );
+          if ( lm1 != NULL ) {
+              std::list< NM_specialsourceExt > files;
+              lm1->getSelFiles( files,
+                                ( am->mode == ActionMessage::AM_MODE_ONLYACTIVE ) ? ListerMode::LM_GETFILES_ONLYACTIVE :
+                                ListerMode::LM_GETFILES_SELORACT,
+                                false );
               WCFiletype *ft;
               
               amsg.startLister = l1;
               amsg.mode = amsg.AM_MODE_SPECIAL;
               
-              if ( files->size() == 0 ) {
+              if ( files.size() == 0 ) {
                   ft = wconfig->getvoidtype();
                   if ( ft != NULL ) {
                       amsg.setFE( NULL );
@@ -177,9 +164,8 @@ void RunCustomAction::normalmoderuncustomaction( ActionMessage *am )
                       am->getWorker()->interpret( ft->getCustomActions( m_customname ), &amsg );
                   }
               } else {
-                  for ( it1 = files->begin(); it1 != files->end(); it1++ ) {
-                      ss1 = *it1;
-                      fe = ss1->entry();
+                  for ( auto &ss1 : files ) {
+                      fe = ss1.entry();
                       if ( fe != NULL ) {
                           ft = fe->filetype;
                           if ( ft == NULL ) {
@@ -195,10 +181,8 @@ void RunCustomAction::normalmoderuncustomaction( ActionMessage *am )
                               am->getWorker()->interpret( ft->getCustomActions( m_customname ), &amsg );
                           }
                       }
-                      delete ss1;
                   }
               }
-              delete files;
           }
           break;
     }
@@ -216,7 +200,7 @@ int RunCustomAction::configure()
     tstr = (char*)_allocsafe( strlen( catalog.getLocale( 293 ) ) +
                               strlen( catalog.getLocaleCom( 4 ) ) + 1 );
     sprintf( tstr, catalog.getLocale( 293 ), catalog.getLocaleCom( 4 ) );
-    win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+    win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
     win->create();
     _freesafe( tstr );
 
