@@ -1,6 +1,6 @@
 /* scriptop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2002-2006,2011 Ralf Hoffmann.
+ * Copyright (C) 2002-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,7 +21,6 @@
 
 #include "scriptop.h"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "wpucontext.h"
 #include "execlass.h"
@@ -32,7 +31,10 @@
 #include <aguix/stringgadget.h>
 #include <aguix/button.h>
 #include <aguix/cyclebutton.h>
+#include <aguix/awindow.h>
+#include <aguix/request.h>
 #include "datei.h"
+#include "wconfig.h"
 
 const char *ScriptOp::name="ScriptOp";
 
@@ -125,9 +127,9 @@ ScriptOp::run( WPUContext *wpu, ActionMessage *msg )
         std::string res_str;
 
         if ( wpu->parse( push_string, res_str, a_max( EXE_STRING_LEN - 1024, 256 ),
-                         ( ( push_useoutput == true ) ? true : false ) ) == NormalMode::PARSE_SUCCESS ) {
+                         ( ( push_useoutput == true ) ? true : false ) ) == WPUContext::PARSE_SUCCESS ) {
           if ( push_useoutput == true ) {
-            std::auto_ptr<ExeClass> ec( new ExeClass( ExeClass::EXECUTE_NORMAL, msg->getWorker() ) );
+            std::unique_ptr<ExeClass> ec( new ExeClass( ExeClass::EXECUTE_NORMAL, msg->getWorker() ) );
             ec->cdDir( wpu->getBaseDir() );
             ec->addCommand( "%s", res_str.c_str() );
             tstr2 = ec->getOutput( &exeerror );
@@ -265,9 +267,9 @@ ScriptOp::run( WPUContext *wpu, ActionMessage *msg )
         if ( progress != NULL ) {
           std::string res_str;
           if ( wpu->parse( progress, res_str, a_max( EXE_STRING_LEN - 1024, 256 ),
-                           ( ( progress_useoutput == true ) ? true : false ) ) == NormalMode::PARSE_SUCCESS ) {
+                           ( ( progress_useoutput == true ) ? true : false ) ) == WPUContext::PARSE_SUCCESS ) {
             if ( progress_useoutput == true ) {
-              std::auto_ptr<ExeClass> ec( new ExeClass( ExeClass::EXECUTE_NORMAL ) );
+              std::unique_ptr<ExeClass> ec( new ExeClass( ExeClass::EXECUTE_NORMAL ) );
               ec->cdDir( wpu->getBaseDir() );
               ec->addCommand( "%s", res_str.c_str() );
               tstr2 = ec->getOutput( &exeerror );
@@ -315,9 +317,9 @@ ScriptOp::run( WPUContext *wpu, ActionMessage *msg )
           std::string res_str;
           bool valid = true;
           if ( wpu->parse( wintext, res_str, a_max( EXE_STRING_LEN - 1024, 256 ),
-                           ( ( wintext_useoutput == true ) ? true : false ) ) == NormalMode::PARSE_SUCCESS ) {
+                           ( ( wintext_useoutput == true ) ? true : false ) ) == WPUContext::PARSE_SUCCESS ) {
             if ( wintext_useoutput == true ) {
-              std::auto_ptr<ExeClass> ec( new ExeClass( ExeClass::EXECUTE_NORMAL ) );
+              std::unique_ptr<ExeClass> ec( new ExeClass( ExeClass::EXECUTE_NORMAL ) );
               ec->cdDir( wpu->getBaseDir() );
               ec->addCommand( "%s", res_str.c_str() );
               tstr2 = ec->getOutput( &exeerror );
@@ -382,7 +384,7 @@ ScriptOp::run( WPUContext *wpu, ActionMessage *msg )
             std::string res_str;
 
             if ( wpu->parse( m_command_str.c_str(), res_str, a_max( EXE_STRING_LEN - 1024, 256 ),
-                             true ) == NormalMode::PARSE_SUCCESS ) {
+                             true ) == WPUContext::PARSE_SUCCESS ) {
                 Lister *l1 = msg->getWorker()->getActiveLister();
                 if ( l1 != NULL ) {
                     ListerMode *lm1 = l1->getActiveMode();
@@ -607,7 +609,7 @@ int ScriptOp::configure()
   
   tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(getName())+1);
   sprintf(tstr,catalog.getLocale(293),getName());
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe(tstr);
 
@@ -1014,25 +1016,33 @@ int ScriptOp::configure()
   command_win_ac1->setMaxSpace( 5 );
   command_win_ac1->add( new Text( aguix, 0, 0, catalog.getLocale( 989 ), 1 ), 0, 0, cincwnr );
 
-  AContainer *command_win_ac1_2 = command_win_ac1->add( new AContainer( command_win, 3, 1 ), 0, 1 );
-  command_win_ac1_2->setMinSpace( 5 );
-  command_win_ac1_2->setMaxSpace( 5 );
+  AContainer *command_win_ac1_2 = command_win_ac1->add( new AContainer( command_win, 5, 1 ), 0, 1 );
+  command_win_ac1_2->setMinSpace( 0 );
+  command_win_ac1_2->setMaxSpace( 0 );
   command_win_ac1_2->setBorderWidth( 0 );
 
   command_win_ac1_2->add( new Text( aguix, 0, 0, catalog.getLocale( 990 ), 1 ), 0, 0, cfix );
+  command_win_ac1_2->setMaxWidth( 5, 1, 0 );
   StringGadget *cmdsg = (StringGadget*)command_win_ac1_2->add( new StringGadget( aguix,
                                                                                  0,
                                                                                  0,
                                                                                  100,
                                                                                  m_command_str.c_str(),
-                                                                                 0 ), 1, 0, cincw );
+                                                                                 0 ), 2, 0, cincw );
+  Button *cmd_command_b = command_win_ac1_2->addWidget( new Button( aguix,
+                                                                    0,
+                                                                    0,
+                                                                    "C",
+                                                                    1,
+                                                                    0,
+                                                                    0 ), 3, 0, cfix );
   Button *cmd_flag_b = (Button*)command_win_ac1_2->add( new Button( aguix,
                                                                  0,
                                                                  0,
                                                                  "F",
                                                                  1,
                                                                  0,
-                                                                 0 ), 2, 0, cfix );
+                                                                 0 ), 4, 0, cfix );
 
   AContainer *ac1_2 = ac1->add( new AContainer( win, 2, 1 ), 0, 3 );
   ac1_2->setMinSpace( 5 );
@@ -1171,6 +1181,12 @@ int ScriptOp::configure()
               cmdsg->insertAtCursor( tstr );
               _freesafe( tstr );
             }
+          } else if ( msg->button.button == cmd_command_b ) {
+            tstr = getRegisteredCommand();
+            if ( tstr != NULL ) {
+              cmdsg->insertAtCursor( tstr );
+              _freesafe( tstr );
+            }
           }
           break;
         case AG_FIELDLV_ONESELECT:
@@ -1379,7 +1395,7 @@ char *ScriptOp::getIfFlag()
   char *returnstr;
   int i, trow, nr, nr2;
   
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, catalog.getLocale( 338 ) );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, catalog.getLocale( 338 ), AWindow::AWINDOW_DIALOG );
   win->create();
 
   AContainer *ac1 = win->setContainer( new AContainer( win, 1, 3 ), true );
@@ -1478,6 +1494,109 @@ char *ScriptOp::getIfFlag()
   return returnstr;
 }
 
+char *ScriptOp::getRegisteredCommand()
+{
+  AGUIX *aguix = Worker::getAGUIX();
+  AWindow *win;
+  FieldListView *lv;
+  AGMessage *msg;
+  int endmode=-1;
+  char *returnstr;
+  
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, catalog.getLocale( 338 ), AWindow::AWINDOW_DIALOG );
+  win->create();
+
+  AContainer *ac1 = win->setContainer( new AContainer( win, 1, 3 ), true );
+  ac1->setMinSpace( 5 );
+  ac1->setMaxSpace( 5 );
+  ac1->setBorderWidth( 5 );
+
+  win->addMultiLineText( catalog.getLocale( 1038 ), *ac1, 0, 0, NULL, NULL );
+
+  lv = (FieldListView*)ac1->add( new FieldListView( aguix,
+                                                    0,
+                                                    0,
+                                                    400,
+                                                    300,
+                                                    0 ),
+                                 0, 1, AContainer::CO_MIN );
+  lv->setHBarState(2);
+  lv->setVBarState(2);
+  lv->setDefaultColorMode( FieldListView::PRECOLOR_ONLYACTIVE );
+  lv->setNrOfFields( 1 );
+  lv->setDisplayFocus( true );
+  lv->setAcceptFocus( true );
+
+  {
+      Lister *l1 = wconfig->getWorker()->getLister( 0 );
+                
+      if ( l1 != NULL ) {
+          ListerMode *lm1 = l1->getActiveMode();
+
+          if ( lm1 != NULL ) {
+              std::list< std::string > l = lm1->getListOfCommandsWithoutArgs();
+
+              for ( auto &name : l ) {
+                  int trow = lv->addRow();
+                  lv->setText( trow, 0, name );
+              }
+          }
+      }
+  }
+  
+  AContainer *ac1_1 = ac1->add( new AContainer( win, 2, 1 ), 0, 2 );
+  ac1_1->setMinSpace( 5 );
+  ac1_1->setMaxSpace( -1 );
+  ac1_1->setBorderWidth( 0 );
+  Button *okb =(Button*)ac1_1->add( new Button( aguix,
+                                                0,
+                                                0,
+                                                catalog.getLocale( 11 ),
+                                                1,
+                                                0,
+                                                0 ), 0, 0, AContainer::CO_FIX );
+  Button *cancelb = (Button*)ac1_1->add( new Button( aguix,
+						     0,
+						     0,
+						     catalog.getLocale( 8 ),
+						     1,
+						     0,
+						     0 ), 1, 0, AContainer::CO_FIX );
+
+  win->setDoTabCycling( true );
+  win->contMaximize( true );
+  win->show();
+  lv->takeFocus();
+  for(;endmode==-1;) {
+    msg=aguix->WaitMessage(win);
+    if(msg!=NULL) {
+      switch(msg->type) {
+        case AG_CLOSEWINDOW:
+          if(msg->closewindow.window==win->getWindow()) endmode=1;
+          break;
+        case AG_BUTTONCLICKED:
+          if(msg->button.button==okb) endmode=0;
+          else if(msg->button.button==cancelb) endmode=1;
+          break;
+      }
+      aguix->ReplyMessage(msg);
+    }
+  }
+  
+  returnstr=NULL;
+  if(endmode==0) {
+    // ok
+    int trow = lv->getActiveRow();
+    if ( lv->isValidRow( trow ) == true ) {
+        returnstr = dupstring( lv->getText( trow, 0 ).c_str() );
+    }
+  }
+  
+  delete win;
+
+  return returnstr;
+}
+
 void ScriptOp::setDoDebug( bool nv )
 {
   dodebug = nv;
