@@ -1,6 +1,6 @@
 /* scrolllisterop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2004,2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2004,2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,12 +21,12 @@
 
 #include "scrolllisterop.h"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "worker_locale.h"
 #include "datei.h"
 #include <aguix/cyclebutton.h>
 #include <aguix/button.h>
+#include "virtualdirmode.hh"
 
 const char *ScrollListerOp::name="ScrollListerOp";
 
@@ -70,9 +70,13 @@ ScrollListerOp::run( WPUContext *wpu, ActionMessage *msg )
     if(l1!=NULL) {
       startlister=l1;
       lm1=startlister->getActiveMode();
-      if(lm1!=NULL)
-        if(lm1->isType("NormalMode")==true)
-          normalmodescrolllister( msg );
+      if(lm1!=NULL) {
+          if(dynamic_cast< VirtualDirMode * >( lm1 ) ) {
+              normalmodescrolllister( msg );
+          } else {
+              lm1->not_supported();
+          }
+      }
     }
   }
   return 0;
@@ -102,19 +106,19 @@ ScrollListerOp::getDescription()
 void
 ScrollListerOp::normalmodescrolllister( ActionMessage *am )
 {
-  NormalMode *nm1=NULL;
-  ListerMode *lm1=NULL;
+  ListerMode *lm1 = NULL;
   
-  if(startlister==NULL) return;
-  lm1=startlister->getActiveMode();
-  if(lm1==NULL) return;
-  if(lm1->isType("NormalMode")!=true) return;
-  nm1=(NormalMode*)lm1;
-  
-  if(dir==SCROLLLISTEROP_RIGHT)
-    nm1->scrolllister(1);
-  else
-    nm1->scrolllister(-1);
+  if ( startlister == NULL ) return;
+  lm1 = startlister->getActiveMode();
+  if ( lm1 == NULL ) return;
+
+  if ( auto vdm = dynamic_cast< VirtualDirMode * >( lm1 ) ) {
+      if ( dir == SCROLLLISTEROP_RIGHT ) {
+          vdm->scrollListView( 1 );
+      } else {
+          vdm->scrollListView( -1 );
+      }
+  }
 }
 
 int
@@ -136,7 +140,7 @@ ScrollListerOp::configure()
   
   tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(35))+1);
   sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(35));
-  win =new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe(tstr);
 
