@@ -1,6 +1,6 @@
 /* searchentryop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2004,2009,2011-2012 Ralf Hoffmann.
+ * Copyright (C) 2001-2004,2009,2011-2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,12 +21,12 @@
 
 #include "searchentryop.h"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "worker_locale.h"
 #include "datei.h"
 #include <aguix/button.h>
 #include <aguix/choosebutton.h>
+#include "virtualdirmode.hh"
 
 const char *SearchEntryOp::name="SearchEntryOp";
 
@@ -76,9 +76,11 @@ SearchEntryOp::run( WPUContext *wpu, ActionMessage *msg )
     if(l1!=NULL) {
       startlister=l1;
       lm1=startlister->getActiveMode();
-      if(lm1!=NULL)
-        if(lm1->isType("NormalMode")==true)
-          normalmodesearchentry();
+      if(lm1!=NULL) {
+          if ( dynamic_cast< VirtualDirMode * >( lm1 ) ) {
+              vdm_searchentry();
+          }
+      }
     }
   }
   return 0;
@@ -90,21 +92,20 @@ SearchEntryOp::getDescription()
   return catalog.getLocaleCom(33);
 }
 
-int
-SearchEntryOp::normalmodesearchentry()
+int SearchEntryOp::vdm_searchentry()
 {
-  NormalMode *nm1=NULL;
-  ListerMode *lm1=NULL;
+    ListerMode *lm1 = NULL;
   
-  if(startlister==NULL) return 1;
-  lm1=startlister->getActiveMode();
-  if(lm1==NULL) return 1;
-  if(lm1->isType("NormalMode")!=true) return 1;
-  nm1=(NormalMode*)lm1;
-  
-  nm1->setFlexibleMatchingMode( m_flexible_match );
-  nm1->searchentry( ignore_case, reverse_search, m_infix_search );
-  return 0;
+    if ( startlister == NULL ) return 1;
+
+    lm1 = startlister->getActiveMode();
+    if ( lm1 == NULL ) return 1;
+
+    if ( auto vdm = dynamic_cast< VirtualDirMode * >( lm1 ) ) {
+        vdm->setFlexibleMatchingMode( m_flexible_match );
+        vdm->searchentry( ignore_case, reverse_search, m_infix_search );
+    }
+    return 0;
 }
 
 bool SearchEntryOp::save( Datei *fh )
@@ -162,7 +163,7 @@ int SearchEntryOp::configure()
 
   tstr = (char*)_allocsafe( strlen( catalog.getLocale( 293 ) ) + strlen( getDescription() ) + 1 );
   sprintf( tstr, catalog.getLocale( 293 ), getDescription() );
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe( tstr );
   
