@@ -1,6 +1,6 @@
 /* searchopbg.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2006-2008,2010-2012 Ralf Hoffmann.
+ * Copyright (C) 2006-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,7 +21,7 @@
 
 #include "searchopbg.hh"
 #include "listermode.h"
-#include "normalmode.h"
+#include "virtualdirmode.hh"
 #include "worker.h"
 #include "execlass.h"
 #include "nwc_virtualdir.hh"
@@ -38,6 +38,8 @@
 #include "worker_locale.h"
 #include "wconfig.h"
 #include "resultstore.hh"
+#include "virtualdirmode.hh"
+#include "argclass.hh"
 
 static const int searchopbg_timeout = 10;
 
@@ -51,6 +53,7 @@ static const int searchopbg_timeout = 10;
 
 static std::list<ResultStore*> _stored_results;
 static int _stored_results_changed = 0;
+int SearchOpBG::s_search_number = 1;
 
 #define DEFAULT_EDIT_COMMAND "{scripts}/xeditor -line {l} {f}"
 
@@ -196,17 +199,20 @@ void SearchOpBG::enterResult( const std::string &fullname )
 {
     Lister *l1;
     ListerMode *lm1;
-    NormalMode *nm1;
 
     l1 = m_worker->getActiveLister();
     if ( l1 != NULL ) {
-        l1->switch2Mode( 0 );
+        if ( l1->getActiveMode() == NULL ||
+             ! ( dynamic_cast< VirtualDirMode *>( l1->getActiveMode() ) ) ) {
+            l1->switch2Mode( 0 );
+        }
+
         lm1 = l1->getActiveMode();
         if ( lm1 != NULL ) {
-            if ( lm1->isType( "NormalMode" ) == true ) {
-                nm1 = (NormalMode*)lm1;
-                nm1->enterPath( fullname );
-            }
+            std::list< RefCount< ArgClass > > args;
+
+            args.push_back( new StringArg( fullname ) );
+            lm1->runCommand( "enter_dir", args );
         }
     }
 }
@@ -225,6 +231,8 @@ void SearchOpBG::clearStoredResults()
   
     if ( _storelv != NULL )
         buildStoreLV();
+
+    updateWindowTitle();
 }
 
 void SearchOpBG::buildLV()
@@ -313,7 +321,15 @@ void SearchOpBG::setResultStore( int pos, const SearchSettings *sets )
     if ( erg == NULL ) {
         // nothing found so create a new one with settings from
         // the current or latest entry
-        erg = new ResultStore( "search1" );
+
+        std::string name = AGUIXUtils::formatStringToString( "search%d", s_search_number++ );
+
+        if ( s_search_number < 0 ) {
+            // avoid negative and zero search number
+            s_search_number = 1;
+        }
+
+        erg = new ResultStore( name );
 
         std::list<ResultStore*>::iterator it;
         for ( it = _stored_results.begin();
@@ -351,6 +367,8 @@ void SearchOpBG::setResultStore( int pos, const SearchSettings *sets )
 
     _rst = erg;
     _rst->set_exclusive( this );
+
+    updateWindowTitle();
 }
 
 class DW_SetFollowSymlinks : public NWC::DirWalkCallBack
@@ -391,13 +409,19 @@ void SearchOpBG::startSearch()
 
     SearchSettings sets = getCurrentSearchSettings();;
 
-    std::auto_ptr<NWC::VirtualDir> use_dir( _rst->getClonedDir() );
+    std::unique_ptr<NWC::VirtualDir> use_dir( _rst->getClonedDir() );
   
     if ( use_dir->size() < 1 ) {
         std::string d = sets.getBaseDir();
-        if ( d == "." || d.length() < 1 )
-            d = m_initial_start_dir;
-        use_dir->add( NWC::VirtualDir( d ) );
+        if ( d == "." || d.length() < 1 || d[0] != '/' ) {
+            if ( m_initial_start_vdir.get() ) {
+                use_dir->add( *m_initial_start_vdir );
+            } else {
+                use_dir->add( NWC::VirtualDir( m_initial_start_dir ) );
+            }
+        } else {
+            use_dir->add( NWC::VirtualDir( d ) );
+        }
     } else {
         setCurrentResultStore( -1, &sets );
     }
@@ -410,7 +434,7 @@ void SearchOpBG::startSearch()
     DW_SetFollowSymlinks dw1( sets.getFollowSymlinks(), sets.getSearchSameFS() );
     use_dir->walk( dw1 );
 
-    _sth = new SearchThread( std::auto_ptr<NWC::Dir>( use_dir ) );
+    _sth = new SearchThread( std::unique_ptr<NWC::Dir>( std::move( use_dir ) ) );
     _sth->setSearchSettings( sets );
     _sth->start();
 
@@ -418,6 +442,8 @@ void SearchOpBG::startSearch()
     buildStoreLV();
   
     _search_started = true;
+
+    updateWindowTitle();
 }
 
 void SearchOpBG::stopSearch()
@@ -490,7 +516,11 @@ void SearchOpBG::newSearch()
 {
     stopSearch();
 
-    _startdir_sg->setText( m_initial_start_dir.c_str() );
+    if ( m_initial_start_vdir.get() ) {
+        _startdir_sg->setText( m_initial_start_vdir->getFullname().c_str() );
+    } else {
+        _startdir_sg->setText( m_initial_start_dir.c_str() );
+    }
     SearchSettings sets = getCurrentSearchSettings();
 
     if ( _rst == NULL ||_rst->getDir()->size() > 0 ) {
@@ -547,6 +577,9 @@ SearchSettings SearchOpBG::getCurrentSearchSettings()
     sets.setSizeGEQ( m_size_geq_sg->getText() );
     sets.setSizeLEQ( m_size_leq_sg->getText() );
     sets.setMaxVFSDepth( atoi( m_max_vfs_depth_sg->getText() ) );
+
+    sets.setMatchAlsoDirectories( m_also_dirs_cb->getState() );
+
     return sets;
 }
 
@@ -561,7 +594,11 @@ void SearchOpBG::restoreSearchSettings()
   
     SearchSettings sets = _rst->getSearchSettings();
     if ( sets.getBaseDir().length() < 1 ) {
-        _startdir_sg->setText( m_initial_start_dir.c_str() );
+        if ( m_initial_start_vdir.get() ) {
+            _startdir_sg->setText( m_initial_start_vdir->getFullname().c_str() );
+        } else {
+            _startdir_sg->setText( m_initial_start_dir.c_str() );
+        }
     } else {
         _startdir_sg->setText( sets.getBaseDir().c_str() );
     }
@@ -586,6 +623,8 @@ void SearchOpBG::restoreSearchSettings()
 
     std::string s1 = AGUIXUtils::formatStringToString( "%d", sets.getMaxVFSDepth() );
     m_max_vfs_depth_sg->setText( s1.c_str() );
+
+    m_also_dirs_cb->setState( sets.getMatchAlsoDirectories() );
 }
 
 void SearchOpBG::setCurrentResultStore( int pos, const SearchSettings *sets )
@@ -610,19 +649,19 @@ void SearchOpBG::removeSelectedResults()
     // same entry (for different lines)
     // then remove it from searchdir
 
-    std::auto_ptr< std::list< std::pair< std::string, int > > > selected_entries = getSelectedResults();
+    std::unique_ptr< std::list< std::pair< std::string, int > > > selected_entries = getSelectedResults();
   
     _rst->removeEntries( *( selected_entries.get() ) );
     buildLV();
     _resultlv->centerActive();
 }
 
-std::auto_ptr< std::list< std::pair< std::string, int > > > SearchOpBG::getSelectedResults()
+std::unique_ptr< std::list< std::pair< std::string, int > > > SearchOpBG::getSelectedResults()
 {
     std::list<SearchThread::SearchResult> *res = _rst->getRes();
     std::list<SearchThread::SearchResult>::iterator active_entry = res->end();
 
-    std::auto_ptr< std::list< std::pair< std::string, int > > > selected_entries( new std::list< std::pair< std::string, int > > );
+    std::unique_ptr< std::list< std::pair< std::string, int > > > selected_entries( new std::list< std::pair< std::string, int > > );
 
     int lvpos = 0;
     for ( std::list<SearchThread::SearchResult>::iterator it = res->begin();
@@ -718,7 +757,7 @@ void SearchOpBG::editFile( const std::string &filename, int line_number )
   
     std::string exestr = rpl.replaceFlags( m_edit_command );
 
-    std::auto_ptr<ExeClass> ec( new ExeClass( ExeClass::EXECUTE_NORMAL ) );
+    std::unique_ptr<ExeClass> ec( new ExeClass( ExeClass::EXECUTE_NORMAL ) );
 
     ec->cdDir( fse.getDirname().c_str() );
     ec->addCommand( "%s", exestr.c_str() );
@@ -755,41 +794,43 @@ void SearchOpBG::initColorDefs()
     resultcol.setBG( FieldListView::CC_SELACT, wconfig->getUnselFileAct( 1 ) );
 }
 
-SearchOpBG::SearchOpBG( Worker *worker ) : m_worker( worker),
-                                           m_show_prev_results( false ),
-                                           m_edit_command( DEFAULT_EDIT_COMMAND ),
-                                           _start_ac( NULL ),
-                                           _startdir_text( NULL ),
-                                           _startdir_sg( NULL ),
-                                           _storelv( NULL ),
-                                           _last_startcont_state( false ),
-                                           _last_result( "" ),
-                                           _rst( NULL ),
-                                           _resultlv( NULL ),
-                                           _sth( NULL ),
-                                           _matchname_sg( NULL ),
-                                           _matchname_regexp_cb( NULL ),
-                                           _matchname_fullpath_cb( NULL ),
-                                           _matchname_case_cb( NULL ),
-                                           _matchcontent_sg( NULL ),
-                                           _matchcontent_case_cb( NULL ),
-                                           _follow_symlinks_cb( NULL ),
-                                           _search_vfs_cb( NULL ),
-                                           _search_same_fs_cb( NULL ),
-                                           m_younger_cb( NULL ),
-                                           m_older_cb( NULL ),
-                                           m_younger_sg( NULL ),
-                                           m_older_sg( NULL ),
-                                           m_size_geq_cb( NULL ),
-                                           m_size_leq_cb( NULL ),
-                                           m_size_geq_sg( NULL ),
-                                           m_size_leq_sg( NULL ),
-                                           m_options_k1( NULL ),
-                                           m_max_vfs_depth_sg( NULL ),
-                                           _search_started( false ),
-                                           m_update_ctr( 0 ),
-                                           m_stored_result_last_changed( 0 ),
-                                           m_timeout_set( false )
+SearchOpBG::SearchOpBG( Worker *worker ) :
+    m_worker( worker),
+    m_show_prev_results( false ),
+    m_edit_command( DEFAULT_EDIT_COMMAND ),
+    _start_ac( NULL ),
+    _startdir_text( NULL ),
+    _startdir_sg( NULL ),
+    _storelv( NULL ),
+    _last_startcont_state( false ),
+    _last_result( "" ),
+    _rst( NULL ),
+    _resultlv( NULL ),
+    _sth( NULL ),
+    _matchname_sg( NULL ),
+    _matchname_regexp_cb( NULL ),
+    _matchname_fullpath_cb( NULL ),
+    _matchname_case_cb( NULL ),
+    _matchcontent_sg( NULL ),
+    _matchcontent_case_cb( NULL ),
+    _follow_symlinks_cb( NULL ),
+    _search_vfs_cb( NULL ),
+    _search_same_fs_cb( NULL ),
+    m_younger_cb( NULL ),
+    m_older_cb( NULL ),
+    m_younger_sg( NULL ),
+    m_older_sg( NULL ),
+    m_size_geq_cb( NULL ),
+    m_size_leq_cb( NULL ),
+    m_size_geq_sg( NULL ),
+    m_size_leq_sg( NULL ),
+    m_options_k1( NULL ),
+    m_max_vfs_depth_sg( NULL ),
+    m_also_dirs_cb( NULL ),
+    _search_started( false ),
+    m_update_ctr( 0 ),
+    m_stored_result_last_changed( 0 ),
+    m_timeout_set( false )
 {
     m_timeout_cb = new SearchOpBGTimeoutCB( this );
 }
@@ -841,7 +882,7 @@ void SearchOpBG::createWindow()
     m_options_k1->add( subwin1 );
     subwin1->create();
 
-    AContainer *acsw1_1 = subwin1->setContainer( new AContainer( subwin1, 1, 2 ), true );
+    AContainer *acsw1_1 = subwin1->setContainer( new AContainer( subwin1, 1, 3 ), true );
     acsw1_1->setMinSpace( 5 );
     acsw1_1->setMaxSpace( 5 );
     acsw1_1->setBorderWidth( 5 );
@@ -854,7 +895,11 @@ void SearchOpBG::createWindow()
     _start_ac->add( new Text( aguix, 0, 0, catalog.getLocale( 722 ), 1 ), 0, 0, AContainer::CO_FIX );
     _startdir_text = (Text*)_start_ac->add( new Text( aguix, 0, 0, catalog.getLocale( 723 ), 1 ), 1, 0, AContainer::CO_INCW );
     _startdir_text->hide();
-    _startdir_sg = (StringGadget*)_start_ac->add( new StringGadget( aguix, 0, 0, 100, m_initial_start_dir.c_str(), 0 ), 1, 0, AContainer::CO_INCW );
+    _startdir_sg = (StringGadget*)_start_ac->add( new StringGadget( aguix, 0, 0, 100,
+                                                                    m_initial_start_vdir.get() != NULL ?
+                                                                    m_initial_start_vdir->getFullname().c_str() :
+                                                                    m_initial_start_dir.c_str(),
+                                                                    0 ), 1, 0, AContainer::CO_INCW );
     _startdir_text->resize( _startdir_text->getWidth(), _startdir_sg->getHeight() );
 
     AContainer *ac1_2 = _start_ac->add( new AContainer( subwin1, 4, 1 ), 1, 1 );
@@ -906,6 +951,15 @@ void SearchOpBG::createWindow()
                                                            0,
                                                            0 ), 4, 0, AContainer::CO_FIX );
 
+    AContainer *ac1_set2 = acsw1_1->add( new AContainer( subwin1, 2, 1 ), 0, 2 );
+    ac1_set2->setMinSpace( 5 );
+    ac1_set2->setMaxSpace( 5 );
+    ac1_set2->setBorderWidth( 0 );
+
+    m_also_dirs_cb = (ChooseButton*)ac1_set2->add( new ChooseButton( aguix, 0, 0, false,
+                                                                     catalog.getLocale( 1003 ), LABEL_RIGHT, 1, 0 ),
+                                                   0, 0, AContainer::CO_FIXNR );
+
     AWindow *subwin2 = new AWindow( aguix, 0, 0, 10, 10, 0, "" );
     m_options_k1->add( subwin2 );
     subwin2->create();
@@ -957,7 +1011,7 @@ void SearchOpBG::createWindow()
     m_options_k1->contMaximize();
     ac1_1->readLimits();
   
-    AContainer *ac1_res = ac1->add( new AContainer( m_win, 1, 2 ), 0, 1 );
+    AContainer *ac1_res = ac1->add( new AContainer( m_win, 1, 3 ), 0, 1 );
     ac1_res->setMinSpace( 5 );
     ac1_res->setMaxSpace( 5 );
     ac1_res->setBorderWidth( 0 );
@@ -971,23 +1025,34 @@ void SearchOpBG::createWindow()
     _resultlv->setFieldTextMerged( 0, true );
     _resultlv->setHBarState( 2 );
     _resultlv->setVBarState( 2 );
+    _resultlv->setConsiderHeaderWidthForDynamicWidth( true );
     _resultlv->setShowHeader( true );
     _resultlv->setAcceptFocus( true );
     _resultlv->setDisplayFocus( true );
     _resultlv->setMBG( wconfig->getListerBG() );
 
-    AContainer *ac1_4 = ac1_res->add( new AContainer( m_win, 4, 1 ), 0, 1 );
+    AContainer *ac1_4 = ac1_res->add( new AContainer( m_win, 2, 1 ), 0, 1 );
     ac1_4->setMinSpace( 0 );
-    ac1_4->setMaxSpace( -1 );
+    ac1_4->setMaxSpace( 0 );
     ac1_4->setBorderWidth( 0 );
 
     m_enterb =(Button*)ac1_4->add( new Button( aguix, 0, 0,
                                                catalog.getLocale( 730 ), catalog.getLocale( 967 ),
-                                               1, 1, 0, 0, 0 ), 0, 0, AContainer::CO_FIX );
-    m_viewb = (Button*)ac1_4->add( new Button( aguix, 0, 0, catalog.getLocale( 731 ), 1, 0, 0 ), 1, 0, AContainer::CO_FIX );
-    m_editb = (Button*)ac1_4->add( new Button( aguix, 0, 0, catalog.getLocale( 732 ), 1, 0, 0 ), 2, 0, AContainer::CO_FIX );
-    m_removeb = (Button*)ac1_4->add( new Button( aguix, 0, 0, catalog.getLocale( 733 ), 1, 0, 0 ),
-                                     3, 0, AContainer::CO_FIX );
+                                               1, 1, 0, 0, 0 ), 0, 0, AContainer::CO_INCW );
+    m_panelize_b =(Button*)ac1_4->add( new Button( aguix, 0, 0,
+                                                   catalog.getLocale( 995 ),
+                                                   catalog.getLocale( 996 ),
+                                                   1, 1, 0, 0, 0 ), 1, 0, AContainer::CO_INCW );
+
+    AContainer *ac1_4_1 = ac1_res->add( new AContainer( m_win, 3, 1 ), 0, 2 );
+    ac1_4_1->setMinSpace( 0 );
+    ac1_4_1->setMaxSpace( 0 );
+    ac1_4_1->setBorderWidth( 0 );
+
+    m_viewb = (Button*)ac1_4_1->add( new Button( aguix, 0, 0, catalog.getLocale( 731 ), 1, 0, 0 ), 0, 0, AContainer::CO_INCW );
+    m_editb = (Button*)ac1_4_1->add( new Button( aguix, 0, 0, catalog.getLocale( 732 ), 1, 0, 0 ), 1, 0, AContainer::CO_INCW );
+    m_removeb = (Button*)ac1_4_1->add( new Button( aguix, 0, 0, catalog.getLocale( 733 ), 1, 0, 0 ),
+                                     2, 0, AContainer::CO_INCW );
   
     AContainerBB *ac1_res_store = dynamic_cast<AContainerBB*>( ac1->add( new AContainerBB( m_win, 1, 3 ), 0, 2 ) );
     ac1_res_store->setMinSpace( 5 );
@@ -1001,13 +1066,10 @@ void SearchOpBG::createWindow()
     _storelv->setNrOfFields( 5 );
     _storelv->setFieldText( 0, catalog.getLocale( 963 ) );
     _storelv->setFieldText( 1, catalog.getLocale( 735 ) );
-    _storelv->setFieldWidth( 1, aguix->getTextWidth( catalog.getLocale( 735 ) ) + 20 );
     _storelv->setFieldText( 2, catalog.getLocale( 729 ) );
-    _storelv->setFieldWidth( 2, aguix->getTextWidth( catalog.getLocale( 729 ) ) + 20 );
     _storelv->setFieldText( 3, catalog.getLocale( 736 ) );
-    _storelv->setFieldWidth( 3, aguix->getTextWidth( catalog.getLocale( 736 ) ) + 20 );
     _storelv->setFieldText( 4, catalog.getLocale( 737 ) );
-    _storelv->setFieldWidth( 4, aguix->getTextWidth( catalog.getLocale( 737 ) ) + 100 );
+    _storelv->setConsiderHeaderWidthForDynamicWidth( true );
     _storelv->setShowHeader( true );
     _storelv->setAcceptFocus( true );
     _storelv->setDisplayFocus( true );
@@ -1113,6 +1175,8 @@ void SearchOpBG::createWindow()
     m_worker->getEventHandler().registerCallback( m_timeout_cb,
                                                   EventCallbacks::TIMEOUT );
 
+    updateWindowTitle();
+
     return;
 }
 
@@ -1148,6 +1212,12 @@ void SearchOpBG::handleAGUIXMessage( AGMessage &msg )
                 } else {
                     endmode = 2;
                 }
+            } else if ( msg.button.button == m_panelize_b ) {
+                if ( msg.button.state == 2 ) {
+                    endmode = 5;
+                } else {
+                    endmode = 4;
+                }
             } else if ( msg.button.button == m_clearstoreb ) {
                 clearStoredResults();
                 newSearch();
@@ -1233,6 +1303,10 @@ void SearchOpBG::handleAGUIXMessage( AGMessage &msg )
                     startSearch();
                 } else if ( msg.key.key == XK_Escape ) {
                     endmode = 1;
+                } else if ( msg.key.key == XK_F1 ) {
+                    endmode = 2;
+                } else if ( msg.key.key == XK_F2 ) {
+                    endmode = 4;
                 } else if ( msg.key.key == XK_F3 ) {
                     viewResult();
                 } else if ( msg.key.key == XK_F4 ) {
@@ -1252,9 +1326,12 @@ void SearchOpBG::handleAGUIXMessage( AGMessage &msg )
         if ( _resultlv->isValidRow( row ) ) {
             enterResult( _resultlv->getText( row, 2 ) );
         }
+    } else if ( endmode == 4 || endmode == 5 ) {
+        // panelize
+        panelizeResults();
     }
 
-    if ( endmode == 1 || endmode == 2 ) {
+    if ( endmode == 1 || endmode == 2 || endmode == 4) {
         stopSearch();
 
         aguix->unregisterBGHandler( m_win );
@@ -1331,3 +1408,52 @@ void SearchOpBG::checkForResults()
         gatherResults( -1, searchopbg_timeout / 2 );
     }
 }
+
+void SearchOpBG::panelizeResults()
+{
+    Lister *l1;
+
+    l1 = m_worker->getActiveLister();
+    if ( l1 != NULL ) {
+        l1->switch2Mode( 0 );
+
+        VirtualDirMode *vdm = dynamic_cast< VirtualDirMode* >( l1->getActiveMode() );
+        if ( vdm != NULL ) {
+            vdm->newTab();
+
+            vdm->showDir( *_rst->getDir() );
+        }
+    }
+}
+
+void SearchOpBG::setInitialVDir( std::unique_ptr< NWC::Dir > d )
+{
+    m_initial_start_vdir = std::move( d );
+}
+
+void SearchOpBG::updateWindowTitle()
+{
+    std::string new_title = catalog.getLocaleCom( 50 );
+
+    if ( _rst != NULL ) {
+
+        new_title += ": ";
+
+        SearchSettings sets = _rst->getSearchSettings();
+
+        new_title += sets.getBaseDir();
+
+        new_title += "/";
+
+        new_title += sets.getMatchName();
+
+        if ( ! sets.getMatchContent().empty() ) {
+
+            new_title += ":";
+
+            new_title += sets.getMatchContent();
+        }
+    }
+
+    m_win->setTitle( new_title );
+}
