@@ -45,7 +45,8 @@ SearchSettings::SearchSettings() : _basedir( "" ),
                                    m_older_date( 0 ),
                                    m_size_geq_int( 0 ),
                                    m_size_leq_int( 0 ),
-                                   m_max_vfs_depth( 0 )
+    m_max_vfs_depth( 0 ),
+    m_match_also_directories( false )
 {
 }
 
@@ -85,7 +86,8 @@ SearchSettings::SearchSettings( const std::string &basedir,
                                                                   m_older_date( 0 ),
                                                                   m_size_geq_int( 0 ),
                                                                   m_size_leq_int( 0 ),
-                                                                  m_max_vfs_depth( 0 )
+    m_max_vfs_depth( 0 ),
+    m_match_also_directories( false )
 {
     convertDates();
 }
@@ -300,3 +302,13 @@ int SearchSettings::getMaxVFSDepth() const
 {
     return m_max_vfs_depth;
 }
+
+void SearchSettings::setMatchAlsoDirectories( bool nv )
+{
+    m_match_also_directories = nv;
+}
+
+bool SearchSettings::getMatchAlsoDirectories() const
+{
+    return m_match_also_directories;
+}
