@@ -1,6 +1,6 @@
 /* setfilterop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2004,2008,2010-2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,7 +21,6 @@
 
 #include "setfilterop.h"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include <aguix/acontainerbb.h>
 #include "bookmarkdbproxy.hh"
@@ -37,11 +36,12 @@
 #include <aguix/stringgadget.h>
 #include <aguix/button.h>
 #include "wconfig.h"
+#include "virtualdirmode.hh"
 
 const char *SetFilterOp::name="SetFilterOp";
 int SetFilterOp::instance_counter = 0;
-size_t SetFilterOp::history_size = 30;
-std::auto_ptr< PersistentStringList > SetFilterOp::m_history;
+size_t SetFilterOp::history_size = 1000;
+std::unique_ptr< PersistentStringList > SetFilterOp::m_history;
 
 class SetFilterCB : public GenericCallbackArg<void, AGMessage*>
 {
@@ -154,9 +154,13 @@ SetFilterOp::run( WPUContext *wpu, ActionMessage *msg )
       startlister=l1;
       endlister = msg->getWorker()->getOtherLister(startlister);
       lm1=startlister->getActiveMode();
-      if(lm1!=NULL)
-        if(lm1->isType("NormalMode")==true)
-          normalmodesf( msg );
+      if(lm1!=NULL) {
+          if(dynamic_cast< VirtualDirMode * > ( lm1 ) ) {
+              normalmodesf( msg );
+          } else {
+              lm1->not_supported();
+          }
+      }
     }
   }
   return 0;
@@ -221,16 +225,18 @@ SetFilterOp::getDescription()
 int
 SetFilterOp::normalmodesf( ActionMessage *am )
 {
-    NormalMode *nm1 = NULL;
+    VirtualDirMode *vdm = NULL;
     ListerMode *lm1 = NULL;
     bool cont = true;
-  
+
     if ( am->mode != am->AM_MODE_DNDACTION ) {
         if ( startlister == NULL ) return 1;
         lm1 = startlister->getActiveMode();
         if ( lm1 == NULL ) return 1;
-        if ( lm1->isType( "NormalMode" ) != true ) return 1;
-        nm1 = (NormalMode*)lm1;
+    }
+
+    if ( ! ( vdm = dynamic_cast< VirtualDirMode * >( lm1 ) ) ) {
+        return 1;
     }
     
     if ( request_flags == true ) {
@@ -251,36 +257,36 @@ SetFilterOp::normalmodesf( ActionMessage *am )
     if ( cont == true ) {
         if ( m_t_change_filters == true ) {
             if ( tfiltermode == UNSET_ALL ) {
-                nm1->unsetAllFilters();
+                vdm->unsetAllFilters();
             } else {
                 if ( ! tfilter.empty() ) {
                     switch ( tfiltermode ) {
                         case INCLUDE_FILTER:
                             try {
-                                NormalMode::nm_filter_t t1 = nm1->checkFilter( tfilter.c_str() );
-                                if ( t1 != NormalMode::NM_FILTER_INCLUDE ) {
-                                    nm1->setFilter( tfilter.c_str(), NormalMode::NM_FILTER_INCLUDE );
+                                VirtualDirMode::vdm_filter_t t1 = vdm->checkFilter( tfilter );
+                                if ( t1 != VirtualDirMode::FILTER_INCLUDE ) {
+                                    vdm->setFilter( tfilter, VirtualDirMode::FILTER_INCLUDE );
                                 } else {
-                                    nm1->setFilter( tfilter.c_str(), NormalMode::NM_FILTER_UNSET );
+                                    vdm->setFilter( tfilter, VirtualDirMode::FILTER_UNSET );
                                 }
                             } catch ( int ) {
-                                nm1->setFilter( tfilter.c_str(), NormalMode::NM_FILTER_INCLUDE );
+                                vdm->setFilter( tfilter, VirtualDirMode::FILTER_INCLUDE );
                             }
                             break;
                         case EXCLUDE_FILTER:
                             try {
-                                NormalMode::nm_filter_t t1 = nm1->checkFilter( tfilter.c_str() );
-                                if ( t1 != NormalMode::NM_FILTER_EXCLUDE ) {
-                                    nm1->setFilter( tfilter.c_str(), NormalMode::NM_FILTER_EXCLUDE );
+                                VirtualDirMode::vdm_filter_t t1 = vdm->checkFilter( tfilter );
+                                if ( t1 != VirtualDirMode::FILTER_EXCLUDE ) {
+                                    vdm->setFilter( tfilter, VirtualDirMode::FILTER_EXCLUDE );
                                 } else {
-                                    nm1->setFilter( tfilter.c_str(), NormalMode::NM_FILTER_UNSET );
+                                    vdm->setFilter( tfilter, VirtualDirMode::FILTER_UNSET );
                                 }
                             } catch ( int ) {
-                                nm1->setFilter( tfilter.c_str(), NormalMode::NM_FILTER_EXCLUDE );
+                                vdm->setFilter( tfilter, VirtualDirMode::FILTER_EXCLUDE );
                             }
                             break;
                         case UNSET_FILTER:
-                            nm1->setFilter( tfilter.c_str(), NormalMode::NM_FILTER_UNSET );
+                            vdm->setFilter( tfilter, VirtualDirMode::FILTER_UNSET );
                             break;
                         default:
                             break;
@@ -294,20 +300,20 @@ SetFilterOp::normalmodesf( ActionMessage *am )
                 openQueryPopUp( am );
             } else {
                 if ( m_t_option_mode == SET_OPTION ) {
-                    nm1->setBookmarkFilter( m_t_bookmark_filter );
+                    vdm->setBookmarkFilter( m_t_bookmark_filter );
                     if ( m_t_bookmark_filter == DirFilterSettings::SHOW_ONLY_LABEL ) {
-                        nm1->setSpecificBookmarkLabel( m_t_bookmark_label );
+                        vdm->setSpecificBookmarkLabel( m_t_bookmark_label );
                     }
                 } else {
                     if ( m_t_bookmark_filter == DirFilterSettings::SHOW_ONLY_LABEL &&
-                         nm1->getBookmarkFilter() != m_t_bookmark_filter ) {
-                        nm1->setSpecificBookmarkLabel( m_t_bookmark_label );
+                         vdm->getBookmarkFilter() != m_t_bookmark_filter ) {
+                        vdm->setSpecificBookmarkLabel( m_t_bookmark_label );
                     }
-                    
-                    if ( nm1->getBookmarkFilter() == m_t_bookmark_filter ) {
-                        nm1->setBookmarkFilter( DirFilterSettings::SHOW_ALL );
+
+                    if ( vdm->getBookmarkFilter() == m_t_bookmark_filter ) {
+                        vdm->setBookmarkFilter( DirFilterSettings::SHOW_ALL );
                     } else {
-                        nm1->setBookmarkFilter( m_t_bookmark_filter );
+                        vdm->setBookmarkFilter( m_t_bookmark_filter );
                     }
                 }
             }
@@ -346,7 +352,7 @@ SetFilterOp::doconfigure(int mode)
   
   tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(getDescription())+1);
   sprintf(tstr,catalog.getLocale(293),getDescription());
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe(tstr);
 
@@ -742,7 +748,7 @@ void SetFilterOp::initHistory()
 
     str1 = WorkerInitialSettings::getInstance().getConfigBaseDir();
     str1 = NWC::Path::join( str1, "setfilter-history" );
-    m_history = std::auto_ptr< PersistentStringList >( new PersistentStringList( str1 ) );
+    m_history = std::unique_ptr< PersistentStringList >( new PersistentStringList( str1 ) );
 }
 
 void SetFilterOp::closeHistory()
@@ -817,28 +823,30 @@ void SetFilterOp::popupCallback( AGMessage *msg )
     if ( msg == NULL ) return;
     if ( msg->type != AG_POPUPMENU_CLICKED ) return;
 
-    NormalMode *nm1 = NULL;
+    VirtualDirMode *vdm = NULL;
     ListerMode *lm1 = NULL;
   
     if ( startlister == NULL ) return;
     lm1 = startlister->getActiveMode();
     if ( lm1 == NULL ) return;
-    if ( lm1->isType( "NormalMode" ) != true ) return;
-    nm1 = (NormalMode*)lm1;
+
+    if ( ! ( vdm = dynamic_cast< VirtualDirMode * >( lm1 ) ) ) {
+        return;
+    }
     
     int id = msg->popupmenu.clicked_entry_id;
     if ( m_label_id_to_action.count( id ) > 0 ) {
         switch ( m_label_id_to_action[id].type ) {
             case DirFilterSettings::SHOW_ONLY_BOOKMARKS:
-                nm1->setBookmarkFilter( DirFilterSettings::SHOW_ONLY_BOOKMARKS );
+                vdm->setBookmarkFilter( DirFilterSettings::SHOW_ONLY_BOOKMARKS );
                 break;
             case DirFilterSettings::SHOW_ONLY_LABEL:
-                nm1->setBookmarkFilter( DirFilterSettings::SHOW_ONLY_LABEL );
-                nm1->setSpecificBookmarkLabel( m_label_id_to_action[id].label );
+                vdm->setBookmarkFilter( DirFilterSettings::SHOW_ONLY_LABEL );
+                vdm->setSpecificBookmarkLabel( m_label_id_to_action[id].label );
                 break;
             case DirFilterSettings::SHOW_ALL:
             default:
-                nm1->setBookmarkFilter( DirFilterSettings::SHOW_ALL );
+                vdm->setBookmarkFilter( DirFilterSettings::SHOW_ALL );
                 break;
         }
     }
@@ -855,23 +863,26 @@ void SetFilterOp::openQueryPopUp( ActionMessage *am )
     m_label_id_to_action.clear();
     cleanUpPopUp();
 
-    NormalMode *nm1 = NULL;
+    VirtualDirMode *vdm = NULL;
     ListerMode *lm1 = NULL;
   
     if ( startlister == NULL ) return;
     lm1 = startlister->getActiveMode();
     if ( lm1 == NULL ) return;
-    if ( lm1->isType( "NormalMode" ) != true ) return;
-    nm1 = (NormalMode*)lm1;
+
+    if ( ! ( vdm = dynamic_cast< VirtualDirMode * >( lm1 ) ) ) {
+        return;
+    }
     
     PopUpMenu::PopUpEntry e1;
     e1.name = catalog.getLocale( 841 );
 
-    if ( nm1->getBookmarkFilter() != DirFilterSettings::SHOW_ALL ) {
+    if ( vdm->getBookmarkFilter() != DirFilterSettings::SHOW_ALL ) {
         e1.type = PopUpMenu::NORMAL;
     } else {
         e1.type = PopUpMenu::GREYED_OUT;
     }
+
     e1.id = descr_id++;
 
     m_label_id_to_action[e1.id] = query_popup_t( DirFilterSettings::SHOW_ALL, "" );
@@ -879,11 +890,13 @@ void SetFilterOp::openQueryPopUp( ActionMessage *am )
     m1.push_back( e1 );
 
     e1.name = catalog.getLocale( 842 );
-    if ( nm1->getBookmarkFilter() != DirFilterSettings::SHOW_ONLY_BOOKMARKS ) {
+
+    if ( vdm->getBookmarkFilter() != DirFilterSettings::SHOW_ONLY_BOOKMARKS ) {
         e1.type = PopUpMenu::NORMAL;
     } else {
         e1.type = PopUpMenu::GREYED_OUT;
     }
+
     e1.id = descr_id++;
 
     m_label_id_to_action[e1.id] = query_popup_t( DirFilterSettings::SHOW_ONLY_BOOKMARKS, "" );
@@ -909,12 +922,13 @@ void SetFilterOp::openQueryPopUp( ActionMessage *am )
           it1 != cats.end();
           ++it1 ) {
 
-        if ( nm1->getBookmarkFilter() != DirFilterSettings::SHOW_ONLY_LABEL ||
-             nm1->getSpecificBookmarkLabel() != *it1 ) {
+        if ( vdm->getBookmarkFilter() != DirFilterSettings::SHOW_ONLY_LABEL ||
+             vdm->getSpecificBookmarkLabel() != *it1 ) {
             e1.type = PopUpMenu::NORMAL;
         } else {
             e1.type = PopUpMenu::GREYED_OUT;
         }
+
         e1.name = *it1;
         e1.id = descr_id++;
 
@@ -931,12 +945,13 @@ void SetFilterOp::openQueryPopUp( ActionMessage *am )
         m1.push_back( e1 );
     }
     
-    if ( nm1->getBookmarkFilter() != DirFilterSettings::SHOW_ONLY_LABEL ||
-         nm1->getSpecificBookmarkLabel() != "" ) {
+    if ( vdm->getBookmarkFilter() != DirFilterSettings::SHOW_ONLY_LABEL ||
+         vdm->getSpecificBookmarkLabel() != "" ) {
         e1.type = PopUpMenu::NORMAL;
     } else {
         e1.type = PopUpMenu::GREYED_OUT;
     }
+
     e1.name = catalog.getLocale( 848 );
     e1.fg = e1.bg = -1;
     e1.id = descr_id++;
@@ -952,8 +967,8 @@ void SetFilterOp::openQueryPopUp( ActionMessage *am )
         m_popup_menu->show();
     } else {
         int tx, ty, tw, th;
-        
-        if ( nm1->queryLVDimensions( tx, ty, tw, th ) == 0 ) {
+
+        if ( vdm->queryLVDimensions( tx, ty, tw, th ) == 0 ) {
             tx += tw / 2;
             ty += th / 2;
             m_popup_menu->show( tx, ty, PopUpMenu::POPUP_CENTER_IN_POSITION );
