@@ -1,6 +1,6 @@
 /* setsortmodeop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2004,2008,2010-2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2004,2008,2010-2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,7 +21,6 @@
 
 #include "setsortmodeop.h"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "verzeichnis.hh"
 #include "datei.h"
@@ -29,6 +28,7 @@
 #include <aguix/cyclebutton.h>
 #include <aguix/choosebutton.h>
 #include "worker_locale.h"
+#include "virtualdirmode.hh"
 
 /*****************
  SetSortmodeOp
@@ -67,15 +67,13 @@ const char *SetSortmodeOp::getName()
 int SetSortmodeOp::run( WPUContext *wpu, ActionMessage *msg )
 {
   ListerMode *lm1;
-  NormalMode *nm1;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
     Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       lm1=l1->getActiveMode();
       if(lm1!=NULL) {
-        if(lm1->isType("NormalMode")==true) {
-          nm1=(NormalMode*)lm1;
-          nm1->setSortmode(mode);
+        if ( auto vdm = dynamic_cast< VirtualDirMode *>( lm1 ) ) {
+            vdm->setSortmode( mode );
         }
       }
     }
@@ -150,7 +148,7 @@ int SetSortmodeOp::configure()
   
   tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(getName())+1);
   sprintf(tstr,catalog.getLocale(293),getName());
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe(tstr);
 
