@@ -1,6 +1,6 @@
 /* showimagemode.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2008 Ralf Hoffmann.
+ * Copyright (C) 2001-2008,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -20,23 +20,24 @@
  */
 
 #include "showimagemode.h"
-#include "normalmode.h"
+#include "listermode.h"
 #include "worker_locale.h"
 #include "wconfig.h"
 #include "wc_color.hh"
 #include "worker.h"
 #include "configheader.h"
-#include "configparser.tab.h"
+#include "configparser.hh"
 #include "fileentry.hh"
 #include "nmspecialsourceext.hh"
 #include <aguix/stringgadget.h>
 #include <aguix/button.h>
 #include <aguix/fieldlistview.h>
+#include <aguix/awindow.h>
 #include "datei.h"
 #include "simplelist.hh"
 
 const char *defaultviewers[] = { "xliwrapper_worker",
-                                "displaywrapper_worker" };
+                                 "displaywrapper_worker" };
 
 const char *ShowImageMode::type="ShowImageMode";
 
@@ -157,7 +158,7 @@ int ShowImageMode::configure()
   
   tstr = (char*)_allocsafe( strlen( catalog.getLocale( 293 ) ) + strlen( getLocaleName() ) + 1 );
   sprintf( tstr, catalog.getLocale( 293 ), getLocaleName() );
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe(tstr);
 
@@ -271,36 +272,6 @@ int ShowImageMode::load()
   return 0;
 }
 
-int ShowImageMode::loadBin(Datei *fh)
-{
-  int chunksize=fh->getInt();
-  int is=Datei::getIntSize();
-  char *tstr;
-  int len;
-
-  if(chunksize>=is) {
-    len=fh->getInt();
-    chunksize-=is;
-    tstr=fh->getString(len);
-    chunksize-=len;
-    
-    // only use the string when not containing a "%s"
-    // so it seems like it was written with worker <= 2.6.1
-    // and I changed the format
-    if ( strstr( tstr, "%s" ) == NULL ) {
-      _freesafe(show_program);
-      show_program=tstr;
-    } else {
-      _freesafe( tstr );
-    }
-  }
-  while(chunksize>0) {
-    fh->getUChar();
-    chunksize--;
-  }
-  return 0;
-}
-
 bool ShowImageMode::save(Datei *fh)
 {
   if ( fh == NULL ) return false;
@@ -315,20 +286,16 @@ void ShowImageMode::setName()
   parentlister->setName(catalog.getLocale(174));
 }
 
-void ShowImageMode::reconfig()
+void ShowImageMode::relayout()
 {
 }
 
 void ShowImageMode::update(bool force)
 {
-  NormalMode *nm1=NULL;
   ListerMode *lm1=NULL;
   Lister *ol=NULL;
-  std::list<NM_specialsourceExt*> *files;
-  std::list<NM_specialsourceExt*>::iterator it1;
   List *colors;
   WC_Color *bgcol;
-  NM_specialsourceExt *ss1;
   const FileEntry *fe;
   char *buf,*colbuf, *buf2;
   std::string str1;
@@ -336,23 +303,16 @@ void ShowImageMode::update(bool force)
   ol=parentlister->getWorker()->getOtherLister(parentlister);
   if(ol!=NULL) {
     lm1=ol->getActiveMode();
-    if(lm1!=NULL)
-      if(lm1->isType("NormalMode")==true)
-        nm1=(NormalMode*)lm1;
   }
   
   if(force==true) clearLastActiveFE();
   
-  if(nm1!=NULL) {
-    files=new std::list<NM_specialsourceExt*>;
+  if(lm1!=NULL) {
+    std::list< NM_specialsourceExt > files;
     
-    nm1->getSelFiles( files, NormalMode::NM_GETFILES_ONLYACTIVE );
-    if ( files->begin() != files->end() )
-      ss1 = *( files->begin() );
-    else
-      ss1 = NULL;
-    if(ss1!=NULL) {
-      fe = ss1->entry();
+    lm1->getSelFiles( files, ListerMode::LM_GETFILES_ONLYACTIVE );
+    if ( ! files.empty() ) {
+      fe = files.begin()->entry();
       if ( fe->equals( lastactivefe ) == false ) {
 	clearLastActiveFE();
 	if ( fe != NULL ) {
@@ -403,12 +363,6 @@ void ShowImageMode::update(bool force)
       clearLastActiveFE();
       blank();
     }
-    
-    for ( it1 = files->begin(); it1 != files->end(); it1++ ) {
-      delete *it1;
-    }
-    
-    delete files;
   }
 }
 
@@ -442,7 +396,7 @@ char *ShowImageMode::requestDefaultViewer()
   ttw = tth = 10;
   ttx = tty = 5;
 
-  win = new AWindow( aguix, 10, 10, ttw, tth, 0, catalog.getLocale( 534 ) );
+  win = new AWindow( aguix, 10, 10, ttw, tth, 0, catalog.getLocale( 534 ), AWindow::AWINDOW_DIALOG );
   win->create();
 
   win->addTextFromString( catalog.getLocale( 535 ),
