@@ -1,6 +1,6 @@
 /* startprogop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2008,2010-2012 Ralf Hoffmann.
+ * Copyright (C) 2001-2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,7 +21,6 @@
 
 #include "startprogop.h"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include <string>
 #include <aguix/kartei.h>
@@ -43,6 +42,9 @@
 #include <aguix/button.h>
 #include "wconfig.h"
 #include "flattypelist.hh"
+#include "argclass.hh"
+#include "nwc_path.hh"
+#include "pers_string_list.hh"
 
 const char *StartProgOp::name="StartProgOp";
 
@@ -92,16 +94,12 @@ StartProgOp::getName()
 int
 StartProgOp::run( WPUContext *wpu, ActionMessage *msg )
 {
-  ListerMode *lm1;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
     Lister *l1 = msg->getWorker()->getActiveLister();
     if(l1!=NULL) {
       startlister=l1;
       endlister = msg->getWorker()->getOtherLister(startlister);
-      lm1=startlister->getActiveMode();
-      if(lm1!=NULL)
-        if(lm1->isType("NormalMode")==true)
-          normalmodestartprog( msg );
+      startprog( msg );
     }
   }
   return 0;
@@ -142,43 +140,52 @@ StartProgOp::getDescription()
 }
 
 int
-StartProgOp::normalmodestartprog( ActionMessage *am )
+StartProgOp::startprog( ActionMessage *am )
 {
-  NormalMode *nm1 = NULL;
   ListerMode *lm1 = NULL;
 
-  char *tmpname, *tmpoutput, *exestr, *tstr;
+  char *tmpname, *tmpoutput, *tstr;
   std::string string1;
   bool useint;
   bool removeTMPFiles;
+  std::string fullname;
 
   if ( startlister == NULL ) return 1;
   lm1 = startlister->getActiveMode();
   if ( lm1 == NULL ) return 1;
-  if ( lm1->isType( "NormalMode" ) != true ) return 1;
-  nm1 = (NormalMode*)lm1;
 
-  std::list<NM_specialsourceExt*> filelist;
+  std::list< NM_specialsourceExt > filelist;
 
-  nm1->getSelFiles( &filelist, NormalMode::NM_GETFILES_ONLYACTIVE );
+  lm1->getSelFiles( filelist, ListerMode::LM_GETFILES_ONLYACTIVE );
 
   const FileEntry *tfe;
   if ( am->mode == am->AM_MODE_SPECIAL ) {
     // Copy of FE is not necessary because it's already duplicated in ActionMessage
     tfe = am->getFE();
-  } else if ( ( filelist.begin() != filelist.end() ) &&
-       ( ( *( filelist.begin() ) )->entry() != NULL ) ) {
-    tfe = (*(filelist.begin()))->entry();
+  } else if ( ! filelist.empty() &&
+              filelist.begin()->entry() != NULL ) {
+    tfe = filelist.begin()->entry();
   } else {
     tfe = NULL;
   }
   if ( tfe != NULL ) {
-    default_file = tfe->name;
+      std::string cwd = lm1->getCurrentDirectory();
+
+      default_file = tfe->name;
+      fullname = tfe->fullname;
+
+      if ( ! cwd.empty() ) {
+          std::string name_string = NWC::Path::get_extended_basename( cwd,
+                                                                      tfe->fullname );
+          if ( ! name_string.empty() ) {
+              default_file = name_string;
+          }
+      }
   } else {
     default_file = "";
   }
 
-  std::auto_ptr< StartSettings > m1;
+  std::unique_ptr< StartSettings > m1;
   m1 = showGUI();
   if ( m1->_startmode == StartSettings::STARTPROG ) {
     tmpname = Datei::createTMPName();
@@ -201,14 +208,14 @@ StartProgOp::normalmodestartprog( ActionMessage *am )
       Datei fp;
       if ( fp.open( tmpname, "w" ) == 0 ) {
         fp.putString( "#! /bin/sh\n" );
-        if ( nm1->getCurrentDir() != NULL ) {
-          if ( ( worker_islocal( nm1->getCurrentDir() ) == 1 ) &&
-               ( m1->_dontcd == false ) ) {
-            tstr = AGUIX_catTrustedAndUnTrusted( "cd ", nm1->getCurrentDir() );
-            fp.putString( tstr );
-            _freesafe( tstr );
-            fp.putString( "\n" );
-          }
+        if ( ! lm1->getCurrentDirectory().empty() ) {
+            if ( ( worker_islocal( lm1->getCurrentDirectory().c_str() ) == 1 ) &&
+                 ( m1->_dontcd == false ) ) {
+                tstr = AGUIX_catTrustedAndUnTrusted( "cd ", lm1->getCurrentDirectory().c_str() );
+                fp.putString( tstr );
+                _freesafe( tstr );
+                fp.putString( "\n" );
+            }
         }
         if ( m1->_global == false ) fp.putString( "./" );
       
@@ -222,6 +229,9 @@ StartProgOp::normalmodestartprog( ActionMessage *am )
           fp.putString( string1.c_str() );
           fp.putString( "\n" );
         }
+
+        std::string exestr;
+
         if ( ( m1->_start == STARTPROGOP_START_IN_TERMINAL_AND_WAIT4KEY )||
              ( m1->_start == STARTPROGOP_START_IN_TERMINAL ) ) {
           /* in terminal starten */
@@ -229,10 +239,11 @@ StartProgOp::normalmodestartprog( ActionMessage *am )
             fp.putStringExt( "echo %s\n", catalog.getLocale( 201 ) );
             fp.putString( "read x\n" );
           }
-          exestr = (char*)_allocsafe( strlen( wconfig->getTerminalStr() ) + strlen( tmpname ) + 1 );
-          sprintf( exestr, wconfig->getTerminalStr(), tmpname );
+          exestr = AGUIXUtils::formatStringToString( wconfig->getTerminalStr(),
+                                                     tmpname );
         } else {
-          exestr = dupstring( tmpname );
+            exestr = "/bin/sh ";
+            exestr += tmpname;
         }
         if ( m1->_start == STARTPROGOP_SHOW_OUTPUT ) {
           useint = true;
@@ -247,7 +258,7 @@ StartProgOp::normalmodestartprog( ActionMessage *am )
         if ( fp.errors() == 0 ) {
           worker_chmod( tmpname, 0700 );
           removeTMPFiles = false;
-          am->getWorker()->runCommand( exestr, tmpname, tmpoutput, m1->_inbackground, pea );
+          am->getWorker()->runCommand( exestr.c_str(), tmpname, tmpoutput, m1->_inbackground, pea );
         } else {
           Requester req( Worker::getAGUIX() );
           char *textstr = (char*)_allocsafe( strlen( catalog.getLocale( 647 ) ) + strlen( tmpname ) + 1 );
@@ -257,7 +268,6 @@ StartProgOp::normalmodestartprog( ActionMessage *am )
           req.request( catalog.getLocale( 347 ), textstr, button.c_str() );
           _freesafe( textstr );
         }
-        _freesafe( exestr );
       }
       if ( removeTMPFiles == true ) {
         worker_unlink( tmpname );
@@ -301,10 +311,18 @@ StartProgOp::normalmodestartprog( ActionMessage *am )
       dir = tfe->fullname;
       dir += "#";
       dir += m1->_archive;
-      nm1->enterDir( dir.c_str() );
+
+      std::list< RefCount< ArgClass > > args;
+
+      args.push_back( new StringArg( dir ) );
+      lm1->runCommand( "enter_dir", args );
     }
   }
-  NormalMode::freeSelFiles( &filelist );
+
+  if ( m1->_startmode != StartSettings::CANCEL ) {
+      am->getWorker()->storePathPersIfInProgress( fullname );
+  }
+
   return 0;
 }
 
@@ -314,7 +332,7 @@ StartProgOp::configure()
   return doconfigure();
 }
 
-std::auto_ptr< StartProgOp::SettingsWidgets > StartProgOp::buildW( AWindow *win )
+std::unique_ptr< StartProgOp::SettingsWidgets > StartProgOp::buildW( AWindow *win )
 {
   SettingsWidgets *m1;
 
@@ -403,7 +421,7 @@ std::auto_ptr< StartProgOp::SettingsWidgets > StartProgOp::buildW( AWindow *win
   
   m1 = new SettingsWidgets( sg, cyb, gcb, ibcb, dcdcb );
 
-  return std::auto_ptr< SettingsWidgets >( m1 );
+  return std::unique_ptr< SettingsWidgets >( m1 );
 }
 
 int StartProgOp::doconfigure()
@@ -424,7 +442,7 @@ int StartProgOp::doconfigure()
   
   tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(32))+1);
   sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(32));
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe(tstr);
 
@@ -435,7 +453,7 @@ int StartProgOp::doconfigure()
   AWindow *subwin1 = new AWindow( aguix, 10, 10, 10, 10, 0, "" );
   ac1->add( subwin1, 0, 0, cmin );
 
-  std::auto_ptr< SettingsWidgets > m1 = buildW( subwin1 );
+  std::unique_ptr< SettingsWidgets > m1 = buildW( subwin1 );
   ac1->readLimits();
 
   AContainer *ac1_3 = ac1->add( new AContainer( win, 2, 1 ), 0, 1 );
@@ -580,6 +598,15 @@ StartProgOp::SettingsWidgets::SettingsWidgets( StringGadget *sg,
 {
 }
 
+static bool isCorrectViewProg( const char *str )
+{
+    const char *pos;
+    if ( strlen( str ) < 1 ) return true;
+    pos = strstr( str, "%s" );
+    if ( pos != NULL ) return true;
+    return false;
+}
+
 int StartProgOp::SettingsWidgets::validate()
 {
   const char *textstr;
@@ -587,7 +614,7 @@ int StartProgOp::SettingsWidgets::validate()
   int erg;
   Requester req( Worker::getAGUIX() );
 
-  if ( NormalMode::isCorrectViewProg( _sg->getText() ) == false ) {
+  if ( isCorrectViewProg( _sg->getText() ) == false ) {
     // show requester
     textstr = catalog.getLocale( 312 );
     buttonstr = (char*)_allocsafe( strlen( catalog.getLocale( 313 ) ) + 1 +
@@ -605,7 +632,7 @@ int StartProgOp::SettingsWidgets::validate()
   return 0;
 }
 
-std::auto_ptr< StartProgOp::StartSettings > StartProgOp::showGUI()
+std::unique_ptr< StartProgOp::StartSettings > StartProgOp::showGUI()
 {
   AGUIX *aguix = Worker::getAGUIX();
   int w, h, sw;
@@ -668,7 +695,7 @@ std::auto_ptr< StartProgOp::StartSettings > StartProgOp::showGUI()
   for ( i = 0; i < lines; i++ ) _freesafe( liness[i] );
   _freesafe( liness );
 
-  AContainer *acsw11_1 = acsw11->add( new AContainer( subwin11, 2, 1 ), 0, 1 );
+  AContainer *acsw11_1 = acsw11->add( new AContainer( subwin11, 2, 2 ), 0, 1 );
   acsw11_1->setMinSpace( 5 );
   acsw11_1->setMaxSpace( 5 );
   acsw11_1->setBorderWidth( 0 );
@@ -676,27 +703,38 @@ std::auto_ptr< StartProgOp::StartSettings > StartProgOp::showGUI()
   tstr = AGUIX_catQuotedAndUnQuoted( "", default_file.c_str() );
 
   acsw11_1->add( new Text( aguix, 0, 0, catalog.getLocale( 639 ), 1 ), 0, 0, cfix );
-  StringGadget *sg1 = (StringGadget*)acsw11_1->add( new StringGadget( aguix, 0,
-								      0, 100, tstr, 0 ), 1, 0, cincw );
+  m_cmd_sg = (StringGadget*)acsw11_1->add( new StringGadget( aguix, 0,
+                                                             0, 100, tstr, 0 ), 1, 0, cincw );
   _freesafe( tstr );
 
+  acsw11_1->add( new Text( aguix, 0, 0, catalog.getLocale( 1035 ), 1 ), 0, 1,
+                 AContainer::ACONT_MINH +
+                 AContainer::ACONT_MINW +
+                 AContainer::ACONT_MAXW +
+                 AContainer::ACONT_NORESIZE );
+  m_completion_lv = acsw11_1->addWidget( new FieldListView( aguix, 0, 0, 100, 75, 0 ),
+                                         1, 1,
+                                         cmin );
+  m_completion_lv->setHBarState( 2 );
+  m_completion_lv->setVBarState( 2 );
+
   AWindow *subwin11_2 = new AWindow( aguix, 0, 0, 10, 10, 0, "" );
   acsw11->add( subwin11_2, 0, 2, cmin );
   subwin11_2->create();
-  std::auto_ptr< SettingsWidgets > m1 = buildW( subwin11_2 );
+  std::unique_ptr< SettingsWidgets > m1 = buildW( subwin11_2 );
   acsw11->readLimits();
 
   AWindow *subwin12 = new AWindow( aguix, 0, 0, 10, 10, 0, "" );
   k1->add( subwin12 );
   subwin12->create();
 
-  std::auto_ptr< FiletypeWindow > ftw = buildFTWindow( subwin12 );
+  std::unique_ptr< FiletypeWindow > ftw = buildFTWindow( subwin12 );
 
   AWindow *subwin13 = new AWindow( aguix, 0, 0, 10, 10, 0, "" );
   k1->add( subwin13 );
   subwin13->create();
 
-  std::auto_ptr< ArchiveWindow > arcw = buildArcWindow( subwin13 );
+  std::unique_ptr< ArchiveWindow > arcw = buildArcWindow( subwin13 );
 
   str1 = catalog.getLocale( 641 );
   str1 += " - F1";
@@ -749,7 +787,21 @@ std::auto_ptr< StartProgOp::StartSettings > StartProgOp::showGUI()
   win->show();
   k1->show();
 
-  sg1->takeFocus();
+  m_cmd_sg->takeFocus();
+
+  std::string cfgfile = Worker::getWorkerConfigDir();
+#ifdef USEOWNCONFIGFILES
+  cfgfile = NWC::Path::join( cfgfile, "startprog-history2" );
+#else
+  cfgfile = NWC::Path::join( cfgfile, "startprog-history" );
+#endif
+
+  PersistentStringList history( cfgfile );
+  m_current_original_file = m_cmd_sg->getText();
+  m_completion_enabled = true;
+  m_previous_sg_content = "";
+
+  updateSGFromComplete( m_cmd_sg, history, m_completion_lv );
 
   AGMessage *msg;
   while((msg=aguix->GetMessage(NULL))!=NULL) aguix->ReplyMessage(msg);
@@ -777,13 +829,27 @@ std::auto_ptr< StartProgOp::StartSettings > StartProgOp::showGUI()
             if ( arcw->_lv->isValidRow( row ) == true ) {
 	      arcw->_sg->setText( arcw->_lv->getText( row, 0 ).c_str() );
             }
+          } else if ( msg->fieldlv.lv == m_completion_lv ) {
+              updateSGFromRow( msg->fieldlv.row, history );
           }
           break;
 	case AG_STRINGGADGET_OK:
-	  if ( msg->stringgadget.sg == sg1 ) {
+	  if ( msg->stringgadget.sg == m_cmd_sg ) {
 	    if ( m1->validate() == 1 ) ende = 1;
 	  }
 	  break;
+    case AG_STRINGGADGET_CURSORCHANGE:
+        if ( msg->stringgadget.sg == m_cmd_sg ) {
+            m_completion_enabled = false;
+        }
+        break;
+    case AG_STRINGGADGET_CONTENTCHANGE:
+        if ( msg->stringgadget.sg == ftw->m_filter_sg ) {
+            ftw->updateView();
+        } else if ( msg->stringgadget.sg == m_cmd_sg ) {
+            updateSGFromComplete( m_cmd_sg, history, m_completion_lv );
+        }
+        break;
         case AG_KEYPRESSED:
           if ( win->isParent( msg->key.window, false ) == true ) {
             switch ( msg->key.key ) {
@@ -819,6 +885,36 @@ std::auto_ptr< StartProgOp::StartSettings > StartProgOp::showGUI()
 	      case XK_F3:
 		k1->optionChange( 2 );
 		break;
+                case XK_Down:
+                    if ( m_completion_lv->getElements() > 0 ) {
+                        int trow = m_completion_lv->getActiveRow();
+
+                        trow++;
+
+                        if ( trow >= m_completion_lv->getElements() ) {
+                            trow = m_completion_lv->getElements() - 1;
+                        }
+
+                        m_completion_lv->setActiveRow( trow );
+                        m_completion_lv->showActive();
+
+                        updateSGFromRow( trow, history );
+                    }
+                    break;
+                case XK_Up:
+                    if ( m_completion_lv->getElements() > 0 ) {
+                        int trow = m_completion_lv->getActiveRow();
+
+                        if ( trow > 0 ) {
+                            trow--;
+                        }
+
+                        m_completion_lv->setActiveRow( trow );
+                        m_completion_lv->showActive();
+
+                        updateSGFromRow( trow, history );
+                    }
+                    break;
             }
           }
           break;
@@ -830,50 +926,64 @@ std::auto_ptr< StartProgOp::StartSettings > StartProgOp::showGUI()
   StartSettings *sets = new StartSettings();
 
   if ( ende == 1 ) {
-    if ( k1->getCurOption() == 0 ) {
-      sets->_startmode = StartSettings::STARTPROG;
-      sets->_command = sg1->getText();
+      if ( k1->getCurOption() == 0 ) {
+          sets->_startmode = StartSettings::STARTPROG;
+          sets->_command = m_cmd_sg->getText();
 
-      sets->_view_str = m1->_sg->getText();
-      switch ( m1->_cyb->getSelectedOption() ) {
-	case 1:
-	  sets->_start = STARTPROGOP_START_IN_TERMINAL;
-	  break;
-	case 2:
-	  sets->_start = STARTPROGOP_START_IN_TERMINAL_AND_WAIT4KEY;
-	  break;
-	case 3:
-	  sets->_start = STARTPROGOP_SHOW_OUTPUT;
-	  break;
-	case 4:
-	  sets->_start = STARTPROGOP_SHOW_OUTPUT_INT;
-	  break;
-	default:
-	  sets->_start = STARTPROGOP_START_NORMAL;
-	  break;
-      }
-      sets->_global = m1->_gcb->getState();
-      sets->_inbackground = m1->_ibcb->getState();
-      sets->_dontcd = m1->_dcdcb->getState();
-    } else if ( k1->getCurOption() == 1 ) {
-      sets->_startmode = StartSettings::HANDLE_TYPE;
+          if ( AGUIXUtils::ends_with( sets->_command, m_current_original_file ) ) {
+              std::string base = sets->_command;
+              base.resize( base.length() - m_current_original_file.length() );
 
-      row = ftw->_lv->getActiveRow();
-      if ( ( ftw->_lv->isValidRow( row ) == true ) &&
-	   ( row < ftw->_flatlist->getNrOfEntries() ) ) {
-        WCFiletype *ft = ftw->_flatlist->getEntry( row ).filetype;
-	if ( ft != NULL ) {
-	  sets->_type = ft;
-	}
+              AGUIXUtils::rstrip( base );
+
+              if ( ! base.empty() ) {
+                  if ( base[0] != ' ' ) {
+                      history.removeEntry( base );
+                      history.pushFrontEntry( base );
+                  }
+              }
+          }
+
+          sets->_view_str = m1->_sg->getText();
+          switch ( m1->_cyb->getSelectedOption() ) {
+              case 1:
+                  sets->_start = STARTPROGOP_START_IN_TERMINAL;
+                  break;
+              case 2:
+                  sets->_start = STARTPROGOP_START_IN_TERMINAL_AND_WAIT4KEY;
+                  break;
+              case 3:
+                  sets->_start = STARTPROGOP_SHOW_OUTPUT;
+                  break;
+              case 4:
+                  sets->_start = STARTPROGOP_SHOW_OUTPUT_INT;
+                  break;
+              default:
+                  sets->_start = STARTPROGOP_START_NORMAL;
+                  break;
+          }
+          sets->_global = m1->_gcb->getState();
+          sets->_inbackground = m1->_ibcb->getState();
+          sets->_dontcd = m1->_dcdcb->getState();
+      } else if ( k1->getCurOption() == 1 ) {
+          sets->_startmode = StartSettings::HANDLE_TYPE;
+
+          row = ftw->_lv->getActiveRow();
+          if ( ( ftw->_lv->isValidRow( row ) == true ) &&
+               ( row < ftw->_flatlist->getNrOfEntries() ) ) {
+              WCFiletype *ft = ftw->_flatlist->getEntry( row ).filetype;
+              if ( ft != NULL ) {
+                  sets->_type = ft;
+              }
+          }
+      } else if ( k1->getCurOption() == 2 ) {
+          sets->_startmode = StartSettings::HANDLE_ARCHIVE;
+          sets->_archive = arcw->_sg->getText();
       }
-    } else if ( k1->getCurOption() == 2 ) {
-      sets->_startmode = StartSettings::HANDLE_ARCHIVE;
-      sets->_archive = arcw->_sg->getText();
-    }
   }
   delete win;
 
-  return std::auto_ptr< StartSettings >( sets );
+  return std::unique_ptr< StartSettings >( sets );
 }
 
 StartProgOp::StartSettings::StartSettings() : _startmode( CANCEL ),
@@ -885,7 +995,7 @@ StartProgOp::StartSettings::StartSettings() : _startmode( CANCEL ),
 {
 }
 
-std::auto_ptr< StartProgOp::FiletypeWindow > StartProgOp::buildFTWindow( AWindow *win )
+std::unique_ptr< StartProgOp::FiletypeWindow > StartProgOp::buildFTWindow( AWindow *win )
 {
   FiletypeWindow *m1;
 
@@ -899,30 +1009,42 @@ std::auto_ptr< StartProgOp::FiletypeWindow > StartProgOp::buildFTWindow( AWindow
   const int cincwnr = cincw +
                       AContainer::ACONT_NORESIZE;
   
-  AContainer *ac1 = win->setContainer( new AContainerBB( win, 1, 2 ), true );
+  AContainer *ac1 = win->setContainer( new AContainerBB( win, 1, 3 ), true );
   ac1->setMinSpace( 5 );
   ac1->setMaxSpace( 5 );
 
   ac1->add( new Text( aguix, 0, 0, catalog.getLocale( 644 ), 1 ), 0, 0, cincwnr );
 
-  lv = (FieldListView*)ac1->add( new FieldListView( aguix, 0, 0, 100, 100, 0 ), 0, 1, cmin );
+  AContainer *ac1_sub1 = ac1->add( new AContainer( win, 2, 1 ),
+                                   0, 1 );
+  ac1_sub1->setMinSpace( 5 );
+  ac1_sub1->setMaxSpace( 5 );
+
+  ac1_sub1->add( new Text( aguix, 0, 0, catalog.getLocale( 793 ), 1 ), 0, 0, AContainer::CO_FIX );
+  auto sg = ac1_sub1->addWidget( new StringGadget( aguix, 0, 0, 50, "", 0 ),
+                                 1, 0, cincw );
+
+  lv = (FieldListView*)ac1->add( new FieldListView( aguix, 0, 0, 100, 100, 0 ), 0, 2, cmin );
   lv->setHBarState( 2 );
   lv->setVBarState( 2 );
   lv->setDisplayFocus( true );
   lv->setAcceptFocus( true );
+  lv->setDefaultColorMode( FieldListView::PRECOLOR_ONLYACTIVE );
 
   FlatTypeList *l1 = wconfig->getFlatTypeList();
   l1->buildLV( lv, NULL );
 
   win->contMaximize( true );
   
-  m1 = new FiletypeWindow( lv, l1 );
+  m1 = new FiletypeWindow( lv, sg, l1 );
 
-  return std::auto_ptr< FiletypeWindow >( m1 );
+  return std::unique_ptr< FiletypeWindow >( m1 );
 }
 
 StartProgOp::FiletypeWindow::FiletypeWindow( FieldListView *lv,
-					     FlatTypeList *flatlist ) : _lv( lv ),
+                                             StringGadget *sg,
+                                             FlatTypeList *flatlist ) : _lv( lv ),
+                                                                        m_filter_sg( sg ),
                                                                         _flatlist( flatlist )
 {
 }
@@ -932,7 +1054,18 @@ StartProgOp::FiletypeWindow::~FiletypeWindow()
   if ( _flatlist != NULL ) delete _flatlist;
 }
 
-std::auto_ptr< StartProgOp::ArchiveWindow > StartProgOp::buildArcWindow( AWindow *win )
+void StartProgOp::FiletypeWindow::updateView()
+{
+    delete _flatlist;
+
+    _flatlist = new FlatTypeList( wconfig->getFiletypes(),
+                                  m_filter_sg->getText() );
+    _flatlist->buildLV( _lv, NULL );
+
+    _lv->redraw();
+}
+
+std::unique_ptr< StartProgOp::ArchiveWindow > StartProgOp::buildArcWindow( AWindow *win )
 {
   ArchiveWindow *m1;
   
@@ -973,7 +1106,7 @@ std::auto_ptr< StartProgOp::ArchiveWindow > StartProgOp::buildArcWindow( AWindow
   win->contMaximize( true );
   m1 = new ArchiveWindow( lv, sg );
 
-  return std::auto_ptr< ArchiveWindow >( m1 );
+  return std::unique_ptr< ArchiveWindow >( m1 );
 }
 
 StartProgOp::ArchiveWindow::ArchiveWindow( FieldListView *lv,
@@ -986,3 +1119,114 @@ void StartProgOp::setGUIMsg( std::string msg )
 {
   gui_msg = msg;
 }
+
+void StartProgOp::updateCompletionList( const std::string &base,
+                                        PersistentStringList &history )
+{
+
+   std::list< std::string > l;
+
+   l = history.getList();
+
+   m_current_completions.clear();
+
+   for ( auto &s : l ) {
+       if ( AGUIXUtils::starts_with( s, base ) ) {
+           m_current_completions.push_back( s );
+       }
+   }
+}
+
+void StartProgOp::updateSGFromComplete( StringGadget *sg,
+                                        PersistentStringList &history,
+                                        FieldListView *lv )
+{
+    std::string old_base;
+
+    old_base = sg->getText();
+
+    std::string prefix( old_base );
+
+    if ( sg->getCursor() < (int)prefix.length() ) {
+        prefix.resize( sg->getCursor() );
+    }
+    
+    updateCompletionList( prefix, history );
+
+    lv->setSize( m_current_completions.size() );
+
+    int row = 0;
+    for ( auto &s : m_current_completions ) {
+        std::string s2( s );
+
+        s2 += " ";
+        s2 += m_current_original_file;
+
+        lv->setText( row, 0, s2 );
+        lv->setPreColors( row, FieldListView::PRECOLOR_ONLYACTIVE );
+
+        row++;
+    }
+    lv->redraw();
+
+    if ( ! m_completion_enabled ) return;
+
+    if ( old_base == m_previous_sg_content ||
+         old_base.length() < m_previous_sg_content.length() ) {
+        m_completion_enabled = false;
+        return;
+    }
+
+    if ( m_current_completions.empty() ) {
+        std::string new_text( prefix );
+
+        new_text += " ";
+        new_text += m_current_original_file;
+
+        sg->setText( new_text.c_str() );
+
+        sg->redraw();
+
+        m_completion_enabled = false;
+        return;
+    }
+
+    if ( sg->getCursor() > 0 ) {
+
+        std::string new_base = m_current_completions.front();
+
+        std::string new_text( new_base );
+
+        m_previous_sg_content = new_text;
+
+        if ( sg->getCursor() < (int)m_previous_sg_content.length() ) {
+            m_previous_sg_content.resize( sg->getCursor() );
+        }
+
+        new_text += " ";
+        new_text += m_current_original_file;
+
+        sg->setText( new_text.c_str() );
+
+        sg->setSelection( sg->getCursor(),
+                          new_text.length() );
+
+        sg->redraw();
+    }
+}
+
+void StartProgOp::updateSGFromRow( int row,
+                                   PersistentStringList &history )
+{
+    std::string t = m_completion_lv->getText( row, 0 );
+
+    m_cmd_sg->setText( t.c_str() );
+    m_cmd_sg->setSelection( m_cmd_sg->getCursor(),
+                            t.length() );
+    m_cmd_sg->redraw();
+
+    m_completion_enabled = false;
+
+    updateSGFromComplete( m_cmd_sg, history, m_completion_lv );
+}
+
