@@ -1,6 +1,6 @@
 /* startprogop.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2008,2010-2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2008,2010-2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -28,6 +28,7 @@
 
 class FlatTypeList;
 class AWindow;
+class PersistentStringList;
 
 class StartProgOp:public FunctionProto
 {
@@ -76,7 +77,7 @@ protected:
   // temp variables
   Lister *startlister,*endlister;
   
-  int normalmodestartprog( ActionMessage *am );
+  int startprog( ActionMessage *am );
 
   class SettingsWidgets
   {
@@ -95,19 +96,24 @@ protected:
     class ChooseButton *_ibcb;
     class ChooseButton *_dcdcb;
   };
-  std::auto_ptr< SettingsWidgets > buildW( AWindow *win );
+  std::unique_ptr< SettingsWidgets > buildW( AWindow *win );
 
   class FiletypeWindow
   {
   public:
-    FiletypeWindow( class FieldListView *lv, FlatTypeList *flatlist );
+    FiletypeWindow( class FieldListView *lv,
+                    StringGadget *sg,
+                    FlatTypeList *flatlist );
     ~FiletypeWindow();
     FiletypeWindow( const FiletypeWindow &other );
     FiletypeWindow &operator=( const FiletypeWindow &other );
     class FieldListView *_lv;
+    StringGadget *m_filter_sg;
     FlatTypeList *_flatlist;
+
+      void updateView();
   };
-  std::auto_ptr< FiletypeWindow > buildFTWindow( AWindow *win );
+  std::unique_ptr< FiletypeWindow > buildFTWindow( AWindow *win );
 
   class ArchiveWindow
   {
@@ -119,7 +125,7 @@ protected:
     class StringGadget *_sg;
     std::vector<std::string> _arclist;
   };
-  std::auto_ptr< ArchiveWindow > buildArcWindow( AWindow *win );
+  std::unique_ptr< ArchiveWindow > buildArcWindow( AWindow *win );
 
   class StartSettings
   {
@@ -138,10 +144,27 @@ protected:
     bool _inbackground;
     bool _dontcd;
   };
-  std::auto_ptr< StartSettings > showGUI();
+  std::unique_ptr< StartSettings > showGUI();
+
+    void updateCompletionList( const std::string &base,
+                               PersistentStringList &history );
+    void updateSGFromComplete( StringGadget *sg,
+                               PersistentStringList &history,
+                               FieldListView *lv );
+    void updateSGFromRow( int row,
+                          PersistentStringList &history );
+
   std::string default_file;
   
   std::string gui_msg;
+
+    std::list< std::string > m_current_completions;
+    std::string m_current_original_file;
+    bool m_completion_enabled;
+    std::string m_previous_sg_content;
+
+    StringGadget *m_cmd_sg;
+    FieldListView *m_completion_lv;
 };
 
 #endif
