@@ -1,6 +1,6 @@
 /* switchbuttonbankop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2011 Ralf Hoffmann.
+ * Copyright (C) 2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -103,7 +103,7 @@ int SwitchButtonBankOp::configure()
     tstr = (char*)_allocsafe( strlen( catalog.getLocale( 293 ) ) +
                               strlen( getDescription() ) + 1 );
     sprintf( tstr, catalog.getLocale( 293 ), getDescription() );
-    win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+    win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
     win->create();
     _freesafe( tstr );
 
