@@ -1,6 +1,6 @@
 /* textviewmode.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2011,2013 Ralf Hoffmann.
+ * Copyright (C) 2011-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -39,6 +39,9 @@ TextViewMode::TextViewMode( Lister *parent ) : ListerMode( parent )
     m_lastactivefe = NULL;
     m_tv = NULL;
     m_cont = NULL;
+    m_info_text = nullptr;
+    m_file_name_text = nullptr;
+    m_file_percentage_text = nullptr;
 
     registerCommand( "up", new MethodCallback<TextViewMode, &TextViewMode::command_up>( this ) );
     registerCommand( "down", new MethodCallback<TextViewMode, &TextViewMode::command_down>( this ) );
@@ -96,7 +99,7 @@ void TextViewMode::on()
     m_ac1->setMinSpace( 5 );
     m_ac1->setMaxSpace( 5 );
 
-    m_ac1->add( new Text( aguix, 0, 0, catalog.getLocale( 404 ), 1 ), 0, 0, AContainer::CO_FIX );
+    m_info_text = m_ac1->addWidget( new Text( aguix, 0, 0, catalog.getLocale( 404 ), 1 ), 0, 0, AContainer::CO_FIX );
     m_file_name_text = (Text*)m_ac1->add( new Text( aguix, 0, 0, "", 1 ),
                                           1, 0, AContainer::ACONT_MINH + AContainer::ACONT_MAXH );
     m_file_percentage_text = (Text*)m_ac1->add( new Text( aguix, 0, 0, "", 1 ),
@@ -120,9 +123,20 @@ void TextViewMode::off()
 {
     delete m_tv;
     m_tv = NULL;
+
+    delete m_info_text;
+    m_info_text = nullptr;
+
+    delete m_file_name_text;
+    m_file_name_text = nullptr;
+
+    delete m_file_percentage_text;
+    m_file_percentage_text = nullptr;
+
     parentlister->setContainer( NULL );
     delete m_cont;
     m_cont = NULL;
+
     parentlister->setActiveMode( NULL );
     parentlister->setName( "" );
     clearLastActiveFE();
