@@ -1,6 +1,6 @@
 /* togglelistermodeop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2005,2011 Ralf Hoffmann.
+ * Copyright (C) 2001-2005,2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,11 +21,11 @@
 
 #include "togglelistermodeop.h"
 #include "listermode.h"
-#include "normalmode.h"
 #include "worker.h"
 #include "datei.h"
 #include <aguix/cyclebutton.h>
 #include <aguix/button.h>
+#include <aguix/awindow.h>
 #include "worker_locale.h"
 
 const char *ToggleListermodeOp::name="ToggleListermodeOp";
@@ -125,7 +125,7 @@ ToggleListermodeOp::configure()
   
   tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(39))+1);
   sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(39));
-  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
+  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr, AWindow::AWINDOW_DIALOG );
   win->create();
   _freesafe(tstr);
 
