@@ -1,6 +1,6 @@
 /* volumemanagerui.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2009-2012 Ralf Hoffmann.
+ * Copyright (C) 2009-2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -46,12 +46,13 @@ VolumeManagerUI::VolumeManagerUI( AGUIX &aguix, HWVolumeManager &hwman ) : m_agu
 									   m_menu_command( VOLMANUI_MENU_EJECT )
 {
     std::string cfgfile = getHiddenDevicesFile();
-    m_hide_entries = std::auto_ptr< PersistentStringList >( new PersistentStringList( cfgfile ) );
+    m_hide_entries = std::unique_ptr< PersistentStringList >( new PersistentStringList( cfgfile ) );
 
-    m_win = std::auto_ptr<AWindow>( new AWindow( &m_aguix,
-                                                 0, 0,
-                                                 500, 400,
-                                                 0, catalog.getLocale( 871 ) ) );
+    m_win = std::unique_ptr<AWindow>( new AWindow( &m_aguix,
+                                                   0, 0,
+                                                   500, 400,
+                                                   0, catalog.getLocale( 871 ),
+                                                   AWindow::AWINDOW_DIALOG ) );
     m_win->create();
 
     AContainer *cont0 = m_win->setContainer( new AContainer( m_win.get(), 1, 4 ), true );
@@ -514,7 +515,8 @@ void VolumeManagerUI::editHideList()
     AWindow *win = new AWindow( &m_aguix,
                                 0, 0,
                                 500, 400,
-                                0, catalog.getLocale( 882 ) );
+                                0, catalog.getLocale( 882 ),
+                                AWindow::AWINDOW_DIALOG );
     win->create();
     
     AContainer *cont0 = win->setContainer( new AContainer( win, 1, 4 ), true );
