@@ -1,6 +1,6 @@
 /* wcfiletype.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2006-2013 Ralf Hoffmann.
+ * Copyright (C) 2006-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -200,7 +200,7 @@ void WCFiletype::setPattern(const char *npattern)
   }
 }
 
-char *WCFiletype::getName()
+const char *WCFiletype::getName() const
 {
   return name;
 }
