@@ -179,6 +179,9 @@ public:
   void setShowHeader( int side, bool nv );
   bool getShowHeader( int side ) const;
 
+  void setPathEntryOnTop( int side, bool nv );
+  bool getPathEntryOnTop( int side ) const;
+
   int writeDateToString( std::string &str, struct tm *timeptr );
   
   bool checkForNewerConfig();
@@ -310,6 +313,9 @@ public:
   void setUseVersionStringCompare( bool nv );
   bool getUseVersionStringCompare() const;
 
+  void setPathJumpStoreFilesAlways( bool nv );
+  bool getPathJumpStoreFilesAlways() const;
+
   void setApplyWindowDialogType( bool nv );
   bool getApplyWindowDialogType() const;
 
@@ -409,6 +415,7 @@ protected:
   char *stringForDirSize;
   int stringForDirSizeLen;
   bool showHeader[2];
+  bool m_path_entry_on_top[2];
   
   WCFiletype *notyettype; // Nur zum schnelleren Auslesen, muss nicht abgespeichert werden
   WCFiletype *dirtype; // Nur zum schnelleren Auslesen, muss nicht abgespeichert werden
@@ -510,6 +517,7 @@ protected:
     bool m_use_version_string_compare;
     bool m_apply_window_dialog_type;
     std::list< std::string > m_pathjump_allow_dirs;
+    bool m_pathjump_store_files_always;
 
     int m_loaded_major_version;
     int m_loaded_minor_version;
