@@ -49,10 +49,9 @@ BookmarkColorPanel::~BookmarkColorPanel()
 
 int BookmarkColorPanel::create()
 {
-    int res;
     int trow;
     
-    res = Panel::create();
+    Panel::create();
 
     AContainer *ac1 = setContainer( new AContainer( this, 1, 4 ), true );
     ac1->setMinSpace( 5 );
