@@ -1,6 +1,6 @@
 /* wconfig_filetype.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2006-2008,2011 Ralf Hoffmann.
+ * Copyright (C) 2006-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -74,6 +74,7 @@ int FiletypePanel::create()
   lv->setVBarState( 2 );
   lv->setAcceptFocus( true );
   lv->setDisplayFocus( true );
+  lv->setDefaultColorMode( FieldListView::PRECOLOR_NORMAL );
 
   AContainer *ac1_1 = ac1->add( new AContainer( this, 1, 3 ), 0, 3 );
   ac1_1->setBorderWidth( 0 );
@@ -135,7 +136,6 @@ void FiletypePanel::run( Widget *elem, AGMessage &msg )
   int row;
   std::string s1;
   WCFiletype *pft;
-  bool rem;
 
   if ( _need_recreate == true ) return;
 
@@ -185,40 +185,95 @@ void FiletypePanel::run( Widget *elem, AGMessage &msg )
 	}
       }
     } else if ( msg.button.button == delb ) {
-      row = lv->getActiveRow();
-      if ( ( lv->isValidRow( row ) == true ) &&
-	   ( row < l1->getNrOfEntries() ) ) {
-	if ( l1->getEntry( row ).filetype != NULL ) {
-	  WCFiletype *tf1 = l1->getEntry( row ).filetype;
-	  if( ( tf1->getinternID() == NORMALTYPE ) ||
-	      ( allowRemoveUnique == true ) ) {
-	    rem = true;
-	    if ( tf1->getSubTypeList() != NULL ) {
-	      if ( tf1->getSubTypeList()->size() > 0 ) {
-		s1 = catalog.getLocale( 11 );
-		s1 += "|";
-		s1 += catalog.getLocale( 8 );
-		int erg = request( catalog.getLocale( 123 ),
-                                   catalog.getLocale( 597 ),
-                                   s1.c_str() );
-		if ( erg != 0 ) rem = false;
-	      }
-	    }
-	    if ( rem == true ) {
-	      pft = l1->findParent( row );
-	      if ( pft == NULL ) {
-		// no parent
-		filetypes->removeElement( tf1 );
-	      } else {
-		pft->removeSubType( tf1 );
-	      }
-	      delete tf1;
-	      
-	      updateConf( filetypes, &l1, lv, NULL );
-	    }
-	  }
-	}
-      }
+        std::list< std::pair< WCFiletype *, int > > selected;
+        std::set< WCFiletype * > to_delete;
+        int max_depth = 0;
+        bool rem = true;
+        bool ask = false;
+
+        // get all selected and store corresponding depth
+        for ( row = 0; row < lv->getElements(); row++ ) {
+            if ( lv->getSelect( row ) ) {
+                WCFiletype *tf1 = l1->getEntry( row ).filetype;
+                int depth = l1->getEntry( row ).depth;
+
+                if ( tf1 ) {
+                    selected.push_back( std::make_pair( tf1, depth ) );
+                    if ( depth > max_depth ) max_depth = depth;
+                }
+            }
+        }
+
+        // if none are selected, use the active one
+        if ( selected.empty() ) {
+            row = lv->getActiveRow();
+            if ( lv->isValidRow( row ) ) {
+                WCFiletype *tf1 = l1->getEntry( row ).filetype;
+                int depth = l1->getEntry( row ).depth;
+
+                if ( tf1 ) {
+                    selected.push_back( std::make_pair( tf1, depth ) );
+                    if ( depth > max_depth ) max_depth = depth;
+                }
+            }
+        }
+
+        // now check for each entry if any of its parent is already
+        // inside the list to delete
+        // start with top level entries, then go deeper
+        for ( int depth = 0; depth <= max_depth; depth++ ) {
+            for ( auto &p : selected ) {
+                if ( p.second == depth ) {
+                    WCFiletype *tft = p.first->getParentType();
+                    bool found = false;
+                    while ( tft != NULL ) {
+                        if ( to_delete.count( tft ) != 0 ) {
+                            found = true;
+                            break;
+                        }
+                        tft = tft->getParentType();
+                    }
+
+                    if ( ! found ) {
+                        if ( p.first->getSubTypeList() != NULL &&
+                             p.first->getSubTypeList()->size() > 0 ) {
+                            ask = true;
+                        }
+
+                        to_delete.insert( p.first );
+                    }
+                }
+            }
+        }
+
+        if ( to_delete.size() > 1 ||
+             ask ) {
+            s1 = catalog.getLocale( 11 );
+            s1 += "|";
+            s1 += catalog.getLocale( 8 );
+            int erg = request( catalog.getLocale( 123 ),
+                               catalog.getLocale( 1037 ),
+                               s1.c_str() );
+            if ( erg != 0 ) rem = false;
+        }
+
+        if ( rem ) {
+            for ( auto &tf1 : to_delete ) {
+                if( ( tf1->getinternID() == NORMALTYPE ) ||
+                    ( allowRemoveUnique == true ) ) {
+                    pft = tf1->getParentType();
+                    if ( pft == NULL ) {
+                        // no parent
+                        filetypes->removeElement( tf1 );
+                    } else {
+                        pft->removeSubType( tf1 );
+                    }
+                    delete tf1;
+                }
+            }
+
+            updateConf( filetypes, &l1, lv, NULL );
+        }
     } else if ( msg.button.button == cutb ) {
       row = lv->getActiveRow();
       if ( ( lv->isValidRow( row ) == true ) &&
