@@ -1,6 +1,6 @@
 /* wconfig_generalconf.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2007,2011 Ralf Hoffmann.
+ * Copyright (C) 2007,2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -44,7 +44,11 @@ int GeneralConfPanel::create()
     
     ac1->add( new Text( _aguix, 0, 0, catalog.getLocale( 784 ), 1 ), 0, 0, AContainer::CO_INCWNR );
 
+#if 0
+    AContainerBB *ac1_1 = (AContainerBB*)ac1->add( new AContainerBB( this, 1, 3 ), 0, 1 );
+#else
     AContainerBB *ac1_1 = (AContainerBB*)ac1->add( new AContainerBB( this, 1, 2 ), 0, 1 );
+#endif
     ac1_1->setMinSpace( 5 );
     ac1_1->setMaxSpace( 5 );
     ac1_1->setBorderWidth( 7 );
@@ -66,6 +70,17 @@ int GeneralConfPanel::create()
                                                                 LABEL_LEFT,
                                                                 1,
                                                                 0 ), 0, 1, AContainer::CO_INCWNR );
+
+#if 0
+    m_dialog_cb = (ChooseButton*)ac1_1->add( new ChooseButton( _aguix,
+                                                               0,
+                                                               0,
+                                                               _baseconfig.getApplyWindowDialogType(),
+                                                               catalog.getLocale( 999 ),
+                                                               LABEL_LEFT,
+                                                               1,
+                                                               0 ), 0, 2, AContainer::CO_INCWNR );
+#endif
     
     contMaximize( true );
     return 0;
@@ -75,5 +90,8 @@ int GeneralConfPanel::saveValues()
 {
     _baseconfig.setSaveWorkerStateOnExit( m_cb->getState() );
     _baseconfig.setUseVersionStringCompare( m_versort_cb->getState() );
+#if 0
+    _baseconfig.setApplyWindowDialogType( m_dialog_cb->getState() );
+#endif
     return 0;
 }
