@@ -49,9 +49,7 @@ PathPanel::~PathPanel()
 
 int PathPanel::create()
 {
-  int res;
-  
-  res = Panel::create();
+  Panel::create();
 
   char bankstr[ A_BYTESFORNUMBER( int ) ];
   unsigned int i2;
