@@ -1,6 +1,6 @@
 /* wconfig_pathjump_allow.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2012 Ralf Hoffmann.
+ * Copyright (C) 2012-2014 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -25,6 +25,7 @@
 #include "worker_locale.h"
 #include <aguix/fieldlistview.h>
 #include <aguix/button.h>
+#include <aguix/choosebutton.h>
 #include "datei.h"
 #include "nwc_path.hh"
 
@@ -43,7 +44,7 @@ int PathJumpAllowPanel::create()
 
     int trow;
 
-    AContainer *ac1 = setContainer( new AContainer( this, 1, 3 ), true );
+    AContainer *ac1 = setContainer( new AContainer( this, 1, 4 ), true );
     ac1->setMinSpace( 5 );
     ac1->setMaxSpace( 5 );
 
@@ -69,6 +70,13 @@ int PathJumpAllowPanel::create()
                                   1, 0, AContainer::CO_INCW );
     m_delb->connect( this );
 
+    m_store_files_cb = ac1->addWidget( new ChooseButton( _aguix, 0, 0,
+                                                         _baseconfig.getPathJumpStoreFilesAlways(),
+                                                         catalog.getLocale( 1043 ),
+                                                         LABEL_RIGHT,
+                                                         1,
+                                                         0 ), 0, 3, AContainer::CO_INCWNR );
+
     contMaximize( true );
 
     const std::list< std::string > &l = _baseconfig.getPathJumpAllowDirs();
@@ -100,6 +108,8 @@ int PathJumpAllowPanel::saveValues()
   
     _baseconfig.setPathJumpAllowDirs( l );
 
+    _baseconfig.setPathJumpStoreFilesAlways( m_store_files_cb->getState() );
+
     return 0;
 }
 
