@@ -36,9 +36,7 @@ TermConfPanel::~TermConfPanel()
 
 int TermConfPanel::create()
 {
-  int res;
-  
-  res = Panel::create();
+  Panel::create();
 
   AContainer *ac1 = setContainer( new AContainer( this, 1, 4 ), true );
   ac1->setBorderWidth( 5 );
