@@ -235,6 +235,8 @@ public:
     static PersistentKVP &getKVPStore();
 
     PrefixDB &getDirHistPrefixDB();
+
+    bool getKeyChainCandidateActive() const;
 protected:
   void createMainWin();
   void setupMainWin();
@@ -411,6 +413,8 @@ protected:
 
     std::string getOverrideXIMPath() const;
 
+    void setKeyChainCandidateActive( bool nv );
+
     DeepPathStore m_path_store;
     std::unique_ptr< PersDeepPathStore > m_pers_path_store;
 
@@ -436,6 +440,8 @@ protected:
     std::string m_path_jump_last_filter;
 
     std::unique_ptr< PrefixDB > m_pdb;
+
+    bool m_key_chain_cand_active;
 };
 
 #endif
