@@ -1,6 +1,6 @@
 /* worker_commands.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2007,2009,2011-2012 Ralf Hoffmann.
+ * Copyright (C) 2001-2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -211,6 +211,9 @@ FunctionProto *Worker::getCommand4ID(int id)
     case 61:
       fpr = new ClipboardOp();
       break;
+    case 62:
+      fpr = new CommandMenuOp();
+      break;
     default:
       fpr=new DNDAction();
   }
@@ -282,6 +285,7 @@ int Worker::getID4Command(FunctionProto *fpr)
   else if ( fpr->isName( SwitchButtonBankOp::name ) == true ) type = 59;
   else if ( fpr->isName( PathJumpOp::name ) == true ) type = 60;
   else if ( fpr->isName( ClipboardOp::name ) == true ) type = 61;
+  else if ( fpr->isName( CommandMenuOp::name ) == true ) type = 62;
   return type;
 }
 
@@ -289,7 +293,7 @@ int Worker::getMaxModeNr()
 {
   // leider hardcoded, da aber die schlechte Methode in dieser Klasse bleibt, nicht so
   // schlimm
-  return 4;
+    return 4;
 }
 
 ListerMode *Worker::getMode4ID(int nr,Lister *parent)
@@ -306,7 +310,7 @@ ListerMode *Worker::getMode4ID(int nr,Lister *parent)
       lm = new TextViewMode( parent );
       break;
     default:
-      lm=new NormalMode(parent);
+      lm = new VirtualDirMode( parent );
       break;
   }
   return lm;
@@ -325,7 +329,7 @@ const char *Worker::getNameOfMode( int nr )
       return TextViewMode::getStaticType();
       break;
   }
-  return NormalMode::getStaticType();
+  return VirtualDirMode::getStaticType();
 }
 
 const char *Worker::getLocaleNameOfMode( int nr )
@@ -341,7 +345,7 @@ const char *Worker::getLocaleNameOfMode( int nr )
       return TextViewMode::getStaticLocaleName();
       break;
   }
-  return NormalMode::getStaticLocaleName();
+  return VirtualDirMode::getStaticLocaleName();
 }
 
 int Worker::getID4Name( const char *str )
@@ -350,17 +354,17 @@ int Worker::getID4Name( const char *str )
   if ( strcasecmp( str, ShowImageMode::getStaticType() ) == 0 ) return 1;
   if ( strcasecmp( str, InformationMode::getStaticType() ) == 0 ) return 2;
   if ( strcasecmp( str, TextViewMode::getStaticType() ) == 0 ) return 3;
-  if ( strcasecmp( str, NormalMode::getStaticType() ) == 0 ) return 0;
+  if ( strcasecmp( str, VirtualDirMode::getStaticType() ) == 0 ) return 0;
   return -1;
 }
 
 int Worker::getID4Mode(ListerMode *lm)
 {
   int type=-1;
-  if(lm->isType("NormalMode")==true) type=0;
-  else if(lm->isType("ShowImageMode")==true) type=1;
+  if(lm->isType("ShowImageMode")==true) type=1;
   else if(lm->isType("InformationMode")==true) type=2;
   else if ( lm->isType( "TextViewMode" ) == true ) type = 3;
+  else if ( lm->isType( "VirtualDirMode" ) == true ) type = 0;
   return type;
 }
 
