@@ -1,6 +1,6 @@
 /* worker_commands.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2009,2011-2012 Ralf Hoffmann.
+ * Copyright (C) 2001-2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -65,12 +65,13 @@
 #include "switchbuttonbankop.hh"
 #include "pathjumpop.hh"
 #include "clipboardop.hh"
+#include "commandmenuop.hh"
 
 #include "listermode.h"
-#include "normalmode.h"
 #include "showimagemode.h"
 #include "informationmode.h"
 #include "textviewmode.hh"
+#include "virtualdirmode.hh"
 
 #endif
 
