@@ -27,7 +27,7 @@
 #include "nwc_path.hh"
 #include "datei.h"
 
-#define CATENTRY 1039
+#define CATENTRY 1048
 #define CATCOMENTRY 63
 #define CATFLAGENTRY 59
 
@@ -1141,6 +1141,15 @@ match any font." );
   catalog[1036]=dupstring( "true if entry is a broken symlink" );
   catalog[1037]=dupstring( "Do you really want to delete all selected file types|and its sub-types?" );
   catalog[1038]=dupstring( "Select registered command:" );
+  catalog[1039]=dupstring( "Continue in background" );
+  catalog[1040]=dupstring( "You are going to copy %s|which is currently (partly) a destination for a background copy.|This may lead to unwanted results!" );
+  catalog[1041]=dupstring( "Path entry on top" );
+  catalog[1042]=dupstring( "The source file %s has been changed while copying.|Do you want to continue until the file is complete,|or skip this file and copy the other remaining files,|or cancel the operation at all?" );
+  catalog[1043]=dupstring( "Always store used files in database" );
+  catalog[1044]=dupstring( "Also show \"..\" entry" );
+  catalog[1045]=dupstring( "Show breadcrumb navigation" );
+  catalog[1046]=dupstring( "Copy finished" );
+  catalog[1047]=dupstring( "Keep window opened" );
 
 //  catalog[]=dupstring( "" );
   
