@@ -1,6 +1,6 @@
 /* wpucontext.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2002-2005,2008,2011 Ralf Hoffmann.
+ * Copyright (C) 2002-2005,2008,2011,2013 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -21,19 +21,24 @@
 
 #include "wpucontext.h"
 #include "scriptop.h"
-#include "normalmode.h"
 #include "execlass.h"
 #include "nmextlist.hh"
 #include "simplelist.hh"
 #include "worker.h"
 #include "dnd.h"
 #include "worker_locale.h"
+#include <aguix/aguix.h>
 #include <aguix/solidbutton.h>
 #include <aguix/button.h>
+#include <aguix/awindow.h>
+#include <aguix/bevelbox.h>
 #include "datei.h"
 #include <iostream>
 #include "nmexternfe.hh"
 #include "nwc_fsentry.hh"
+#include "listermode.h"
+#include "nwc_path.hh"
+#include "virtualdirmode.hh"
 
 WPUStack::WPUStack()
 {
@@ -149,7 +154,6 @@ WPUContext::WPUContext( Worker *w, List *coms, bool delete_commands_at_destroy,
   for ( p = keywords; p->token != IFP_NONE; p++ ) {
     ifp_insert( p->str, p->token );
   }
-  extorder.wpu = this;
   lasterror = 0;
   
   userwin = NULL;
@@ -163,51 +167,40 @@ WPUContext::WPUContext( Worker *w, List *coms, bool delete_commands_at_destroy,
 
   activemode = worker->getActiveLister()->getActiveMode();
   nonactivemode = worker->getOtherLister( worker->getActiveLister() )->getActiveMode();
+  destmode = NULL;
+
+  m_recursive = false;
+  m_take_dirs = false;
+  m_use_virtual_temp_copies = false;
+  m_use_extended_basename = true;
 
   if ( msg != NULL ) {
     if ( msg->mode != ActionMessage::AM_MODE_NORMAL ) {
       if ( msg->mode == ActionMessage::AM_MODE_DNDACTION ) {
         const DNDMsg *dndmsg = msg->dndmsg;
         ListerMode *lm1;
-        NormalMode *nm1;
 
-        lm1 = dndmsg->getDestMode();
-        if ( lm1 != NULL ) {
-          if ( lm1->isType( "NormalMode" ) == true ) {
-            nm1 = (NormalMode*)lm1;
-            extorder.destmode = nm1;
-          }
-        }
+        destmode = dndmsg->getDestMode();
+
         lm1 = dndmsg->getSourceMode();
         if ( lm1 != NULL ) {
-          if ( lm1->isType( "NormalMode" ) == true ) {
-            nm1 = (NormalMode*)lm1;
             extlist[0] = new NMExtList();
-            extlist[0]->createExtList( nm1, msg, false );
-            extlist[0]->setTakeDirs( extorder.take_dirs );
-          }
+            extlist[0]->createExtList( lm1, msg, false );
+            extlist[0]->setTakeDirs( m_take_dirs );
         }
         activemode = lm1;
         nonactivemode = worker->getOtherLister( dndmsg->getSourceLister() )->getActiveMode();
       } else if ( msg->mode == ActionMessage::AM_MODE_SPECIAL ) {
-        NormalMode *nm1;
-
         if ( activemode != NULL ) {
-          if ( activemode->isType( "NormalMode" ) == true ) {
-            nm1 = (NormalMode*)activemode;
             extlist[0] = new NMExtList();
-            extlist[0]->createExtList( nm1, msg, false );
-            extlist[0]->setTakeDirs( extorder.take_dirs );
-          }
+            extlist[0]->createExtList( activemode, msg, false );
+            extlist[0]->setTakeDirs( m_take_dirs );
         }
       } else if ( msg->mode == ActionMessage::AM_MODE_SPECIFIC_ENTRIES ) {
         if ( activemode != NULL ) {
-          if ( activemode->isType( "NormalMode" ) == true ) {
-              NormalMode *nm1 = dynamic_cast<NormalMode*>( activemode );
-              extlist[0] = new NMExtList();
-              extlist[0]->createExtList( nm1, msg, false );
-              extlist[0]->setTakeDirs( extorder.take_dirs );
-          }
+            extlist[0] = new NMExtList();
+            extlist[0]->createExtList( activemode, msg, false );
+            extlist[0]->setTakeDirs( m_take_dirs );
         }
       }
     }
@@ -373,7 +366,7 @@ int WPUContext::openWin()
   
   if ( userwin == NULL ) {
     aguix = worker->getAGUIX();
-    userwin = new AWindow( aguix, 10, 10, 100, 50, 0, catalog.getLocale( 499 ) );
+    userwin = new AWindow( aguix, 10, 10, 100, 50, 0, catalog.getLocale( 499 ), AWindow::AWINDOW_DIALOG );
     userwin->create();
     tx = ty = 5;
     nr_of_text = 1;
@@ -421,20 +414,35 @@ int WPUContext::closeWin()
   return 0;
 }
 
-NormalMode::parse_com_str_t WPUContext::parse( const char *str1, std::string &return_str, int maxlen, bool quote )
+WPUContext::parse_com_str_t WPUContext::parse( const char *str1, std::string &return_str, int maxlen, bool quote )
 {
-    NormalMode::parse_com_str_t res = NormalMode::PARSE_ERROR;
-    NormalMode *nm;
+    parse_com_str_t res = PARSE_ERROR;
   
     if ( worker != NULL ) {
         buildExtList();
-        if ( activemode != NULL ) {
-            if ( activemode->isType( "NormalMode" ) == true ) {
-                nm = (NormalMode*)activemode;
 
-                res = nm->parseComStrExt( str1, &extorder, maxlen, extlist,
-                                          quote, return_str);
-            }
+        int old_size = 0;
+        std::string fullname;
+
+        if ( extlist[0] != NULL &&
+             extlist[0]->getSize() > 0 ) {
+            int id = extlist[0]->initEnum();
+            NM_extern_fe *efe1 = (NM_extern_fe*)extlist[0]->getFirstElement( id );
+
+            old_size = extlist[0]->getSize();
+            fullname = efe1->getFullname( false );
+
+            extlist[0]->closeEnum( id );
+        }
+
+        res = parseComStr( str1, maxlen,
+                           quote, return_str);
+
+        if ( extlist[0] == NULL ||
+             extlist[0]->getSize() != old_size ) {
+            worker->storePathPersIfInProgress( fullname );
+        } else {
+            worker->storePathPersIfInProgress( NWC::Path::dirname( fullname ) );
         }
     }
     return res;
@@ -903,7 +911,7 @@ struct WPUContext::ifp_erg *WPUContext::ifp_factor()
           /*execute tstr */
           std::string res_str;
 
-          if ( parse( tstr, res_str, a_max( EXE_STRING_LEN - 1024, 256 ) ) == NormalMode::PARSE_SUCCESS ) {
+          if ( parse( tstr, res_str, a_max( EXE_STRING_LEN - 1024, 256 ) ) == PARSE_SUCCESS ) {
               _freesafe( tstr );
               tstr = dupstring( res_str.c_str() );
           } else {
@@ -913,7 +921,7 @@ struct WPUContext::ifp_erg *WPUContext::ifp_factor()
   
           if ( tstr != NULL ) {
             e = (struct ifp_erg*)_allocsafe( sizeof( struct ifp_erg ) );
-            std::auto_ptr<ExeClass> ec( new ExeClass( ExeClass::EXECUTE_NORMAL, worker ) );
+            std::unique_ptr<ExeClass> ec( new ExeClass( ExeClass::EXECUTE_NORMAL, worker ) );
             ec->cdDir( getBaseDir() );
             ec->addCommand( "%s", tstr );
             e->numval = ec->getReturnCode( &exeerror );
@@ -969,7 +977,7 @@ struct WPUContext::ifp_erg *WPUContext::ifp_factor()
         if ( ignore == 0 ) {
           std::string res_str;
 
-          if ( parse( tstr, res_str, a_max( EXE_STRING_LEN - 1024, 256 ) ) == NormalMode::PARSE_SUCCESS ) {
+          if ( parse( tstr, res_str, a_max( EXE_STRING_LEN - 1024, 256 ) ) == PARSE_SUCCESS ) {
               _freesafe( tstr );
               tstr = dupstring( res_str.c_str() );
           } else {
@@ -979,7 +987,7 @@ struct WPUContext::ifp_erg *WPUContext::ifp_factor()
   
           if ( tstr != NULL ) {
             e = (struct ifp_erg*)_allocsafe( sizeof( struct ifp_erg ) );
-            std::auto_ptr<ExeClass> ec( new ExeClass( ExeClass::EXECUTE_NORMAL, worker ) );
+            std::unique_ptr<ExeClass> ec( new ExeClass( ExeClass::EXECUTE_NORMAL, worker ) );
             ec->cdDir( getBaseDir() );
             ec->addCommand( "%s", tstr );
             e->strval = ec->getOutput( &exeerror );
@@ -1039,7 +1047,7 @@ struct WPUContext::ifp_erg *WPUContext::ifp_factor()
       } else {
           std::string res_str;
 
-          if ( parse( ifp_lexbuf, res_str, a_max( EXE_STRING_LEN - 1024, 256 ), false ) == NormalMode::PARSE_SUCCESS ) {
+          if ( parse( ifp_lexbuf, res_str, a_max( EXE_STRING_LEN - 1024, 256 ), false ) == PARSE_SUCCESS ) {
               e->strval = dupstring( res_str.c_str() );
           } else {
               e->strval = dupstring( "0" );
@@ -1124,28 +1132,20 @@ bool WPUContext::filelistEmpty( bool other )
 
 void WPUContext::buildExtList()
 {
-  NormalMode *nm;
-  
   if ( worker != NULL ) {
     if ( extlist[0] == NULL ) {
       extlist[0] = new NMExtList();
       if ( activemode != NULL ) {
-        if ( activemode->isType( "NormalMode" ) == true ) {
-          nm = (NormalMode*)activemode;
-          extlist[0]->createExtList( nm, extorder.recursive );
-        }
+          extlist[0]->createExtList( activemode, m_recursive );
       }
-      extlist[0]->setTakeDirs( extorder.take_dirs );
+      extlist[0]->setTakeDirs( m_take_dirs );
     }
     if ( extlist[1] == NULL ) {
       extlist[1] = new NMExtList();
       if ( nonactivemode != NULL ) {
-        if ( nonactivemode->isType( "NormalMode" ) == true ) {
-          nm = (NormalMode*)nonactivemode;
-          extlist[1]->createExtList( nm, extorder.recursive );
-        }
+          extlist[1]->createExtList( nonactivemode, m_recursive );
       }
-      extlist[1]->setTakeDirs( extorder.take_dirs );
+      extlist[1]->setTakeDirs( m_take_dirs );
     }
   }
 }
@@ -1167,12 +1167,12 @@ int WPUContext::getCurrentLine()
 
 void WPUContext::setRecursive( bool nv )
 {
-  extorder.recursive = nv;
+  m_recursive = nv;
 }
 
-bool WPUContext::getRecursive()
+bool WPUContext::getRecursive() const
 {
-  return extorder.recursive;
+  return m_recursive;
 }
 
 void WPUContext::setWinProgress( int nv )
@@ -1268,18 +1268,11 @@ void WPUContext::setWinText( const char *str )
 
 const char *WPUContext::getBaseDir()
 {
-  const char *tstr = NULL;
-  NormalMode *nm;
-  
-  if ( worker != NULL ) {
-    if ( activemode != NULL ) {
-      if ( activemode->isType( "NormalMode" ) == true ) {
-        nm = (NormalMode*)activemode;
-        tstr = nm->getCurrentDir();
-      }
+    if ( worker != NULL && activemode != NULL && m_base_dir.empty() ) {
+        m_base_dir = activemode->getCurrentDirectory();
     }
-  }
-  return tstr;
+
+    return m_base_dir.c_str();
 }
 
 int WPUContext::filelistSize( bool other )
@@ -1296,7 +1289,8 @@ int WPUContext::filelistRealSize( bool other )
 
 void WPUContext::setTakeDirs( bool nv )
 {
-  extorder.take_dirs = nv;
+  m_take_dirs = nv;
+
   for ( int i = 0; i < 2; i++ ) {
     if ( extlist[i] != NULL ) {
       extlist[i]->setTakeDirs( nv );
@@ -1304,19 +1298,29 @@ void WPUContext::setTakeDirs( bool nv )
   }
 }
 
-bool WPUContext::getTakeDirs()
+bool WPUContext::getTakeDirs() const
 {
-  return extorder.take_dirs;
+    return m_take_dirs;
 }
 
 void WPUContext::setUseVirtualTempCopies( bool nv )
 {
-  extorder.m_use_virtual_temp_copies = nv;
+    m_use_virtual_temp_copies = nv;
 }
 
 bool WPUContext::getUseVirtualTempCopies() const
 {
-    return extorder.m_use_virtual_temp_copies;
+    return m_use_virtual_temp_copies;
+}
+
+void WPUContext::setUseExtendedBasename( bool nv )
+{
+    m_use_extended_basename = nv;
+}
+
+bool WPUContext::getUseExtendedBasename() const
+{
+    return m_use_extended_basename;
 }
 
 std::string WPUContext::getTempName4File( const char *filename )
@@ -1442,3 +1446,625 @@ void WPUContext::calculateFilesizes()
         m_progress_callback->setCompleteFilesizes( sum );
     }
 }
+
+class NM_CLI_String
+{
+public:
+    NM_CLI_String( const size_t maxlen, const bool quote ) : m_maxlen( maxlen ),
+                                                             m_quote( quote ),
+                                                             m_error_occurred( false )
+    {}
+
+    bool add_str( const std::string str1, const bool force_noquoting )
+    {
+        return add_str( str1.c_str(), force_noquoting );
+    }
+
+    bool add_str( const char *str1, const bool force_noquoting )
+    {
+        if ( ! str1 ) return false;
+
+        if ( m_quote == true && force_noquoting == false ) {
+
+            char *newdstr = AGUIX_catTrustedAndUnTrusted( m_current_str.c_str(), str1 );
+
+            if ( ! newdstr ) {
+                m_error_occurred = true;
+                return false;
+            }
+
+            if ( strlen( newdstr ) >= m_maxlen ) {
+                _freesafe( newdstr );
+                m_error_occurred = true;
+                return false;
+            }
+
+            m_current_str = newdstr;
+            m_last_valid_str = m_current_str;
+
+            _freesafe( newdstr );
+        } else {
+            size_t tlen = strlen( str1 );
+
+            if ( m_current_str.size() + tlen >= m_maxlen ) {
+                m_error_occurred = true;
+                return false;
+            }
+
+            m_current_str += str1;
+            m_last_valid_str = m_current_str;
+        }
+
+        return true;
+    }
+
+    bool add_char( const char ch )
+    {
+        if ( m_current_str.size() + 1 >= m_maxlen ) {
+            m_error_occurred = true;
+            return false;
+        }
+
+        m_current_str.push_back( ch );
+
+        return true;
+    }
+
+    std::string get_last_valid_string() const
+    {
+        if ( ! m_error_occurred ) return m_current_str;
+
+        return m_last_valid_str;
+    }
+
+    bool is_error_occurred() const
+    {
+        return m_error_occurred;
+    }
+
+    size_t get_current_length() const
+    {
+        return m_current_str.size();
+    }
+private:
+    std::string m_current_str;
+    std::string m_last_valid_str;
+    const size_t m_maxlen;
+    const bool m_quote;
+    bool m_error_occurred;
+};
+
+WPUContext::parse_com_str_t WPUContext::parseComStr( const char *sstr,
+                                                     int maxlen, bool quote,
+                                                     std::string &return_str )
+{
+    char ch;
+    int spos = 0, bracketcount = 0, bufpos = 0;
+    int mode = 0;
+    bool ende;
+    char *tstr;
+    int id;
+    bool cancel = false;
+    int num;
+    std::string str1;
+    char *flagbuf;
+    int flagbuf_size;
+    NM_CLI_String res_cli_str( maxlen, quote );
+    const bool use_temp_copies_option = m_use_virtual_temp_copies;
+    const bool use_extended_basenames = m_use_extended_basename;
+
+    flagbuf_size = strlen( sstr );
+    flagbuf = (char*)_allocsafe( flagbuf_size + 1 );
+    for ( ende = false; ende == false; ) {
+        ch = sstr[ spos++ ];
+        if ( ch == '\0' ) break;
+        switch ( mode ) {
+            case 1:
+                // we are in a open bracket and waiting for close bracket
+                if ( ch == '{' ) {
+                    // other flag, perhaps useless, but perhaps it is in a String-requester
+                    // in any case just overtake it in the buffer
+                    if ( bufpos < flagbuf_size ) flagbuf[ bufpos++ ] = ch;
+                    bracketcount++;
+                } else if ( ch == '}' ) {
+                    // a closeing bracket
+                    if ( bracketcount > 1 ) {
+                        // this is a bracket in the bracket
+                        if ( bufpos < flagbuf_size ) flagbuf[ bufpos++ ] = ch;
+                    } else {
+                        // this is the awaited closebracket
+                        flagbuf[ bufpos ] = 0;
+                        // now flagbuf contains a flag which must be parsed
+                        mode = 3;
+                    }
+                    bracketcount--;
+                } else if ( ch == '\\' ) {
+                    // backslash are only resolved on toplevel (bracketcount==0)
+                    // so overtake also this
+                    if ( bufpos < flagbuf_size ) flagbuf[ bufpos++ ] = ch;
+                    mode = 4;
+                } else {
+                    if ( bufpos < flagbuf_size ) flagbuf[ bufpos++ ] = ch;
+                }
+                break;
+            case 2:
+                if ( ! res_cli_str.add_char( ch ) ) {
+                    ende = true;
+                }
+                mode = 0;
+                break;
+            case 4:
+                if ( bufpos < flagbuf_size ) flagbuf[ bufpos++ ] = ch;
+                mode = 1;
+                break;
+            default:
+                // we are in no bracket
+                if ( ch == '\\' ) {
+                    // next character is protected and will be overtaken
+                    mode = 2;
+                } else if ( ch != '{' ) {
+                    // a normal character
+                    if ( ! res_cli_str.add_char( ch ) ) {
+                        ende = true;
+                    }
+                } else {
+                    mode = 1;
+                    bracketcount++;
+                    bufpos = 0;
+                }
+                break;
+        }
+        if ( mode == 3 ) {
+            const char *buf1 = flagbuf;
+            bool force_noquoting = false;
+            bool force_notempfiles = false;
+
+            while ( true ) {
+                if ( buf1[0] == '-' ) {
+                    force_noquoting = true;
+                    buf1++;
+                } else if ( buf1[0] == '*' ) {
+                    force_notempfiles = true;
+                    buf1++;
+                } else {
+                    break;
+                }
+            }
+            // parse buf1
+            if ( strncmp( buf1, "Rs", 2 ) == 0 ) {
+                // Stringrequest
+                tstr = StringRequest( buf1, maxlen - res_cli_str.get_current_length() );
+                if ( tstr != NULL ) {
+                    if ( ! res_cli_str.add_str( tstr, force_noquoting ) ) {
+                        ende = true;
+                    }
+                    _freesafe( tstr );
+                } else {
+                    cancel = true;
+                    ende = true;
+                }
+            } else if ( strncmp( buf1, "scripts", 7 ) == 0 ) {
+                // dir of the scripts
+                str1 = Worker::getDataDir();
+                str1 += "/scripts";
+                if ( ! res_cli_str.add_str( str1, force_noquoting ) ) {
+                    ende = true;
+                }
+            } else if ( strncmp( buf1, "pop", 3 ) == 0 ) {
+                if ( strlen( buf1 ) > 3 ) {
+                    num = atoi( buf1 + 3 );
+                    if ( num < 0 ) num = 0;
+                } else {
+                    num = 0;
+                }
+                if ( size( num ) > 0 ) {
+                    str1 = pop( num );
+                    if ( ! res_cli_str.add_str( str1, force_noquoting ) ) {
+                        ende = true;
+                    }
+                }
+            } else if ( strncmp( buf1, "top", 3 ) == 0 ) {
+                if ( strlen( buf1 ) > 3 ) {
+                    num = atoi( buf1 + 3 );
+                    if ( num < 0 ) num = 0;
+                } else {
+                    num = 0;
+                }
+                if ( size( num ) > 0 ) {
+                    str1 = top( num );
+                    if ( ! res_cli_str.add_str( str1, force_noquoting ) ) {
+                        ende = true;
+                    }
+                }
+            } else if ( strncmp( buf1, "size", 4 ) == 0 ) {
+                if ( strlen( buf1 ) > 4 ) {
+                    num = atoi( buf1 + 4 );
+                    if ( num < 0 ) num = 0;
+                } else {
+                    num = 0;
+                }
+
+                char buf2[ A_BYTESFORNUMBER( int ) + 2 ];
+
+                sprintf( buf2, "%d", size( num ) );
+                str1 = buf2;
+                if ( ! res_cli_str.add_str( str1, force_noquoting ) ) {
+                    ende = true;
+                }
+            } else if ( strncmp( buf1, "p", 1 ) == 0 ) {
+                // path
+                if ( activemode ) {
+                    std::string cwd = activemode->getCurrentDirectory();
+
+                    if ( ! cwd.empty() ) {
+                        if ( ! res_cli_str.add_str( cwd, force_noquoting ) ) {
+                            ende = true;
+                        }
+                    }
+                }
+            } else if ( strncmp( buf1, "op", 2 ) == 0 ) {
+                // other side
+                if ( nonactivemode ) {
+                    std::string cwd = nonactivemode->getCurrentDirectory();
+
+                    if ( ! cwd.empty() ) {
+                        if ( ! res_cli_str.add_str( cwd, force_noquoting ) ) {
+                            ende = true;
+                        }
+                    }
+                }
+            } else if ( strncmp( buf1, "lp", 2 ) == 0 ) {
+                // left side
+
+                ListerMode *lm;
+
+                lm = worker->getLister( 0 )->getActiveMode();
+
+                if ( lm ) {
+                    std::string cwd = lm->getCurrentDirectory();
+
+                    if ( ! cwd.empty() ) {
+                        if ( ! res_cli_str.add_str( cwd, force_noquoting ) ) {
+                            ende = true;
+                        }
+                    }
+                }
+            } else if ( strncmp( buf1, "rp", 2 ) == 0 ) {
+                // right side
+
+                ListerMode *lm;
+
+                lm = worker->getLister( 1 )->getActiveMode();
+
+                if ( lm ) {
+                    std::string cwd = lm->getCurrentDirectory();
+
+                    if ( ! cwd.empty() ) {
+                        if ( ! res_cli_str.add_str( cwd, force_noquoting ) ) {
+                            ende = true;
+                        }
+                    }
+                }
+            } else if ( strncmp( buf1, "dp", 2 ) == 0 ) {
+                // destination path
+                if ( destmode ) {
+                    std::string cwd = destmode->getCurrentDirectory();
+
+                    if ( ! cwd.empty() ) {
+                        if ( ! res_cli_str.add_str( cwd, force_noquoting ) ) {
+                            ende = true;
+                        }
+                    }
+                }
+            } else if ( strncmp( buf1, "v", 1 ) == 0 ) {
+                // env variable
+                tstr = getenv( buf1 + 1 );
+                if ( tstr != NULL ) {
+                    if ( ! res_cli_str.add_str( tstr, force_noquoting ) ) {
+                        ende = true;
+                    }
+                }
+            } else {
+                // file flag or unknown
+                bool otherside, nounselect, noext;
+                int filemode, tpos, tmode;
+
+                filemode = -1;
+                otherside = false;
+                nounselect = false;
+                noext = false;
+                tpos = 0;
+                tmode = 0;
+                for( ; tmode < 4 && tpos < 4; ) {
+                    ch = buf1[ tpos++ ];
+                    if ( ch == 'o' ) {
+                        if ( tmode < 1 ) {
+                            otherside = true;
+                            tmode = 1;
+                        } else {
+                            // an "o" at the wrong position
+                            tmode = 6;
+                        }
+                    } else if ( ch == 'u' ) {
+                        if ( tmode < 2 ) {
+                            nounselect = true;
+                            tmode = 2;
+                        } else {
+                            tmode = 6;
+                        }
+                    } else if ( ch == 'f' && tmode < 3 ) {
+                        filemode = 0;
+                        tmode = 3;
+                    } else if ( ch == 'a' && tmode < 3 ) {
+                        filemode = 1;
+                        tmode = 3;
+                    } else if ( ch == 'F' && tmode < 3 ) {
+                        filemode = 2;
+                        tmode = 3;
+                    } else if ( ch == 'A' && tmode < 3 ) {
+                        filemode = 3;
+                        tmode = 3;
+                    } else if ( ch == 'E' && tmode == 3 ) {
+                        noext = true;
+                        tmode = 4;
+                    } else if ( ch == 't' && tmode < 3 ) {
+                        filemode = 4;
+                        tmode = 4;  // no E allowed so directly change to mode 4
+                    } else if ( ch == '\0' ) tmode = 4;
+                }
+                if (  filemode >= 0 && filemode < 5 ) {
+                    // we have a valid flag
+                    bool takefirst = true;
+                    NM_extern_fe *efe1;
+                    NMExtList *l;
+                    bool efeskip;
+
+                    if ( filemode == 1 || filemode == 3 ) takefirst = false;
+                    if ( otherside == true ) l = extlist[1];
+                    else l = extlist[0];
+                    id = l->initEnum();
+                    efe1 = (NM_extern_fe*)l->getFirstElement( id );
+                    for ( int c = 0;; ) {
+                        if ( efe1 == NULL ) break;
+              
+                        efeskip = false;
+                        if ( ( efe1->getDirFinished() != 0 ) &&
+                             ( m_take_dirs == false ) ) {
+                            efeskip = true;
+                        } else {
+                            std::string name_string;
+                            const char *name_cstring = NULL;
+
+                            // this is a valid entry
+                            if ( use_temp_copies_option == true &&
+                                 force_notempfiles == false &&
+                                 worker_islocal( efe1->getFullname( false ) ) == 0 ) {
+
+                                name_string = getTempName4File( efe1->getFullname( false ) );
+                                if ( ! name_string.empty() ) {
+                                    name_cstring = name_string.c_str();
+                                }
+                            } else {
+                                if ( filemode == 4 ) {
+                                    name_string = getTempName4File( efe1->getFullname( false ) );
+                                    if ( ! name_string.empty() ) {
+                                        name_cstring = name_string.c_str();
+                                    }
+                                } else if ( filemode < 2 ) {
+                                    std::string cwd;
+
+                                    if ( use_extended_basenames ) {
+                                        // do it only for virtualdirmode at the moment
+                                        // even though it works for normalmode as well
+                                        if ( ! otherside && activemode ) {
+                                            if ( dynamic_cast< VirtualDirMode *>( activemode ) ) { 
+                                                cwd = activemode->getCurrentDirectory();
+                                            }
+                                        } else if ( otherside && nonactivemode ) {
+                                            if ( dynamic_cast< VirtualDirMode *>( nonactivemode ) ) { 
+                                                cwd = nonactivemode->getCurrentDirectory();
+                                            }
+                                        }
+                                    }
+
+                                    name_cstring = efe1->getName( noext );
+
+                                    if ( ! cwd.empty() ) {
+                                        name_string = NWC::Path::get_extended_basename( cwd,
+                                                                                        efe1->getFullname( noext ) );
+                                        if ( ! name_string.empty() ) {
+                                            name_cstring = name_string.c_str();
+                                        }
+                                    }
+                                } else {
+                                    name_cstring = efe1->getFullname( noext );
+                                }
+                            }
+
+                            if ( name_cstring != NULL ) {
+                                if ( c > 0 ) {
+                                    if ( ! res_cli_str.add_char( ' ' ) ) {
+                                        ende = true;
+                                    }
+                                }
+                                if ( ende == false && ! res_cli_str.add_str( name_cstring, force_noquoting ) ) {
+                                    ende = true;
+                                }
+                            }               
+                            if ( ende == true ) break;
+                        }
+                        for (;;) {
+                            if ( nounselect == false ) {
+                                // remove efe1;
+                                l->removeElement( efe1 );
+                                // now deactivate LVC if not NULL
+                                if ( efe1->getFE() != NULL ) {
+                                    if ( otherside == false ) {
+                                        // in my list
+                                        activemode->deselectEntry( efe1->getFullname( false ) );
+                                    } else {
+                                        if ( nonactivemode ) {
+                                            nonactivemode->deselectEntry( efe1->getFullname( false ) );
+                                        }
+                                    }
+                                }
+                                delete efe1;
+                                efe1 = (NM_extern_fe*)l->getFirstElement( id );
+                            } else {
+                                efe1 = (NM_extern_fe*)l->getNextElement( id );
+                            }
+                            if ( efe1 == NULL ) break;
+                            if ( ( efe1->getDirFinished() == 0 ) ||
+                                 ( m_take_dirs == true ) ) {
+                                break;
+                            }
+                        }
+
+                        if ( efeskip == true ) continue;
+                        if ( c == 0 && takefirst == true ) break;
+                        c = 1;
+                    }
+                    l->closeEnum( id );
+                }
+            }
+            mode = 0;
+        }
+    }
+    _freesafe( flagbuf );
+
+    if ( cancel == true ) {
+        return PARSE_ERROR;
+    }
+
+    if ( res_cli_str.is_error_occurred() ) {
+        RefCount<AFontWidth> lencalc( new AFontWidth( Worker::getAGUIX(), NULL ) );
+
+        std::string message = catalog.getLocale( 938 );
+        message += sstr;
+
+        TextStorageString ts( message, lencalc );
+
+        std::string buttons = catalog.getLocale( 629 );
+        buttons += "|";
+        buttons += catalog.getLocale( 8 );
+
+        int erg = worker->getRequester()->request( catalog.getLocale( 347 ), ts, buttons.c_str() );
+
+        if ( erg == 1 ) {
+            return PARSE_ERROR;
+        }
+    }
+
+    return_str = res_cli_str.get_last_valid_string();
+    return PARSE_SUCCESS;
+}
+
+char *WPUContext::StringRequest( const char *sstr, int maxlen )
+{
+    char *istr=NULL,ch;
+    char *defstr=NULL;
+    char *ristr=NULL,*rdefstr=NULL;
+    int spos=0,bracketcount=0;
+    int istart,iend,defstart,defend;
+    int mode;
+    char *buttonstr,*textstr,*return_str;
+    int erg;
+  
+    spos=2; // buf1 starts with Rs
+    istart=iend=defstart=defend=-1;
+    mode=0;
+    for(;mode<4;) {
+        ch=sstr[spos++];
+        if(ch=='\0') break;
+        switch(mode) {
+            case 0:
+                // we are in no bracket
+                if(ch=='{') {
+                    mode=1;
+                    bracketcount++;
+                    istart=spos;
+                }
+                break;
+            case 1:
+                // in the infotext
+                if(ch=='{') bracketcount++;
+                else if(ch=='}') {
+                    if(bracketcount==1) {
+                        iend=spos-2;
+                        mode=2;
+                    }
+                    bracketcount--;
+                }
+                break;
+            case 2:
+                // we are in no bracket
+                if(ch=='{') {
+                    mode=3;
+                    bracketcount++;
+                    defstart=spos;
+                }
+                break;
+            case 3:
+                // in the defaulttext
+                if(ch=='{') bracketcount++;
+                else if(ch=='}') {
+                    if(bracketcount==1) {
+                        defend=spos-2;
+                        mode=4;
+                    }
+                    bracketcount--;
+                }
+                break;
+            default:
+                break;
+        }
+    }
+    if((istart>=0)&&(iend>=istart)) {
+        // we have an infotext
+        istr=dupstring(&sstr[istart]);
+        istr[iend-istart+1]=0;
+
+        std::string res_str;
+
+        if ( parseComStr( istr, maxlen, false, res_str ) == PARSE_SUCCESS ) {
+            ristr = dupstring( res_str.c_str() );
+        }
+        _freesafe(istr);
+    }
+    if((defstart>=0)&&(defend>=istart)) {
+        // we have a defaulttext
+        defstr=dupstring(&sstr[defstart]);
+        defstr[defend-defstart+1]=0;
+
+        std::string res_str;
+
+        if ( parseComStr( defstr, maxlen, false, res_str ) == PARSE_SUCCESS ) {
+            rdefstr = dupstring( res_str.c_str() );
+        }
+        _freesafe(defstr);
+    }
+  
+    textstr=NULL;
+    if(ristr!=NULL) {
+        if(strlen(ristr)>0) textstr=ristr;
+        else _freesafe(ristr);
+    }
+    if(textstr==NULL) textstr=dupstring(catalog.getLocale(200));
+    buttonstr=(char*)_allocsafe(strlen(catalog.getLocale(11))+1+
+                                strlen(catalog.getLocale(8))+1);
+    sprintf(buttonstr,"%s|%s",catalog.getLocale(11),
+            catalog.getLocale(8));
+    erg = worker->getRequester()->string_request( catalog.getLocale( 123 ),
+                                                  textstr,
+                                                  ( rdefstr != NULL ) ? rdefstr : "",
+                                                  buttonstr,
+                                                  &return_str );
+    _freesafe(buttonstr);
+    _freesafe(textstr);
+    if(rdefstr!=NULL) _freesafe(rdefstr);
+    if(erg!=0) {
+        _freesafe(return_str);
+        return NULL;
+    }
+    return return_str;
+}
